searchData={"items":[{"type":"module","title":"Beacon","doc":"Beacon is a Content Management System for [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view).\n\nKey features include:\n\n* Rendering pages fast\n* Reloading content at runtime\n* Improved resource usage and scalability\n* Integration with existing Phoenix applications\n\nYou can build virtually any type of website with Beacon, from a simple blog to a complex business site.\n\nThe following are the main APIs provided by Beacon. You can find more information in the documentation for each of these modules:\n\n* `Beacon.Config` - configure your site(s)\n* `Beacon.Router` - mount site(s) into the router of your Phoenix application\n* `Beacon.Lifecycle` - inject custom logic into Beacon's lifecycle to change how pages are loaded, rendered, and more\n* `Beacon.Content` - manage content such as layouts, pages, page variants, snippets, etc.\n* `Beacon.MediaLibrary` - upload images, videos, and documents that can be used in your content\n* `Beacon.Test` - utilities for testing\n\nGet started with [your first site](https://hexdocs.pm/beacon/your-first-site.html) and check out the guides for more information.","ref":"Beacon.html"},{"type":"function","title":"Beacon.boot/1","doc":"Boot a site.\n\nIt will restart the Site Supervisor if it's already running, otherwise it will start it in the main Beacon Supervisor.\n\nThis function is not necessary to be called in most cases, as Beacon will automatically boot all sites when it starts,\nbut in some cases where a site is started with the `:manual` mode, you may want to call this function to boot the site\nin the `:live` mode to activate resource loading and PubSub events broadcasting.\n\nNote that `:live` sites that are not reachable will not be started,\nsee [deployment topologies](https://hexdocs.pm/beacon/deployment-topologies.html) for more info.","ref":"Beacon.html#boot/1"},{"type":"function","title":"Beacon.start_link/1","doc":"Start `Beacon` and a supervisor for each site, which will load all layouts, pages, components, and so on.\n\nYou must include the `Beacon` supervisor on each application that you want it loaded. For a single Phoenix application\nthat would go in the `children` list on the file `lib/my_app/application.ex`. For Umbrella apps you can have\nmultiple apps running Beacon, suppose your project has 3 apps: core (regular app), blog (phoenix app), and marketing (phoenix app)\nand you want to load one Beacon instance on each Phoenix app, so you would include `Beacon` in the list of `children` applications\nin both blog and marketing applications with their own `:sites` configuration.\n\nNote that each Beacon instance may have multiple sites and each site loads in its own supervisor. That gives you the\nflexibility to plan your architecture from simple to complex environments. For example, you can have a single site\nserving all pages in a single Phoenix application or you can create a new site to isolate a landing page for a marketing\ncampaign that may receive too much traffic.\n\nSee `Beacon.Router` and [Deployment Topologies](https://hexdocs.pm/beacon/deployment-topologies.html) for more information.","ref":"Beacon.html#start_link/1"},{"type":"function","title":"Options - Beacon.start_link/1","doc":"Each site in `:sites` may have its own configuration, see all available options at `Beacon.Config.new/1`.","ref":"Beacon.html#start_link/1-options"},{"type":"function","title":"Examples - Beacon.start_link/1","doc":"# runtime.exs\n    config :beacon,\n      my_site: [site: :my_site, repo: MyApp.Repo, endpoint: MyAppWeb.Endpoint, router: MyAppWeb.Router]\n\n    # lib/my_app/application.ex\n    def start(_type, _args) do\n      children = [\n        MyApp.Repo,\n        {Phoenix.PubSub, name: MyApp.PubSub},\n        {Beacon, [\n          sites: [\n            Application.fetch_env!(:beacon, :my_site)\n          ]\n        ]},\n        MyAppWeb.Endpoint\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","ref":"Beacon.html#start_link/1-examples"},{"type":"module","title":"Beacon.Plug","doc":"Used to ensure consistency for Beacon Page rendering.\n\nThis is especially important when using Page Variants.","ref":"Beacon.Plug.html"},{"type":"module","title":"Usage - Beacon.Plug","doc":"Add the plug to your Router's `:browser` pipeline:\n\n```\npipeline :browser do\n  ...\n  plug Beacon.Plug\nend\n```","ref":"Beacon.Plug.html#module-usage"},{"type":"module","title":"Beacon.Test","doc":"Testing utilities to create and assert Beacon resources.","ref":"Beacon.Test.html"},{"type":"module","title":"Usage - Beacon.Test","doc":"First you need to activate the `:testing` mode in your site configuration:\n\n```elixir\n# test.exs\n# active testing mode for all sites under test\nconfig :my_app, :my_site, mode: :testing\n```\n\nSee `Beacon.start_link/1` for more info on how to setup your Beacon configuration.\n\nThen use this module either in your test module or in your test case template:\n\n```elixir\ndefmodule MyAppWeb.CMSTest do\n  use MyAppWeb.ConnCase\n  use Beacon.Test\n  # ...\nend\n```\n\nor make it available for all your tests by adding it to your test case template:\n\n```elixir\ndefmodule MyAppWeb.ConnCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use Beacon.Test\n      # ...\n    end\n  end\nend\n```\n\nWith this configuration, Beacon will behave in a way that is better suited for testing:\n\n- Do not hot-load resources during boot\n- Do not broadcast events on Content changes\n- Perform most operations in a synchronous way\n- Reload module as fixture data is created\n\nAnd all functions in `Beacon.Test.Fixtures` will be imported to help you create resources in your tests.","ref":"Beacon.Test.html#module-usage"},{"type":"module","title":"Default site - Beacon.Test","doc":"Most of the functions need a `site` option to know which site to operate on.\nIf you don't provide it, the default site `:my_site` is used:\n\n```elixir\ncreate_page_fixture(title: \"Home\")\n%Beacon.Content.Page{site: :my_site, title: \"Home\"}\n```\n\nOr you can override it:\n\n```elixir\ncreate_page_fixture(site: :blog, title: \"Home\")\n%Beacon.Content.Page{site: :blog, title: \"Home\"}\n```\n\nBut doing so every time is not efficient, so you can set a default site that will be used in all function calls:\n\n```elixir\nuse Beacon.Test, site: :blog\n\ncreate_page_fixture(title: \"Home\")\n%Beacon.Content.Page{site: :blog, title: \"Home\"}\n```","ref":"Beacon.Test.html#module-default-site"},{"type":"function","title":"Beacon.Test.default_site/0","doc":"Returns the site defined in the current test.\n\nDefault is `:my_site` if the option `:site` was not set in `use Beacon.Test`","ref":"Beacon.Test.html#default_site/0"},{"type":"module","title":"Beacon.Test.Fixtures","doc":"Fixture data for testing Beacon content.\n\n> #### Only use for testing {: .warning}\n>\n> These fixtures should be used only for testing purposes,\n> if you are looking to run seeds or some sort of content automation\n> then you should use `Beacon.Content` functions instead.","ref":"Beacon.Test.Fixtures.html"},{"type":"module","title":"Usage - Beacon.Test.Fixtures","doc":"Most of the times you'll use the `Beacon.Test` function instead of using fixtures directly:\n\n```elixir\nuse Beacon.Test\n```\n\n_Using `Beacon.Test` will import the fixtures for you._\n\nBut you can also use the fixtures directly for some cases:\n\n```elixir\nuse Beacon.Test.Fixtures\n```","ref":"Beacon.Test.Fixtures.html#module-usage"},{"type":"module","title":"Fixtures - Beacon.Test.Fixtures","doc":"All fixtures accept either a map or a keyword list, so these are equivalent:\n\n```elixir\nbeacon_page_fixture(path: \"/contact\")\nbeacon_page_fixture(%{path: \"/contact\"})\nbeacon_page_fixture(%{\"path\" => \"/contact\"})\n```\n\nOr no attributes at all to use the default values:\n\n```elixir\nbeacon_page_fixture()\n```","ref":"Beacon.Test.Fixtures.html#module-fixtures"},{"type":"module","title":"Default site - Beacon.Test.Fixtures","doc":"You can pass a default site to be used in the attrs for all fixture functions:\n\n```elixir\nuse Beacon.Test.Fixtures, site: :blog\n```\n\nNote that only one default site is permitted per test module,\nif you have a test that requires asserting multiple sites\nyou can just override particular fixtures:\n\n```elixir\nuse Beacon.Test.Fixtures, site: :blog\n\n# create a page for the default site\nbeacon_page_fixture()\n\n# create a page for another site\nbeacon_page_fixture(site: :other)\n\n```","ref":"Beacon.Test.Fixtures.html#module-default-site"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_component_fixture/1","doc":"Creates a `Beacon.Content.Component`.","ref":"Beacon.Test.Fixtures.html#beacon_component_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_component_fixture/1","doc":"iex> beacon_component_fixture(name: \"sample_component\")\n    %Beacon.Content.Component{}","ref":"Beacon.Test.Fixtures.html#beacon_component_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_error_page_fixture/1","doc":"Creates a `Beacon.Content.ErrorPage`.","ref":"Beacon.Test.Fixtures.html#beacon_error_page_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_error_page_fixture/1","doc":"iex> beacon_error_page_fixture(status: 404, template: \"nothing here\")\n    %Beacon.Content.ErrorPage{}","ref":"Beacon.Test.Fixtures.html#beacon_error_page_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_event_handler_fixture/1","doc":"Creates a `Beacon.Content.EventHandler`.","ref":"Beacon.Test.Fixtures.html#beacon_event_handler_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_event_handler_fixture/1","doc":"iex> beacon_event_handler_fixture(code: ~S\"\"\"\n      email = event_params[\"newsletter\"][\"email\"]\n      MyApp.Newsletter.subscribe(email)\n      {:noreply, socket}\n    \"\"\")\n    %Beacon.Content.EventHandler{}","ref":"Beacon.Test.Fixtures.html#beacon_event_handler_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_info_handler_fixture/1","doc":"Creates a `Beacon.Content.InfoHandler`.","ref":"Beacon.Test.Fixtures.html#beacon_info_handler_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_info_handler_fixture/1","doc":"iex> beacon_info_handler_fixture(msg: \"{:subscribed, email}\", code: ~S\"\"\"\n    MyApp.Notifications.send_email(email, \"Welcome!\")\n    {:noreply, socket}\n    \"\"\")\n    %Beacon.Content.InfoHandler{}","ref":"Beacon.Test.Fixtures.html#beacon_info_handler_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_js_hook_fixture/1","doc":"Creates a `Beacon.Content.JSHook`.","ref":"Beacon.Test.Fixtures.html#beacon_js_hook_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_js_hook_fixture/1","doc":"iex> code = \"export const MyHook = {mounted() {console.log(\"foo\")}}\"\n    iex> beacon_js_hook_fixture(site: :my_site, name: \"MyHook\", code: code)\n    %Beacon.Content.JSHook{}","ref":"Beacon.Test.Fixtures.html#beacon_js_hook_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_layout_fixture/1","doc":"Creates a draft `Beacon.Content.Layout`.","ref":"Beacon.Test.Fixtures.html#beacon_layout_fixture/1"},{"type":"function","title":"Examples - Beacon.Test.Fixtures.beacon_layout_fixture/1","doc":"iex> beacon_layout_fixture()\n    %Beacon.Content.Layout{}\n\n    iex> beacon_layout_fixture(template: \"<%= @inner_content %>\")\n    %Beacon.Content.Layout{}","ref":"Beacon.Test.Fixtures.html#beacon_layout_fixture/1-examples"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_live_data_assign_fixture/1","doc":"Creates a `Beacon.Content.LiveDataAssign`.","ref":"Beacon.Test.Fixtures.html#beacon_live_data_assign_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_live_data_assign_fixture/1","doc":"iex> beacon_live_data_assign_fixture(live_data: live_data, key: \"user\", value: \"%{id: 1, name: \"John\"}\")\n    %Beacon.Content.LiveDataAssign{}","ref":"Beacon.Test.Fixtures.html#beacon_live_data_assign_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_live_data_fixture/1","doc":"Creates a `Beacon.Content.LiveData`.","ref":"Beacon.Test.Fixtures.html#beacon_live_data_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_live_data_fixture/1","doc":"iex> beacon_live_data_fixture(path: \"/contact\")\n    %Beacon.Content.LiveData{}","ref":"Beacon.Test.Fixtures.html#beacon_live_data_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_media_library_asset_fixture/1","doc":"Uploads a given \"upload metadata\" created by `beacon_upload_metadata_fixture/1`.","ref":"Beacon.Test.Fixtures.html#beacon_media_library_asset_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_media_library_asset_fixture/1","doc":"iex> beacon_media_library_asset_fixture(upload_metadata)\n    %Beacon.MediaLibrary.Asset{}","ref":"Beacon.Test.Fixtures.html#beacon_media_library_asset_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_page_fixture/1","doc":"Creates a draft `Beacon.Content.Page`","ref":"Beacon.Test.Fixtures.html#beacon_page_fixture/1"},{"type":"function","title":"Examples - Beacon.Test.Fixtures.beacon_page_fixture/1","doc":"iex> beacon_page_fixture()\n    %Beacon.Content.Page{}\n\n    iex> beacon_page_fixture(title: \"Home\")\n    %Beacon.Content.Page{}","ref":"Beacon.Test.Fixtures.html#beacon_page_fixture/1-examples"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_page_variant_fixture/1","doc":"Creates a `Beacon.Content.PageVariant`.","ref":"Beacon.Test.Fixtures.html#beacon_page_variant_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_page_variant_fixture/1","doc":"iex> beacon_page_variant_fixture(page: page, weight: 50, template: \" Variant B \")\n    %Beacon.Content.PageVariant{}","ref":"Beacon.Test.Fixtures.html#beacon_page_variant_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_published_layout_fixture/1","doc":"Similar to `beacon_layout_fixture/1`, but also publishes the layout.","ref":"Beacon.Test.Fixtures.html#beacon_published_layout_fixture/1"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_published_page_fixture/1","doc":"Similar to `beacon_page_fixture/1`, but also publishes the page.","ref":"Beacon.Test.Fixtures.html#beacon_published_page_fixture/1"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_snippet_helper_fixture/1","doc":"Creates a `Beacon.Content.Snippets.Helper`.","ref":"Beacon.Test.Fixtures.html#beacon_snippet_helper_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_snippet_helper_fixture/1","doc":"iex> beacon_snippet_helper_fixture(body: ~S\"\"\"\n      assigns |> get_in([\"page\", \"title\"]) |> String.trim()\n    \"\"\")\n    %Beacon.Content.Snippets.Helper{}","ref":"Beacon.Test.Fixtures.html#beacon_snippet_helper_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_stylesheet_fixture/1","doc":"Creates a `Beacon.Content.Stylesheet`.","ref":"Beacon.Test.Fixtures.html#beacon_stylesheet_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_stylesheet_fixture/1","doc":"iex> beacon_stylesheet_fixture(content: \"h1 { color: red; }\")\n    %Beacon.Content.Stylesheet{}","ref":"Beacon.Test.Fixtures.html#beacon_stylesheet_fixture/1-example"},{"type":"function","title":"Beacon.Test.Fixtures.beacon_upload_metadata_fixture/1","doc":"Creates a `Beacon.MediaLibrary.UploadMetadata`.","ref":"Beacon.Test.Fixtures.html#beacon_upload_metadata_fixture/1"},{"type":"function","title":"Example - Beacon.Test.Fixtures.beacon_upload_metadata_fixture/1","doc":"iex> beacon_upload_metadata_fixture(file_size: 100_000)\n    %Beacon.MediaLibrary.UploadMetadata{}","ref":"Beacon.Test.Fixtures.html#beacon_upload_metadata_fixture/1-example"},{"type":"task","title":"mix beacon.gen.proxy_endpoint","doc":"Generates a ProxyEndpoint in the current project, enabling Beacon to serve sites at multiple hosts.","ref":"Mix.Tasks.Beacon.Gen.ProxyEndpoint.html"},{"type":"task","title":"Example - mix beacon.gen.proxy_endpoint","doc":"```bash\nmix beacon.gen.proxy_endpoint\n```","ref":"Mix.Tasks.Beacon.Gen.ProxyEndpoint.html#module-example"},{"type":"task","title":"Options - mix beacon.gen.proxy_endpoint","doc":"* `--secret-key-base` (optional) - The value to use for secret_key_base in your app config.\n  By default, Beacon will generate a new value and update all existing config to match that value.\n  If you don't want this behavior, copy the secret_key_base from your app config and provide it here.\n* `--signing-salt` (optional) - The value to use for signing_salt in your app config.\n  By default, Beacon will generate a new value and update all existing config to match that value.\n  But in order to avoid connection errors for existing clients, it's recommened to copy the `signing_salt` from your app config and provide it here.\n* `--session-key` (optional) - The value to use for key in the session config. Defaults to `\"_your_app_name_key\"`\n* `--session-same-site` (optional) - Set the cookie session SameSite attributes. Defaults to `\"Lax\"`","ref":"Mix.Tasks.Beacon.Gen.ProxyEndpoint.html#module-options"},{"type":"task","title":"mix beacon.gen.site","doc":"Generates a new Beacon site in the current project.\n\nRemember to execute [`mix beacon.install`](https://hexdocs.pm/beacon/Mix.Tasks.Beacon.Install.html)\nfirst if this is the first site you're generating in your project and you have not installed Beacon yet.","ref":"Mix.Tasks.Beacon.Gen.Site.html"},{"type":"task","title":"Examples - mix beacon.gen.site","doc":"```bash\nmix beacon.gen.site --site my_site\n```\n\n```bash\nmix beacon.gen.site --site my_site --path / --host mysite.com\n```\n\nTo define a custom host to work locally and the production host:\n\n```bash\nmix beacon.gen.site --site my_site --path / --host-dev local.mysite.com --host mysite.com\n```","ref":"Mix.Tasks.Beacon.Gen.Site.html#module-examples"},{"type":"task","title":"Using --host-dev for Multiple Sites - mix beacon.gen.site","doc":"The `--host-dev` option is particularly useful when you need to run multiple sites at the root path (`/`). Without custom hosts, all sites would try to serve at `localhost:4000/`, which would cause conflicts.\n\nFor example, if you have two sites:\n\n```bash\nmix beacon.gen.site --site blog --path / --host-dev local.blog.mysite.com --host blog.mysite.com\nmix beacon.gen.site --site shop --path / --host-dev local.shop.mysite.com --host shop.mysite.com\n```\n\nTo make this work locally, you have two options:\n\n1. Edit your `/etc/hosts` file (or equivalent) to add:\n   ```\n   127.0.0.1 local.blog.mysite.com\n   127.0.0.1 local.shop.mysite.com\n   ```\n\n   This is the simplest solution and works well for local development. Locally in the dev environment, sites would be accessible at:\n   - `http://local.blog.mysite.com:4000/`\n   - `http://local.shop.mysite.com:4000/`\n\n2. Use a local development tunneling service. Some options:\n   - [ngrok](https://ngrok.com/)\n   - [localtunnel](https://localtunnel.github.io/www/)\n   - [cloudflare tunnel](https://developers.cloudflare.com/cloudflare-one/connections/connect-apps/install-and-setup/tunnel-guide/local/)","ref":"Mix.Tasks.Beacon.Gen.Site.html#module-using-host-dev-for-multiple-sites"},{"type":"task","title":"Options - mix beacon.gen.site","doc":"* `--site` (required) - The name of your site. Should not contain special characters nor start with \"beacon_\"\n* `--path` (optional) - Where your site will be mounted. Follows the same convention as Phoenix route prefixes. Defaults to `\"/\"`\n* `--host` (optional) - If provided, site will be served on that host for production environments.\n* `--host-dev` (optional) - If provided, site will be served on that host for dev (local) environments.\n* `--port` (optional) - The port to use for http requests. Only needed when `--host` is provided.  If no port is given, one will be chosen at random.\n* `--secure-port` (optional) - The port to use for https requests. Only needed when `--host` is provided.  If no port is given, one will be chosen at random.\n* `--endpoint` (optional) - The name of the Endpoint Module for your site. If not provided, a default will be generated based on the `site`.\n   For example, `beacon.gen.site --site my_site` will use `MySiteEndpoint`\n* `--secret-key-base` (optional) - The value to use for secret_key_base in your app config.\n   By default, Beacon will generate a new value and update all existing config to match that value.\n   If you don't want this behavior, copy the secret_key_base from your app config and provide it here.\n* `--signing-salt` (optional) - The value to use for signing_salt in your app config.\n   By default, Beacon will generate a new value and update all existing config to match that value.\n   But in order to avoid connection errors for existing clients, it's recommened to copy the `signing_salt` from your app config and provide it here.\n* `--session-key` (optional) - The value to use for key in the session config. Defaults to `\"_your_app_name_key\"`\n* `--session-same-site` (optional) - Set the cookie session SameSite attributes. Defaults to \"Lax\"","ref":"Mix.Tasks.Beacon.Gen.Site.html#module-options"},{"type":"task","title":"mix beacon.gen.tailwind_config","doc":"Generates a new Tailwind config in the format expected by Beacon\n\nIt will also update your Phoenix project configuration to bundle the Tailwind configuration.\n\nSee https://hexdocs.pm/beacon/tailwind-setup.html for more info.","ref":"Mix.Tasks.Beacon.Gen.TailwindConfig.html"},{"type":"task","title":"Example - mix beacon.gen.tailwind_config","doc":"```bash\nmix beacon.gen.tailwind_config\n```","ref":"Mix.Tasks.Beacon.Gen.TailwindConfig.html#module-example"},{"type":"task","title":"mix beacon.install","doc":"Installs Beacon in a Phoenix LiveView app.\n\nIt will add the necessary dependencies and configuration into your Phoenix LiveView app.\n\nThe options `--site` and `--path` are optional but you can bootstrap a new site by providing them,\notherwise execute [`mix beacon.gen.site`](https://hexdocs.pm/beacon/Mix.Tasks.Beacon.Gen.Site.html) at anytime to generate new sites.\n\nYou might want to install [Beacon LiveAdmin](https://hexdocs.pm/beacon_live_admin/Mix.Tasks.Beacon.LiveAdmin.Install.html)\nas well to manage the content of your sites.","ref":"Mix.Tasks.Beacon.Install.html"},{"type":"task","title":"Examples - mix beacon.install","doc":"```bash\nmix beacon.install\n```\n\n```bash\nmix beacon.install --site my_site --path /\n```","ref":"Mix.Tasks.Beacon.Install.html#module-examples"},{"type":"task","title":"Options - mix beacon.install","doc":"* `--site` (optional) - The name of your site. Should not contain special characters nor start with `\"beacon_\"`.\n* `--path` (optional, defaults to `\"/\"`) - Where your site will be mounted. Follows the same convention as Phoenix route prefixes.","ref":"Mix.Tasks.Beacon.Install.html#module-options"},{"type":"module","title":"Beacon.Migration","doc":"Functions which can be called in an Ecto migration for Beacon installation and upgrades.","ref":"Beacon.Migration.html"},{"type":"module","title":"Usage - Beacon.Migration","doc":"To install Beacon, you'll need to generate an `Ecto.Migration` that wraps calls to `Beacon.Migration`:\n\n```\n$ mix ecto.gen.migration create_beacon_tables\n```\n\nOpen the generated migration in your editor and either call or delegate to `up/1` and `down/1`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.CreateBeaconTables do\n  use Ecto.Migration\n  def up, do: Beacon.Migration.up()\n  def down, do: Beacon.Migration.down()\nend\n```\n\nThen, run the migrations for your app to create the necessary Beacon tables in your database:\n\n```\n$ mix ecto.migrate\n```\n\nBy calling `up()` with no arguments, this will execute all migration steps from the initial version to\nthe latest version.  As new versions are released, you may need to repeat this process, by first\ngenerating a new migration:\n\n```\n$ mix ecto.gen.migration upgrade_beacon_tables_to_v2\n```\n\nThen in the generated migration, you could simply call `up()` again, because the migrations are\nidempotent, but you can be safer and more efficient by specifying the migration version to execute:\n\n```elixir\n defmodule MyApp.Repo.Migrations.UpgradeBeaconTables do\n  use Ecto.Migration\n  def up, do: Beacon.Migration.up(version: 2)\n  def down, do: Beacon.Migration.down(version: 2)\nend\n```\n\nNow this migration will update to v2, but if rolled back, will only roll back the v2 changes,\nleaving v1 tables in-place.\n\nTo see this step within the larger context of installing Beacon, check out the [your first site](your-first-site.html) guide.","ref":"Beacon.Migration.html#module-usage"},{"type":"function","title":"Beacon.Migration.down/1","doc":"Downgrades Beacon database schemas.\n\nIf a specific version number is provided, Beacon will only downgrade to that version (inclusive).\nOtherwise, it will completely uninstall Beacon from your app's database.","ref":"Beacon.Migration.html#down/1"},{"type":"function","title":"Example - Beacon.Migration.down/1","doc":"Run all migrations from current version down to the first:\n\n    Beacon.Migration.down()\n\nRun migrations down to and including a specified version:\n\n    Beacon.Migration.down(version: 2)","ref":"Beacon.Migration.html#down/1-example"},{"type":"function","title":"Beacon.Migration.up/1","doc":"Upgrades Beacon database schemas.\n\nIf a specific version number is provided, Beacon will only upgrade to that version.\nOtherwise, it will bring you fully up-to-date with the current version.","ref":"Beacon.Migration.html#up/1"},{"type":"function","title":"Example - Beacon.Migration.up/1","doc":"Run all migrations up to the current version:\n\n    Beacon.Migration.up()\n\nRun migrations up to a specified version:\n\n    Beacon.Migration.down(version: 2)","ref":"Beacon.Migration.html#up/1-example"},{"type":"module","title":"Beacon.Router","doc":"Controls pages routing and provides helpers to mount sites in your application router and generate links to pages.\n\n    defmodule MyAppWeb.Router do\n      use Phoenix.Router\n      use Beacon.Router\n\n      scope \"/\", MyAppWeb do\n        pipe_through :browser\n        beacon_site \"/blog\", site: :blog\n      end\n    end","ref":"Beacon.Router.html"},{"type":"module","title":"Helpers - Beacon.Router","doc":"A `~p` sigil is provided to generate links to pages taking the `scope` and site `prefix` into account.\n\nUsing that sigil in a template in the `:blog` site defined above would result in the following links:\n\n```\n~p\"/contact\" => \"/blog/contact\"\n~p\"/posts/#{@post}\" => \"/blog/posts/my-post\"\n```\n\nIn this example `post` is a `Beacon.Content.Page` that implements the `Phoenix.Param` protocol to resolve the page path.","ref":"Beacon.Router.html#module-helpers"},{"type":"module","title":"Path - Beacon.Router","doc":"The full path of the site is calculated resolving the `scope` prefix plus the site `prefix`.\n\nThe simplest scenario is mounting a site at the root of your application:\n\n    scope \"/\", MyAppWeb do\n      pipe_through :browser\n      beacon_site \"/\", site: :my_site\n    end\n\nIn this case the site `:my_site` will be available at `https://yourapp.com/`\n\nBy mixing prefixes you have the flexibility to mount sites in different paths,\nfor example both declarations below will mount the site at `https://yourapp.com/blog`:\n\n    scope \"/blog\", MyAppWeb do\n      pipe_through :browser\n      beacon_site \"/\", site: :blog\n    end\n\n    scope \"/\", MyAppWeb do\n      pipe_through :browser\n      beacon_site \"/blog\", site: :blog\n    end\n\nThere's no difference between the two approaches, but that is important to group and organize your routes and sites,\nfor example a scope might be served through a different pipeline:\n\n    scope \"/marketing\", MyAppWeb do\n      pipe_through :browser_analytics\n      beacon_site \"/super-campaign\", site: :marketing_super_campaign\n      beacon_site \"/\", site: :marketing\n    end\n\nNote in the last example that `/super-campaign` is defined _before_ `/` and there's an important reason for that: router precedence.","ref":"Beacon.Router.html#module-path"},{"type":"module","title":"Route Precedence - Beacon.Router","doc":"Beacon pages are defined dynamically so it doesn't know which pages are available when the router is compiled,\nwhich means that any route after the `prefix` may match a published page. For example `/contact` may be a valid\npage published under the mounted `beacon_site \"/, site: :marketing` site.\n\nEssentially it mounts a catch-all route like `/*` so if we had inverted the routes below we would end with:\n\n    /*\n    /super-campaign\n\nThe second route would never match since the first one would match all requests.\n\nAs a rule of thumb, put all specific routes first.","ref":"Beacon.Router.html#module-route-precedence"},{"type":"macro","title":"Beacon.Router.beacon_site/2","doc":"Mounts a site at `prefix` in your host application's router.\n\nThis will automatically serve `sitemap.xml` from the `prefix` path defined for this site,\nand also `robots.txt` and `sitemap_index.xml` in the top-level host.","ref":"Beacon.Router.html#beacon_site/2"},{"type":"macro","title":"Options - Beacon.Router.beacon_site/2","doc":"* `:site` (required) `t:Beacon.Types.Site.t/0` - register your site with a unique name.\n    Note that the name has to match the one used in your site configuration.\n    See the module doc and `Beacon.Config` for more info.\n\n  Live Session options:\n  \n  You can also override the following [live_session](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.Router.html#live_session/3) options:\n\n  * `:session` (optional) - an optional extra session map or MFA tuple to merge into Beacon's `live_session`.\n    Useful to authenticate the session using 3rd-party libs like AshAuthentication. Defaults to `%{}`.\n  * `:root_layout` - override the default root layout for the site. Defaults to `{Beacon.Web.Layouts, :runtime}`.\n    See `Beacon.Web.Layouts` for more info.\n  * `:on_mount` (optional) - an optional list of `on_mount` hooks to merge into Beacon's `live_session`.\n    This will allow for authenticated routes, among other uses. Defaults to `[]`.","ref":"Beacon.Router.html#beacon_site/2-options"},{"type":"macro","title":"Examples - Beacon.Router.beacon_site/2","doc":"Initialize assigns for all pages in a site:\n\n    beacon_site \"/\", site: :my_site, on_mount: MyAppWeb.InitAssigns\n\n    defmodule MyAppWeb.InitAssigns do\n      import Phoenix.Component\n\n      def on_mount(:default, _params, session, socket) do\n        {:cont, assign(socket, :logo_url, \"https://mycdn.com/logo.webp\")}\n      end\n    end\n\nIntegrate a Beacon site with Ash:\n\n    beacon_site \"/\", [site: :my_site] ++ AshAuthentication.Phoenix.LiveSession.opts(on_mount: [{MyAppWeb.LiveUserAuth, :live_user_required}])","ref":"Beacon.Router.html#beacon_site/2-examples"},{"type":"module","title":"Beacon.Content","doc":"The building blocks for composing web pages: Layouts, Pages, Components, Stylesheets, and Snippets.","ref":"Beacon.Content.html"},{"type":"module","title":"Templates - Beacon.Content","doc":"Layout and Pages work together as pages require a layout to display its content,\nthe minimal template for a layout that can exist is the following:\n\n```heex\n<%= @inner_content %>\n```\n\nAnd pages templates can be written in [HEEx](https://hexdocs.pm/phoenix_live_view/assigns-eex.html)\nor [Markdown](https://docs.github.com/en/get-started/writing-on-github/getting-started-with-writing-and-formatting-on-github/basic-writing-and-formatting-syntax) formats.","ref":"Beacon.Content.html#module-templates"},{"type":"module","title":"Meta Tags - Beacon.Content","doc":"Meta Tags can are defined in 3 levels:\n\n  * Site - fixed meta tags displayed on all pages, see `default_site_meta_tags/0`\n  * Layouts - applies to all pages used by the template.\n  * Page - only applies to the specific page.","ref":"Beacon.Content.html#module-meta-tags"},{"type":"function","title":"Beacon.Content.change_component/2","doc":"Returns an `%Ecto.Changeset{}` for tracking component changes.","ref":"Beacon.Content.html#change_component/2"},{"type":"function","title":"Example - Beacon.Content.change_component/2","doc":"iex> change_component(component, %{name: \"Header\"})\n    %Ecto.Changeset{data: %Component{}}","ref":"Beacon.Content.html#change_component/2-example"},{"type":"function","title":"Beacon.Content.change_component_attr/3","doc":"Returns an `%Ecto.Changeset{}` for tracking component_attr changes.","ref":"Beacon.Content.html#change_component_attr/3"},{"type":"function","title":"Example - Beacon.Content.change_component_attr/3","doc":"iex> change_component_attr(component_attr, %{name: \"Header\"})\n    %Ecto.Changeset{data: %ComponentAttr{}}\n\n    iex> change_component_attr(component_attr, %{name: \"Header\"}, [\"sites\", [\"pages\"]])\n    %Ecto.Changeset{data: %ComponentAttr{}}","ref":"Beacon.Content.html#change_component_attr/3-example"},{"type":"function","title":"Beacon.Content.change_component_slot/3","doc":"Returns an `%Ecto.Changeset{}` for tracking slot changes.","ref":"Beacon.Content.html#change_component_slot/3"},{"type":"function","title":"Example - Beacon.Content.change_component_slot/3","doc":"iex> change_component_slot(component_slot, %{name: \"slot_a\"}, [\"slot_name_1])\n    %Ecto.Changeset{data: %ComponentSlot{}}","ref":"Beacon.Content.html#change_component_slot/3-example"},{"type":"function","title":"Beacon.Content.change_error_page/2","doc":"Returns an `%Ecto.Changeset{}` for tracking error page changes.","ref":"Beacon.Content.html#change_error_page/2"},{"type":"function","title":"Example - Beacon.Content.change_error_page/2","doc":"iex> change_error_page(error_page, %{status: 404})\n    %Ecto.Changeset{data: %ErrorPage{}}","ref":"Beacon.Content.html#change_error_page/2-example"},{"type":"function","title":"Beacon.Content.change_event_handler/2","doc":"Returns an `%Ecto.Changeset{}` for tracking event handler changes.","ref":"Beacon.Content.html#change_event_handler/2"},{"type":"function","title":"Example - Beacon.Content.change_event_handler/2","doc":"iex> change_event_handler(event_handler, %{name: \"form-submit\"})\n    %Ecto.Changeset{data: %EventHandler{}}","ref":"Beacon.Content.html#change_event_handler/2-example"},{"type":"function","title":"Beacon.Content.change_info_handler/2","doc":"Returns an `%Ecto.Changeset{}` for tracking info handler changes.","ref":"Beacon.Content.html#change_info_handler/2"},{"type":"function","title":"Example - Beacon.Content.change_info_handler/2","doc":"iex> change_info_handler(info_handler, %{code: {:noreply, socket}})\n    %Ecto.Changeset{data: %InfoHandler{}}","ref":"Beacon.Content.html#change_info_handler/2-example"},{"type":"function","title":"Beacon.Content.change_js_hook/2","doc":"Returns an `%Ecto.Changeset{}` for tracking JS Hook changes.","ref":"Beacon.Content.html#change_js_hook/2"},{"type":"function","title":"Example - Beacon.Content.change_js_hook/2","doc":"iex> change_js_hook(js_hook, %{name: \"MyCustomHook\"})\n    %Ecto.Changeset{data: %JSHook{}}","ref":"Beacon.Content.html#change_js_hook/2-example"},{"type":"function","title":"Beacon.Content.change_layout/2","doc":"Returns an `%Ecto.Changeset{}` for tracking layout changes.","ref":"Beacon.Content.html#change_layout/2"},{"type":"function","title":"Example - Beacon.Content.change_layout/2","doc":"iex> change_layout(layout, %{title: \"New Home\"})\n    %Ecto.Changeset{data: %Layout{}}","ref":"Beacon.Content.html#change_layout/2-example"},{"type":"function","title":"Beacon.Content.change_live_data_assign/2","doc":"Returns an `%Ecto.Changeset{}` for tracking LiveDataAssign changes.","ref":"Beacon.Content.html#change_live_data_assign/2"},{"type":"function","title":"Example - Beacon.Content.change_live_data_assign/2","doc":"iex> change_live_data_assign(live_data_assign, %{format: :elixir, value: \"Enum.random(1..100)\"})\n    %Ecto.Changeset{data: %LiveDataAssign{}}","ref":"Beacon.Content.html#change_live_data_assign/2-example"},{"type":"function","title":"Beacon.Content.change_live_data_path/2","doc":"Returns an `%Ecto.Changeset{}` for tracking LiveData `:path` changes.","ref":"Beacon.Content.html#change_live_data_path/2"},{"type":"function","title":"Example - Beacon.Content.change_live_data_path/2","doc":"iex> change_live_data(live_data, %{path: \"/foo/:bar_id\"})\n    %Ecto.Changeset{data: %LiveData{}}","ref":"Beacon.Content.html#change_live_data_path/2-example"},{"type":"function","title":"Beacon.Content.change_page/2","doc":"Returns an `%Ecto.Changeset{}` for tracking page changes.","ref":"Beacon.Content.html#change_page/2"},{"type":"function","title":"Example - Beacon.Content.change_page/2","doc":"iex> change_page(page, %{title: \"My Campaign\"})\n    %Ecto.Changeset{data: %Page{}}","ref":"Beacon.Content.html#change_page/2-example"},{"type":"function","title":"Beacon.Content.change_page_variant/2","doc":"Returns an `%Ecto.Changeset{}` for tracking variant changes.","ref":"Beacon.Content.html#change_page_variant/2"},{"type":"function","title":"Example - Beacon.Content.change_page_variant/2","doc":"iex> change_page_variant(page_variant, %{name: \"Variant A\"})\n    %Ecto.Changeset{data: %PageVariant{}}","ref":"Beacon.Content.html#change_page_variant/2-example"},{"type":"function","title":"Beacon.Content.change_slot_attr/3","doc":"Returns an `%Ecto.Changeset{}` for tracking slot_attr changes.","ref":"Beacon.Content.html#change_slot_attr/3"},{"type":"function","title":"Example - Beacon.Content.change_slot_attr/3","doc":"iex> change_slot_attr(slot_attr, %{name: \"Header\"}, [])\n    %Ecto.Changeset{data: %ComponentSlotAttr{}}","ref":"Beacon.Content.html#change_slot_attr/3-example"},{"type":"function","title":"Beacon.Content.component_categories/0","doc":"Returns a list of all existing component categories.","ref":"Beacon.Content.html#component_categories/0"},{"type":"function","title":"Beacon.Content.count_components/2","doc":"Counts the total number of components based on the amount of pages.","ref":"Beacon.Content.html#count_components/2"},{"type":"function","title":"Options - Beacon.Content.count_components/2","doc":"* `:query` - filter rows count by query. Defaults to `nil`, doesn't filter query.","ref":"Beacon.Content.html#count_components/2-options"},{"type":"function","title":"Beacon.Content.count_layouts/2","doc":"Counts the total number of layouts based on the amount of pages.","ref":"Beacon.Content.html#count_layouts/2"},{"type":"function","title":"Options - Beacon.Content.count_layouts/2","doc":"* `:query` - filter rows count by query. Defaults to `nil`, doesn't filter query.","ref":"Beacon.Content.html#count_layouts/2-options"},{"type":"function","title":"Beacon.Content.count_pages/2","doc":"Counts the total number of pages based on the amount of pages.","ref":"Beacon.Content.html#count_pages/2"},{"type":"function","title":"Options - Beacon.Content.count_pages/2","doc":"* `:query` - filter rows count by query","ref":"Beacon.Content.html#count_pages/2-options"},{"type":"function","title":"Beacon.Content.create_assign_for_live_data/2","doc":"Creates a new LiveDataAssign.","ref":"Beacon.Content.html#create_assign_for_live_data/2"},{"type":"function","title":"Beacon.Content.create_component/1","doc":"Creates a component.\n\nReturns `{:ok, component}` if successful, otherwise `{:error, changeset}`.","ref":"Beacon.Content.html#create_component/1"},{"type":"function","title":"Example - Beacon.Content.create_component/1","doc":"iex> create_component(attrs)\n    {:ok, %Component{}}","ref":"Beacon.Content.html#create_component/1-example"},{"type":"function","title":"Beacon.Content.create_component!/1","doc":"Creates a component, raising an error if unsuccessful.\n\nReturns the new component if successful, otherwise raises a `RuntimeError`.","ref":"Beacon.Content.html#create_component!/1"},{"type":"function","title":"Beacon.Content.create_error_page/1","doc":"Creates a new error page.","ref":"Beacon.Content.html#create_error_page/1"},{"type":"function","title":"Beacon.Content.create_error_page!/1","doc":"Creates a new error page, raising if the operation fails.","ref":"Beacon.Content.html#create_error_page!/1"},{"type":"function","title":"Beacon.Content.create_event_handler/1","doc":"Creates a new event handler.","ref":"Beacon.Content.html#create_event_handler/1"},{"type":"function","title":"Beacon.Content.create_event_handler!/1","doc":"Creates an event handler, raising an error if unsuccessful.","ref":"Beacon.Content.html#create_event_handler!/1"},{"type":"function","title":"Beacon.Content.create_info_handler/1","doc":"Creates a new info handler for creating shared handle_info callbacks.","ref":"Beacon.Content.html#create_info_handler/1"},{"type":"function","title":"Example - Beacon.Content.create_info_handler/1","doc":"iex> create_info_handler(%{site: \"my_site\", msg: \"{:new_msg, arg}\", code: \"{:noreply, socket}\"})\n    {:ok, %InfoHandler{}}","ref":"Beacon.Content.html#create_info_handler/1-example"},{"type":"function","title":"Beacon.Content.create_info_handler!/1","doc":"Creates a info handler, raising an error if unsuccessful.\n\nReturns the new info handler if successful, otherwise raises a `RuntimeError`.","ref":"Beacon.Content.html#create_info_handler!/1"},{"type":"function","title":"Example - Beacon.Content.create_info_handler!/1","doc":"iex> create_info_handler!(%{site: \"my_site\", msg: \"{:new_msg, arg}\", code: \"{:noreply, socket}\"})\n    %InfoHandler{}","ref":"Beacon.Content.html#create_info_handler!/1-example"},{"type":"function","title":"Beacon.Content.create_js_hook/1","doc":"Creates a JS Hook.\n\nReturns `{:ok, js_hook}` if successful, otherwise `{:error, changeset}`.","ref":"Beacon.Content.html#create_js_hook/1"},{"type":"function","title":"Example - Beacon.Content.create_js_hook/1","doc":"iex> code = \"export const ConsoleLogHook = {mounted() {console.log(\"foo\")}}\"\n    iex> create_js_hook(%{site: :my_site, name: \"ConsoleLogHook\", code: code})\n    {:ok, %JSHook{}}","ref":"Beacon.Content.html#create_js_hook/1-example"},{"type":"function","title":"Beacon.Content.create_js_hook!/1","doc":"Creates a JS Hook, raising an error if unsuccessful.","ref":"Beacon.Content.html#create_js_hook!/1"},{"type":"function","title":"Beacon.Content.create_layout/1","doc":"Creates a layout.","ref":"Beacon.Content.html#create_layout/1"},{"type":"function","title":"Example - Beacon.Content.create_layout/1","doc":"iex> create_layout(%{title: \"Home\"})\n    {:ok, %Layout{}}","ref":"Beacon.Content.html#create_layout/1-example"},{"type":"function","title":"Beacon.Content.create_layout!/1","doc":"Creates a layout.","ref":"Beacon.Content.html#create_layout!/1"},{"type":"function","title":"Beacon.Content.create_live_data/1","doc":"Creates a new LiveData for scoping live data to pages.\n\nReturns `{:ok, live_data}` if successful, otherwise `{:error, changeset}`","ref":"Beacon.Content.html#create_live_data/1"},{"type":"function","title":"Beacon.Content.create_live_data!/1","doc":"Creates a new LiveData for scoping live data to pages, raising an error if unsuccessful.\n\nReturns the new LiveData if successful, otherwise raises a `RuntimeError`.","ref":"Beacon.Content.html#create_live_data!/1"},{"type":"function","title":"Beacon.Content.create_page/1","doc":"Creates a new page that's not published.","ref":"Beacon.Content.html#create_page/1"},{"type":"function","title":"Example - Beacon.Content.create_page/1","doc":"iex> create_page(%{\"title\" => \"My New Page\"})\n    {:ok, %Page{}}\n\n`attrs` may contain the following keys:\n\n  * `path` - String.t()\n  * `title` - String.t()\n  * `description` - String.t()\n  * `template` - String.t()\n  * `meta_tags` - list(map()) eg: `[%{\"property\" => \"og:title\", \"content\" => \"My New Site\"}]`\n\nSee `Beacon.Content.Page` for more info.\n\nThe created page is not published automatically,\nyou can make as much changes you need and when the page\nis ready to be published you can call `publish_page/1`\n\nIt will insert a `created` event into the page timeline,\nand no snapshot is created.","ref":"Beacon.Content.html#create_page/1-example"},{"type":"function","title":"Beacon.Content.create_page!/1","doc":"Creates a page.\n\nRaises an error if unsuccessful.","ref":"Beacon.Content.html#create_page!/1"},{"type":"function","title":"Beacon.Content.create_slot_attr/3","doc":"Creates a slot attr.","ref":"Beacon.Content.html#create_slot_attr/3"},{"type":"function","title":"Example - Beacon.Content.create_slot_attr/3","doc":"iex> create_slot_attr(site, attrs)\n    {:ok, %ComponentSlotAttr{}}","ref":"Beacon.Content.html#create_slot_attr/3-example"},{"type":"function","title":"Beacon.Content.create_slot_for_component/2","doc":"Creates a new component slot and returns the component with updated `:slots` association.","ref":"Beacon.Content.html#create_slot_for_component/2"},{"type":"function","title":"Beacon.Content.create_snippet_helper/1","doc":"Creates a snippet helper.\n\nReturns `{:ok, helper}` if successful, otherwise `{:error, changeset}`","ref":"Beacon.Content.html#create_snippet_helper/1"},{"type":"function","title":"Beacon.Content.create_snippet_helper!/1","doc":"Creates a snippet helper, raising an error if unsuccessful.\n\nReturns the new helper if successful, otherwise raises a `RuntimeError`.","ref":"Beacon.Content.html#create_snippet_helper!/1"},{"type":"function","title":"Beacon.Content.create_stylesheet/1","doc":"Creates a stylesheet.\n\nReturns `{:ok, stylesheet}` if successful, otherwise `{:error, changeset}`.","ref":"Beacon.Content.html#create_stylesheet/1"},{"type":"function","title":"Example - Beacon.Content.create_stylesheet/1","doc":"iex> create_stylesheet(%{\n      site: :my_site,\n      name: \"override\",\n      content: ~S|\n      @media (min-width: 768px) {\n        .md:text-red-400 {\n          color: red;\n        }\n      }\n      |\n    })\n    {:ok, %Stylesheet{}}\n\nNote that escape characters must be preserved, so you should use `~S` to avoid issues.","ref":"Beacon.Content.html#create_stylesheet/1-example"},{"type":"function","title":"Beacon.Content.create_stylesheet!/1","doc":"Creates a stylesheet, raising an error if unsuccessful.\n\nReturns the new stylesheet if successful, otherwise raises a `RuntimeError`.","ref":"Beacon.Content.html#create_stylesheet!/1"},{"type":"function","title":"Example - Beacon.Content.create_stylesheet!/1","doc":"iex> create_stylesheet!(%{\n      site: :my_site,\n      name: \"override\",\n      content: ~S|\n      @media (min-width: 768px) {\n        .md:text-red-400 {\n          color: red;\n        }\n      }\n      |\n    })\n    %Stylesheet{}\n\nNote that escape characters must be preserved, so you should use `~S` to avoid issues.","ref":"Beacon.Content.html#create_stylesheet!/1-example"},{"type":"function","title":"Beacon.Content.create_variant_for_page/2","doc":"Creates a new page variant and returns the page with updated `:variants` association.","ref":"Beacon.Content.html#create_variant_for_page/2"},{"type":"function","title":"Beacon.Content.default_error_pages/0","doc":"Returns attr data to load the default error_pages into new sites.","ref":"Beacon.Content.html#default_error_pages/0"},{"type":"function","title":"Beacon.Content.default_hook_code/1","doc":"Generates an empty Hook template for the given name.","ref":"Beacon.Content.html#default_hook_code/1"},{"type":"function","title":"Beacon.Content.default_layout/0","doc":"Returns a map of attrs to load the default layout into new sites.","ref":"Beacon.Content.html#default_layout/0"},{"type":"function","title":"Beacon.Content.default_site_meta_tags/0","doc":"Returns the list of meta tags that are applied to all pages by default.\n\nThese meta tags can be overwritten or extended on a Layout or Page level.","ref":"Beacon.Content.html#default_site_meta_tags/0"},{"type":"function","title":"Beacon.Content.delete_error_page/1","doc":"Deletes an error page.","ref":"Beacon.Content.html#delete_error_page/1"},{"type":"function","title":"Beacon.Content.delete_event_handler/1","doc":"Deletes an event handler.","ref":"Beacon.Content.html#delete_event_handler/1"},{"type":"function","title":"Beacon.Content.delete_info_handler/1","doc":"Deletes info handler.","ref":"Beacon.Content.html#delete_info_handler/1"},{"type":"function","title":"Beacon.Content.delete_js_hook/1","doc":"Deletes a JS Hook.","ref":"Beacon.Content.html#delete_js_hook/1"},{"type":"function","title":"Beacon.Content.delete_live_data/1","doc":"Deletes LiveData.","ref":"Beacon.Content.html#delete_live_data/1"},{"type":"function","title":"Beacon.Content.delete_live_data_assign/2","doc":"Deletes LiveDataAssign.","ref":"Beacon.Content.html#delete_live_data_assign/2"},{"type":"function","title":"Beacon.Content.delete_slot_attr/2","doc":"Deletes a slot attr.","ref":"Beacon.Content.html#delete_slot_attr/2"},{"type":"function","title":"Beacon.Content.delete_slot_from_component/2","doc":"Deletes a component slot and returns the component with updated slots association.","ref":"Beacon.Content.html#delete_slot_from_component/2"},{"type":"function","title":"Beacon.Content.delete_variant_from_page/2","doc":"Deletes a page variant and returns the page with updated `:variants` association.","ref":"Beacon.Content.html#delete_variant_from_page/2"},{"type":"function","title":"Beacon.Content.get_component_by/3","doc":"Gets a single component by `clauses`.","ref":"Beacon.Content.html#get_component_by/3"},{"type":"function","title":"Example - Beacon.Content.get_component_by/3","doc":"iex> get_component_by(site, name: \"header\")\n    %Component{}","ref":"Beacon.Content.html#get_component_by/3-example"},{"type":"function","title":"Beacon.Content.get_error_page/2","doc":"Returns the error page for a given site and status code, or `nil` if no matching error page exists.","ref":"Beacon.Content.html#get_error_page/2"},{"type":"function","title":"Beacon.Content.get_info_handler/2","doc":"Gets a single info handler by `id`.","ref":"Beacon.Content.html#get_info_handler/2"},{"type":"function","title":"Example - Beacon.Content.get_info_handler/2","doc":"iex> get_single_info_handler(:my_site, \"fefebbfe-f732-4119-9116-d031d04f5a2c\")\n    %InfoHandler{}","ref":"Beacon.Content.html#get_info_handler/2-example"},{"type":"function","title":"Beacon.Content.get_info_handler!/2","doc":"Same as `get_info_handler/2` but raises an error if no result is found.","ref":"Beacon.Content.html#get_info_handler!/2"},{"type":"function","title":"Beacon.Content.get_js_hook_by/3","doc":"Gets a single JS hooks by `clauses`.","ref":"Beacon.Content.html#get_js_hook_by/3"},{"type":"function","title":"Example - Beacon.Content.get_js_hook_by/3","doc":"iex> get_component_by(site, name: \"CloseOnGlobalClick\")\n    %JSHook{}","ref":"Beacon.Content.html#get_js_hook_by/3-example"},{"type":"function","title":"Beacon.Content.get_latest_layout_event/2","doc":"Returns the latest layout event.\n\nUseful to find the status of a layout.","ref":"Beacon.Content.html#get_latest_layout_event/2"},{"type":"function","title":"Example - Beacon.Content.get_latest_layout_event/2","doc":"iex> get_latest_layout_event(:my_site, layout_id)\n    %LayoutEvent{event: :published}","ref":"Beacon.Content.html#get_latest_layout_event/2-example"},{"type":"function","title":"Beacon.Content.get_latest_page_event/2","doc":"Returns the latest page event.\n\nUseful to find the status of a page.","ref":"Beacon.Content.html#get_latest_page_event/2"},{"type":"function","title":"Example - Beacon.Content.get_latest_page_event/2","doc":"iex> get_latest_page_event(:my_site, page_id)\n    %PageEvent{event: :published}","ref":"Beacon.Content.html#get_latest_page_event/2-example"},{"type":"function","title":"Beacon.Content.get_layout/2","doc":"Gets a single layout by `id`.","ref":"Beacon.Content.html#get_layout/2"},{"type":"function","title":"Example - Beacon.Content.get_layout/2","doc":"iex> get_layout(:my_site, \"fd70e5fe-9bd8-41ed-94eb-5459c9bb05fc\")\n    %Layout{}","ref":"Beacon.Content.html#get_layout/2-example"},{"type":"function","title":"Beacon.Content.get_layout!/2","doc":"Same as `get_layout/2` but raises an error if no result is found.","ref":"Beacon.Content.html#get_layout!/2"},{"type":"function","title":"Beacon.Content.get_layout_by/3","doc":"Gets a single layout by `clauses`.","ref":"Beacon.Content.html#get_layout_by/3"},{"type":"function","title":"Example - Beacon.Content.get_layout_by/3","doc":"iex> get_layout_by(site, title: \"blog\")\n    %Layout{}","ref":"Beacon.Content.html#get_layout_by/3-example"},{"type":"function","title":"Beacon.Content.get_live_data_by/3","doc":"Gets a single live data by `clauses`.","ref":"Beacon.Content.html#get_live_data_by/3"},{"type":"function","title":"Example - Beacon.Content.get_live_data_by/3","doc":"iex> get_live_data_by(site, id: \"cba9ee2e-d40e-48af-9704-1237e4c23bde\")\n    %LiveData{}\n\n    iex> get_live_data_by(site, path: \"/blog\")\n    %LiveData{}","ref":"Beacon.Content.html#get_live_data_by/3-example"},{"type":"function","title":"Beacon.Content.get_page/3","doc":"Gets a single page by `id`.","ref":"Beacon.Content.html#get_page/3"},{"type":"function","title":"Options - Beacon.Content.get_page/3","doc":"* `:preloads` - a list of preloads to load.","ref":"Beacon.Content.html#get_page/3-options"},{"type":"function","title":"Examples - Beacon.Content.get_page/3","doc":"iex> get_page(:my_site, \"dba8a99e-311a-4806-af04-dd968c7e5dae\")\n    %Page{}\n\n    iex> get_page(:my_site, \"dba8a99e-311a-4806-af04-dd968c7e5dae\", preloads: [:layout])\n    %Page{layout: %Layout{}}","ref":"Beacon.Content.html#get_page/3-examples"},{"type":"function","title":"Beacon.Content.get_page!/3","doc":"Same as `get_page/3` but raises an error if no result is found.","ref":"Beacon.Content.html#get_page!/3"},{"type":"function","title":"Beacon.Content.get_page_by/3","doc":"Gets a single page by `clauses`.","ref":"Beacon.Content.html#get_page_by/3"},{"type":"function","title":"Example - Beacon.Content.get_page_by/3","doc":"iex> get_page_by(site, path: \"/contact\")\n    %Page{}","ref":"Beacon.Content.html#get_page_by/3-example"},{"type":"function","title":"Beacon.Content.get_published_layout/2","doc":"Get latest published layout.\n\nThis operation is cached.","ref":"Beacon.Content.html#get_published_layout/2"},{"type":"function","title":"Beacon.Content.get_published_page/2","doc":"Get latest published page.\n\nThis operation is cached.","ref":"Beacon.Content.html#get_published_page/2"},{"type":"function","title":"Beacon.Content.get_stylesheet_by/3","doc":"Gets a single stylesheet by `clauses`.","ref":"Beacon.Content.html#get_stylesheet_by/3"},{"type":"function","title":"Example - Beacon.Content.get_stylesheet_by/3","doc":"iex> get_stylesheet_by(site, name: \"main\")\n    %Stylesheet{}","ref":"Beacon.Content.html#get_stylesheet_by/3-example"},{"type":"function","title":"Beacon.Content.list_components/2","doc":"List components.","ref":"Beacon.Content.html#list_components/2"},{"type":"function","title":"Options - Beacon.Content.list_components/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records. Defaults to 20.\n  * `:page` - returns records from a specific page. Defaults to 1.\n  * `:query` - search components by title. Defaults to `nil`, doesn't filter query.\n  * `:preloads` - a list of preloads to load.\n  * `:sort` - column in which the result will be ordered by. Defaults to `:name`.","ref":"Beacon.Content.html#list_components/2-options"},{"type":"function","title":"Beacon.Content.list_components_by_name/2","doc":"List components by `name`.","ref":"Beacon.Content.html#list_components_by_name/2"},{"type":"function","title":"Example - Beacon.Content.list_components_by_name/2","doc":"iex> list_components_by_name(site, \"header\")\n    [%Component{name: \"header\"}]","ref":"Beacon.Content.html#list_components_by_name/2-example"},{"type":"function","title":"Beacon.Content.list_error_pages/2","doc":"Lists all error pages for a given site.","ref":"Beacon.Content.html#list_error_pages/2"},{"type":"function","title":"Options - Beacon.Content.list_error_pages/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records.\n  * `:preloads` - a list of preloads to load.","ref":"Beacon.Content.html#list_error_pages/2-options"},{"type":"function","title":"Beacon.Content.list_error_pages_by/3","doc":"Lists all error pages for a given site, filtered by `clauses`.\n\nCurrently the only acceptable clause is `:layout_id`.\nSee `list_error_pages/2` for a list of acceptable `opts`.","ref":"Beacon.Content.html#list_error_pages_by/3"},{"type":"function","title":"Beacon.Content.list_event_handlers/1","doc":"Lists all event handlers for a given Beacon site.","ref":"Beacon.Content.html#list_event_handlers/1"},{"type":"function","title":"Beacon.Content.list_info_handlers/1","doc":"Lists all info handlers for a given site.","ref":"Beacon.Content.html#list_info_handlers/1"},{"type":"function","title":"Example - Beacon.Content.list_info_handlers/1","doc":"iex> list_info_handlers()","ref":"Beacon.Content.html#list_info_handlers/1-example"},{"type":"function","title":"Beacon.Content.list_js_hooks/1","doc":"Lists all JS Hooks for a site.","ref":"Beacon.Content.html#list_js_hooks/1"},{"type":"function","title":"Beacon.Content.list_layout_events/2","doc":"Returns all layout events with associated snapshot if available.","ref":"Beacon.Content.html#list_layout_events/2"},{"type":"function","title":"Example - Beacon.Content.list_layout_events/2","doc":"iex> list_layout_events(:my_site, layout_id)\n    [\n      %LayoutEvent{event: :created, snapshot: nil},\n      %LayoutEvent{event: :published, snapshot: %LayoutSnapshot{}}\n    ]","ref":"Beacon.Content.html#list_layout_events/2-example"},{"type":"function","title":"Beacon.Content.list_layouts/2","doc":"List layouts.","ref":"Beacon.Content.html#list_layouts/2"},{"type":"function","title":"Options - Beacon.Content.list_layouts/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records. Defaults to 20.\n  * `:page` - returns records from a specific page. Defaults to 1.\n  * `:query` - search layouts by title. Defaults to `nil`, doesn't filter query.\n  * `:preloads` - a list of preloads to load.\n  * `:sort` - column in which the result will be ordered by. Defaults to `:title`.\n    Allowed values: `:title`, `:template`, `:meta_tags`, `:resource_links`, `:inserted_at`, `:updated_at`.","ref":"Beacon.Content.html#list_layouts/2-options"},{"type":"function","title":"Beacon.Content.list_page_events/2","doc":"Returns all page events with associated snapshot if available.","ref":"Beacon.Content.html#list_page_events/2"},{"type":"function","title":"Example - Beacon.Content.list_page_events/2","doc":"iex> list_page_events(:my_site, page_id)\n    [\n      %PageEvent{event: :created, snapshot: nil},\n      %PageEvent{event: :published, snapshot: %PageSnapshot{}}\n    ]","ref":"Beacon.Content.html#list_page_events/2-example"},{"type":"function","title":"Beacon.Content.list_pages/2","doc":"List pages.","ref":"Beacon.Content.html#list_pages/2"},{"type":"function","title":"Options - Beacon.Content.list_pages/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records. Defaults to 20.\n  * `:page` - returns records from a specific page. Defaults to 1.\n  * `:query` - search pages by path or title.\n  * `:preloads` - a list of preloads to load.\n  * `:sort` - column in which the result will be ordered by. Defaults to `:path`.\n    Allowed values: `:path`, `:title`, `:description`, `:template`, `:meta_tags`, `:raw_schema`, `:format`, `:inserted_at`, `:updated_at`.","ref":"Beacon.Content.html#list_pages/2-options"},{"type":"function","title":"Beacon.Content.list_published_layouts/1","doc":"Returns all published layouts for `site`.\n\nLayouts are extracted from the latest published `Beacon.Content.LayoutSnapshot`.","ref":"Beacon.Content.html#list_published_layouts/1"},{"type":"function","title":"Beacon.Content.list_published_pages/2","doc":"Lists and search all published pages for `site`.\n\nNote that unpublished pages are not returned even if it was once published before, only the latest snapshot is considered.","ref":"Beacon.Content.html#list_published_pages/2"},{"type":"function","title":"Options - Beacon.Content.list_published_pages/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records. Defaults to 20.\n  * `:page` - returns records from a specific page. Defaults to 1.\n  * `:search` - search by either one or more fields or dynamic query function.\n                Available fields: `path`, `title`, `format`, `extra`. Defaults to `nil` (do not apply search filter).\n  * `:sort` - column in which the result will be ordered by. Defaults to `:title`.","ref":"Beacon.Content.html#list_published_pages/2-options"},{"type":"function","title":"Examples - Beacon.Content.list_published_pages/2","doc":"iex> list_published_pages(:my_site, search: %{path: \"/home\", title: \"Home Page\"})\n    [%Page{}]\n\n    iex> list_published_pages(:my_site, search: %{extra: %{\"tags\" => \"press\"}})\n    [%Page{}]\n\n    iex> list_published_pages(:my_site, search: fn -> dynamic([q], fragment(\"extra->>'tags' ilike 'year-20%'\")) end)\n    [%Page{}]","ref":"Beacon.Content.html#list_published_pages/2-examples"},{"type":"function","title":"Beacon.Content.list_published_pages_for_paths/2","doc":"Similar to `list_published_pages/2`, but does not accept any options.  Instead, provide a list\nof paths, and this function will return any published pages which match one of those paths.","ref":"Beacon.Content.html#list_published_pages_for_paths/2"},{"type":"function","title":"Beacon.Content.list_snippet_helpers/1","doc":"Returns the list of snippet helpers for a `site`.","ref":"Beacon.Content.html#list_snippet_helpers/1"},{"type":"function","title":"Example - Beacon.Content.list_snippet_helpers/1","doc":"iex> list_snippet_helpers()\n    [%SnippetHelper{}, ...]","ref":"Beacon.Content.html#list_snippet_helpers/1-example"},{"type":"function","title":"Beacon.Content.list_stylesheets/1","doc":"Returns the list of stylesheets for `site`.","ref":"Beacon.Content.html#list_stylesheets/1"},{"type":"function","title":"Example - Beacon.Content.list_stylesheets/1","doc":"iex> list_stylesheets()\n    [%Stylesheet{}, ...]","ref":"Beacon.Content.html#list_stylesheets/1-example"},{"type":"function","title":"Beacon.Content.live_data_assign_formats/0","doc":"Returns a list of all existing LiveDataAssign formats.","ref":"Beacon.Content.html#live_data_assign_formats/0"},{"type":"function","title":"Beacon.Content.live_data_for_site/2","doc":"Query LiveData for a given site.","ref":"Beacon.Content.html#live_data_for_site/2"},{"type":"function","title":"Options - Beacon.Content.live_data_for_site/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records.\n  * `:query` - search records by path.\n  * `:select` - returns only the given field(s)\n  * `:preload` - include given association(s) (defaults to `:assigns`)","ref":"Beacon.Content.html#live_data_for_site/2-options"},{"type":"function","title":"Beacon.Content.publish_layout/1","doc":"Publishes `layout` and reload resources to render the updated layout and pages.\n\nEvent + snapshot\n\nThis operation is serialized.","ref":"Beacon.Content.html#publish_layout/1"},{"type":"function","title":"Beacon.Content.publish_layout/2","doc":"Same as `publish_layout/2` but accepts a `site` and `layout_id` with which to lookup the layout.","ref":"Beacon.Content.html#publish_layout/2"},{"type":"function","title":"Beacon.Content.publish_page/1","doc":"Publish `page`.\n\nA new snapshot is automatically created to store the page data,\nwhich is used whenever the site or the page is reloaded. So you\ncan keep editing the page as needed without impacting the published page.\n\nThis operation is serialized.","ref":"Beacon.Content.html#publish_page/1"},{"type":"function","title":"Beacon.Content.publish_page/2","doc":"Same as `publish_page/1` but accepts a `site` and `page_id` with which to lookup the page.","ref":"Beacon.Content.html#publish_page/2"},{"type":"function","title":"Beacon.Content.publish_pages/1","doc":"Publish multiple `pages`.\n\nSimilar to `publish_page/1` but defers loading dependent resources\nas late as possible making the process faster.","ref":"Beacon.Content.html#publish_pages/1"},{"type":"function","title":"Beacon.Content.put_page_extra/2","doc":"Given a map of fields, stores this map as `:extra` fields in a `Page`.\n\nAny existing `:extra` data for that Page will be overwritten!","ref":"Beacon.Content.html#put_page_extra/2"},{"type":"function","title":"Beacon.Content.render_snippet/2","doc":"Renders a snippet `template` with the given `assigns`.\n\nSnippets are small pieces of string with interpolated assigns.\n\nThink of it as small templates.","ref":"Beacon.Content.html#render_snippet/2"},{"type":"function","title":"Examples - Beacon.Content.render_snippet/2","doc":"iex> Beacon.Content.render_snippet(\"title is {{ page.title }}\", %{page: %{title: \"home\"}})\n    {:ok, \"title is home\"}\n\nSnippets use the [Liquid](https://shopify.github.io/liquid/) template under the hood,\nwhich means that all [filters](https://shopify.github.io/liquid/basics/introduction/#filters) are available for use, eg:\n\n    iex> Beacon.Content.render_snippet(\"{{ 'title' | capitalize }}\", assigns)\n    {:ok, \"Title\"}\n\nIn situations where the Liquid filters are not enough, you can create helpers\nto process the template using regular Elixir.\n\nIn the next example a `author_name` is created to simulate a query to fetch the author's name:\n\n    iex> page = Beacon.Content.create_page(%{site: \"my_site\", extra: %{\"author_id\": 1}})\n    iex> Beacon.Content.create_snippet_helper(%{site: \"my_site\", name: \"author_name\", body: ~S\"\"\"\n    ...> author_id = get_in(assigns, [\"page\", \"extra\", \"author_id\"])\n    ...> MyApp.fetch_author_name(author_id)\n    ...> \"\"\"\n    iex> Beacon.Snippet.render(\"Author is {{ helper 'author_name' }}\", %{page: page})\n    {:ok, \"Author is Anon\"}\n\nNote that the `:page` assigns is made available as `assigns[\"page\"]` (String.t) due to how Solid works.\n\nSnippets can be used in:\n\n  * Meta Tag value\n  * Page Schema (structured Schema.org tags)\n  * Page Title\n\nAllowed assigns:\n\n  * :page (map)\n  * :live_data (map)","ref":"Beacon.Content.html#render_snippet/2-examples"},{"type":"function","title":"Beacon.Content.unpublish_page/1","doc":"Unpublish `page`.\n\nThe page will be removed from your site and it will return error 404 for new requests.\n\nThis operation is serialized.","ref":"Beacon.Content.html#unpublish_page/1"},{"type":"function","title":"Beacon.Content.update_component/2","doc":"Updates a component.\n\n    iex> update_component(component, %{name: \"new_component\"})\n    {:ok, %Component{}}","ref":"Beacon.Content.html#update_component/2"},{"type":"function","title":"Beacon.Content.update_error_page/2","doc":"Updates an error page.","ref":"Beacon.Content.html#update_error_page/2"},{"type":"function","title":"Beacon.Content.update_event_handler/2","doc":"Updates an event handler with the given attrs.","ref":"Beacon.Content.html#update_event_handler/2"},{"type":"function","title":"Beacon.Content.update_info_handler/2","doc":"Updates a info handler.","ref":"Beacon.Content.html#update_info_handler/2"},{"type":"function","title":"Example - Beacon.Content.update_info_handler/2","doc":"iex> update_info_handler(info_handler, %{msg: \"{:new_msg, arg}\"})\n    {:ok, %InfoHandler{}}","ref":"Beacon.Content.html#update_info_handler/2-example"},{"type":"function","title":"Beacon.Content.update_js_hook/2","doc":"Updates a JS Hook.","ref":"Beacon.Content.html#update_js_hook/2"},{"type":"function","title":"Beacon.Content.update_layout/2","doc":"Updates a layout.","ref":"Beacon.Content.html#update_layout/2"},{"type":"function","title":"Example - Beacon.Content.update_layout/2","doc":"iex> update_layout(layout, %{title: \"New Home\"})\n    {:ok, %Layout{}}","ref":"Beacon.Content.html#update_layout/2-example"},{"type":"function","title":"Beacon.Content.update_live_data_assign/3","doc":"Updates LiveDataAssign.\n\n    iex> update_live_data_assign(live_data_assign, :my_site, %{code: \"true\"})\n    {:ok, %LiveDataAssign{}}","ref":"Beacon.Content.html#update_live_data_assign/3"},{"type":"function","title":"Beacon.Content.update_live_data_path/2","doc":"Updates LiveDataPath.\n\n    iex> update_live_data_path(live_data, \"/foo/bar/:baz_id\")\n    {:ok, %LiveData{}}","ref":"Beacon.Content.html#update_live_data_path/2"},{"type":"function","title":"Beacon.Content.update_page/2","doc":"Updates a page.","ref":"Beacon.Content.html#update_page/2"},{"type":"function","title":"Example - Beacon.Content.update_page/2","doc":"iex> update_page(page, %{title: \"New Home\"})\n    {:ok, %Page{}}","ref":"Beacon.Content.html#update_page/2-example"},{"type":"function","title":"Beacon.Content.update_slot_attr/4","doc":"Updates a slot attr.\n\n    iex> update_slot(slot_attr, %{name: \"new_slot\"})\n    {:ok, %ComponentSlotAttr{}}","ref":"Beacon.Content.html#update_slot_attr/4"},{"type":"function","title":"Beacon.Content.update_slot_for_component/4","doc":"Updates a component slot and returns the component with updated `:slots` association.","ref":"Beacon.Content.html#update_slot_for_component/4"},{"type":"function","title":"Beacon.Content.update_stylesheet/2","doc":"Updates a stylesheet.","ref":"Beacon.Content.html#update_stylesheet/2"},{"type":"function","title":"Example - Beacon.Content.update_stylesheet/2","doc":"iex> update_stylesheet(stylesheet, %{name: new_value})\n    {:ok, %Stylesheet{}}","ref":"Beacon.Content.html#update_stylesheet/2-example"},{"type":"function","title":"Beacon.Content.update_variant_for_page/3","doc":"Updates a page variant and returns the page with updated `:variants` association.","ref":"Beacon.Content.html#update_variant_for_page/3"},{"type":"function","title":"Beacon.Content.validate_page/3","doc":"Validate `page` with the given `params`.\n\nAll `Beacon.Content.PageField` are validated","ref":"Beacon.Content.html#validate_page/3"},{"type":"module","title":"Beacon.Content.Component","doc":"Beacon's representation of [Phoenix Components](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html),\nwhich can be used and re-used in your pages and layouts.\n\n> #### Do not create or edit components manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.Component.html"},{"type":"function","title":"Beacon.Content.Component.categories/0","doc":"","ref":"Beacon.Content.Component.html#categories/0"},{"type":"type","title":"Beacon.Content.Component.t/0","doc":"","ref":"Beacon.Content.Component.html#t:t/0"},{"type":"module","title":"Beacon.Content.ComponentAttr","doc":"Beacon's representation of Phoenix's [Component attributes](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#module-attributes).\n\nComponentAttrs don't exist on their own, but always belong to a `Beacon.Content.Component`.\n\n> #### Do not create or edit component attrs manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.ComponentAttr.html"},{"type":"type","title":"Beacon.Content.ComponentAttr.t/0","doc":"","ref":"Beacon.Content.ComponentAttr.html#t:t/0"},{"type":"module","title":"Beacon.Content.ComponentSlot","doc":"Beacon's representation of Phoenix's [Slots](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#module-slots).\n\nComponentSlots don't exist on their own, but always belong to a `Beacon.Content.Component`.\n\n> #### Do not create or edit component slots manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.ComponentSlot.html"},{"type":"type","title":"Beacon.Content.ComponentSlot.t/0","doc":"","ref":"Beacon.Content.ComponentSlot.html#t:t/0"},{"type":"module","title":"Beacon.Content.ComponentSlotAttr","doc":"Beacon's representation of Phoenix's [Slot attributes](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#module-slot-attributes).\n\nComponentSlotAttrs don't exist on their own, but always belong to a `Beacon.Content.ComponentSlot`, which in turn\nbelongs to a `Beacon.Content.Component`.\n\n> #### Do not create or edit component slot attrs manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.ComponentSlotAttr.html"},{"type":"type","title":"Beacon.Content.ComponentSlotAttr.t/0","doc":"","ref":"Beacon.Content.ComponentSlotAttr.html#t:t/0"},{"type":"module","title":"Beacon.Content.ErrorPage","doc":"Stores a template which can be rendered for error responses.\n\nAn ErrorPage contains four main fields:\n\n  * `:status` - the status code for which this ErrorPage is to be used\n  * `:template` - the template to be rendered\n  * `:site` - the Beacon site which should use this page\n  * `:layout_id` - the ID of the Beacon Layout which is used for rendering\n\n> #### Do not create or edit ErrorPages manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.ErrorPage.html"},{"type":"type","title":"Beacon.Content.ErrorPage.error_status/0","doc":"","ref":"Beacon.Content.ErrorPage.html#t:error_status/0"},{"type":"type","title":"Beacon.Content.ErrorPage.t/0","doc":"","ref":"Beacon.Content.ErrorPage.html#t:t/0"},{"type":"function","title":"Beacon.Content.ErrorPage.valid_statuses/0","doc":"","ref":"Beacon.Content.ErrorPage.html#valid_statuses/0"},{"type":"module","title":"Beacon.Content.EventHandler","doc":"Beacon's representation of a LiveView [handle_event/3](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#c:handle_event/3).\n\nThis is the Elixir code which will receive form submission or on-click events.\n\n> #### Do not create or edit page event handlers manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.EventHandler.html"},{"type":"type","title":"Beacon.Content.EventHandler.t/0","doc":"","ref":"Beacon.Content.EventHandler.html#t:t/0"},{"type":"module","title":"Beacon.Content.InfoHandler","doc":"Beacon's representation of a LiveView [handle_info/2](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#c:handle_info/2)\nthat applies to all of a site's pages.\n\nThis is the Elixir code which will handle messages from other Elixir processes.\n\n> #### Do not create or edit info handlers manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.InfoHandler.html"},{"type":"type","title":"Beacon.Content.InfoHandler.t/0","doc":"","ref":"Beacon.Content.InfoHandler.html#t:t/0"},{"type":"module","title":"Beacon.Content.JSHook","doc":"Stores a JS Hook which can be referenced from your Beacon pages, layouts, and components.\n\n> #### Do not create or edit JS Hooks manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.JSHook.html"},{"type":"type","title":"Beacon.Content.JSHook.t/0","doc":"","ref":"Beacon.Content.JSHook.html#t:t/0"},{"type":"module","title":"Beacon.Content.Layout","doc":"The wrapper content for a `Beacon.Content.Page`.\n\n> #### Do not create or layouts pages manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.Layout.html"},{"type":"type","title":"Beacon.Content.Layout.t/0","doc":"","ref":"Beacon.Content.Layout.html#t:t/0"},{"type":"function","title":"Beacon.Content.Layout.version/0","doc":"Current data structure version.\n\nBump when schema changes.","ref":"Beacon.Content.Layout.html#version/0"},{"type":"module","title":"Beacon.Content.LayoutEvent","doc":"Represents a point in time when a `Beacon.Content.Layout` was created or published.\n\nAt that time, a `Beacon.Content.LayoutSnapshot` is created and stored with the LayoutEvent.\n\n> #### Do not create or edit layout events manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.LayoutEvent.html"},{"type":"type","title":"Beacon.Content.LayoutEvent.t/0","doc":"","ref":"Beacon.Content.LayoutEvent.html#t:t/0"},{"type":"module","title":"Beacon.Content.LayoutSnapshot","doc":"Represents the template of a `Beacon.Content.Layout` at a specific moment in time.\n\nLayoutSnapshots don't exist on their own, but are created as part of a `Beacon.Content.LayoutEvent`\nwhenever a Layout is created or published.\n\n> #### Do not create or edit layout snapshots manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.LayoutSnapshot.html"},{"type":"type","title":"Beacon.Content.LayoutSnapshot.t/0","doc":"","ref":"Beacon.Content.LayoutSnapshot.html#t:t/0"},{"type":"module","title":"Beacon.Content.LiveData","doc":"Scopes `Beacon.Content.LiveDataAssign`s to `Beacon.Content.Page`s via site and path.\n\n> #### Do not create or edit live data manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.LiveData.html"},{"type":"function","title":"Beacon.Content.LiveData.changeset/2","doc":"","ref":"Beacon.Content.LiveData.html#changeset/2"},{"type":"function","title":"Beacon.Content.LiveData.path_changeset/2","doc":"","ref":"Beacon.Content.LiveData.html#path_changeset/2"},{"type":"type","title":"Beacon.Content.LiveData.t/0","doc":"","ref":"Beacon.Content.LiveData.html#t:t/0"},{"type":"module","title":"Beacon.Content.LiveDataAssign","doc":"Dynamic key/value assigns to be used by `Beacon.Content.Page` templates and updated with `Beacon.Content.EventHandler`s.\n\nLiveDataAssigns don't exist on their own, but exist as part of a `Beacon.Content.LiveData` struct.\n\n> #### Do not create or edit live data manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.LiveDataAssign.html"},{"type":"function","title":"Beacon.Content.LiveDataAssign.changeset/2","doc":"","ref":"Beacon.Content.LiveDataAssign.html#changeset/2"},{"type":"function","title":"Beacon.Content.LiveDataAssign.formats/0","doc":"","ref":"Beacon.Content.LiveDataAssign.html#formats/0"},{"type":"type","title":"Beacon.Content.LiveDataAssign.t/0","doc":"","ref":"Beacon.Content.LiveDataAssign.html#t:t/0"},{"type":"module","title":"Beacon.Content.Page","doc":"Pages are the central piece of content in Beacon used to render templates with meta tags, components, and other resources.\n\nPages are rendered as a LiveView handled by Beacon.\n\nPages can be extended with custom fields, see `Beacon.Content.PageField`","ref":"Beacon.Content.Page.html"},{"type":"module","title":"SEO - Beacon.Content.Page","doc":"Meta Tags\n\nRaw Schema\n\n> #### Do not create or edit pages manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.Page.html#module-seo"},{"type":"type","title":"Beacon.Content.Page.t/0","doc":"","ref":"Beacon.Content.Page.html#t:t/0"},{"type":"function","title":"Beacon.Content.Page.version/0","doc":"Current data structure version.\n\nBump when schema changes.","ref":"Beacon.Content.Page.html#version/0"},{"type":"module","title":"Beacon.Content.PageEvent","doc":"Represents a point in time when a `Beacon.Content.Page` was created, published, or unpublished.\n\nAt that time, a `Beacon.Content.PageSnapshot` is created and stored with the PageEvent.\n\n> #### Do not create or edit page events manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.PageEvent.html"},{"type":"type","title":"Beacon.Content.PageEvent.event/0","doc":"The event name. Can be one of :created, :published, :unpublished","ref":"Beacon.Content.PageEvent.html#t:event/0"},{"type":"type","title":"Beacon.Content.PageEvent.t/0","doc":"","ref":"Beacon.Content.PageEvent.html#t:t/0"},{"type":"module","title":"Beacon.Content.PageSnapshot","doc":"Represents the template of a `Beacon.Content.Page` at a specific moment in time.\n\nPageSnapshots don't exist on their own, but are created as part of a `Beacon.Content.PageEvent`\nwhenever a Page is created, published, or unpublished.\n\n> #### Do not create or edit page snapshots manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.PageSnapshot.html"},{"type":"type","title":"Beacon.Content.PageSnapshot.t/0","doc":"","ref":"Beacon.Content.PageSnapshot.html#t:t/0"},{"type":"module","title":"Beacon.Content.PageVariant","doc":"Stores an alternate template which can be randomly rendered as a replacement for a Page's standard template.\n\nA PageVariant contains three main fields:\n\n  * `:name` - a brief description of what change(s) the variant template provides\n  * `:weight` - the percentage of page renders which should use this template (0 - 100)\n  * `:template` - the template which should be rendered whenever this variant is used\n\n> #### Do not create or edit page variants manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.PageVariant.html"},{"type":"type","title":"Beacon.Content.PageVariant.t/0","doc":"","ref":"Beacon.Content.PageVariant.html#t:t/0"},{"type":"module","title":"Beacon.Content.Snippets.Helper","doc":"Snippet Helpers are blocks of custom Elixir code which can be called from\nmeta tags and various other page fields.\n\nIn addition to being scoped by `:site`, it stores a `:name` with which to call it\nand a `:body` with the code block to run.\n\nSee `Beacon.Content.render_snippet/2` for an example of usage.\n\n> #### Do not create or edit snippet helpers manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.Snippets.Helper.html"},{"type":"type","title":"Beacon.Content.Snippets.Helper.t/0","doc":"","ref":"Beacon.Content.Snippets.Helper.html#t:t/0"},{"type":"module","title":"Beacon.Content.Stylesheet","doc":"Stores CSS content to use when rendering pages with Beacon.\n\n> #### Do not create or edit stylesheets manually {: .warning}\n>\n> Use the public functions in `Beacon.Content` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.Content.Stylesheet.html"},{"type":"type","title":"Beacon.Content.Stylesheet.t/0","doc":"","ref":"Beacon.Content.Stylesheet.html#t:t/0"},{"type":"module","title":"Beacon.Template","doc":"Template for layouts, pages, and any other resource that display HTML/HEEx.\n\nTemplates are defined as [Phoenix.LiveView.Rendered](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.Rendered.html) structs,\nwhich holds nested static literal strings and also dynamic content for the LiveView engine.\n\nTemplate engines that do not support dynamic content can make use of the `:static` field to store its contents.","ref":"Beacon.Template.html"},{"type":"type","title":"Beacon.Template.ast/0","doc":"The AST representation of a `t:Phoenix.LiveView.Rendered.t/0` struct.","ref":"Beacon.Template.html#t:ast/0"},{"type":"type","title":"Beacon.Template.t/0","doc":"","ref":"Beacon.Template.html#t:t/0"},{"type":"module","title":"Beacon.Template.HEEx","doc":"Handles loading and compilation of HEEx templates.","ref":"Beacon.Template.HEEx.html"},{"type":"function","title":"Beacon.Template.HEEx.compile/4","doc":"Compile `template` returning its AST.","ref":"Beacon.Template.HEEx.html#compile/4"},{"type":"function","title":"Beacon.Template.HEEx.compile!/4","doc":"","ref":"Beacon.Template.HEEx.html#compile!/4"},{"type":"function","title":"Beacon.Template.HEEx.render/3","doc":"Renders the HEEx `template` with `assigns`.\n\n> #### Use only to render isolated templates {: .warning}\n>\n> This function should not be used to render Page Templates,\n> its purpose is only to render isolated pieces of templates.","ref":"Beacon.Template.HEEx.html#render/3"},{"type":"function","title":"Example - Beacon.Template.HEEx.render/3","doc":"iex> Beacon.Template.HEEx.render(:my_site, ~S|<.link patch=\"/contact\" replace={true}><%= @text %></.link>|, %{text: \"Book Meeting\"})\n    \" Book Meeting \"","ref":"Beacon.Template.HEEx.html#render/3-example"},{"type":"module","title":"Beacon.Template.Markdown","doc":"GitHub Flavored Markdown\n\nUses https://github.com/leandrocp/mdex to convert Markdown to HTML","ref":"Beacon.Template.Markdown.html"},{"type":"function","title":"Beacon.Template.Markdown.convert_to_html/3","doc":"Convert a markdown `template` into HTML using [mdex](https://hex.pm/packages/mdex)","ref":"Beacon.Template.Markdown.html#convert_to_html/3"},{"type":"function","title":"Options - Beacon.Template.Markdown.convert_to_html/3","doc":"* `:syntax_highlight_theme` (default `\"onedark\"`) - see https://hexdocs.pm/mdex/MDEx.html#to_html/2 for more info.","ref":"Beacon.Template.Markdown.html#convert_to_html/3-options"},{"type":"module","title":"Beacon.MediaLibrary","doc":"Provides functions to upload and serve assets.","ref":"Beacon.MediaLibrary.html"},{"type":"function","title":"Beacon.MediaLibrary.change_asset/2","doc":"Returns an `%Ecto.Changeset{}` for tracking asset changes.","ref":"Beacon.MediaLibrary.html#change_asset/2"},{"type":"function","title":"Example - Beacon.MediaLibrary.change_asset/2","doc":"iex> change_asset(asset)\n    %Ecto.Changeset{data: %Asset{}}","ref":"Beacon.MediaLibrary.html#change_asset/2-example"},{"type":"function","title":"Beacon.MediaLibrary.change_derivation/2","doc":"Returns an `%Ecto.Changeset{}` for updating an Asset's `:usage_tag` and/or `:source_id`.","ref":"Beacon.MediaLibrary.html#change_derivation/2"},{"type":"function","title":"Example - Beacon.MediaLibrary.change_derivation/2","doc":"iex> change_asset(asset)\n    %Ecto.Changeset{data: %Asset{}}","ref":"Beacon.MediaLibrary.html#change_derivation/2-example"},{"type":"function","title":"Beacon.MediaLibrary.count_assets/2","doc":"Counts the total number of assets based on the amount of pages.","ref":"Beacon.MediaLibrary.html#count_assets/2"},{"type":"function","title":"Options - Beacon.MediaLibrary.count_assets/2","doc":"* `:query` - filter rows count by query. Defaults to `nil`, doesn't filter query.","ref":"Beacon.MediaLibrary.html#count_assets/2-options"},{"type":"function","title":"Beacon.MediaLibrary.file_stat/2","doc":"Returns information about the uploaded file.\n\nSee `File.stat/2` for more information.","ref":"Beacon.MediaLibrary.html#file_stat/2"},{"type":"function","title":"Beacon.MediaLibrary.get_asset_by/2","doc":"Gets a single asset by `clauses`.","ref":"Beacon.MediaLibrary.html#get_asset_by/2"},{"type":"function","title":"Example - Beacon.MediaLibrary.get_asset_by/2","doc":"iex> get_asset_by(site, file_name: \"logo.webp\")\n    %Asset{}","ref":"Beacon.MediaLibrary.html#get_asset_by/2-example"},{"type":"function","title":"Beacon.MediaLibrary.is_image?/1","doc":"Returns true if the given Asset is an image.\n\nAccepted filetypes are `.jpg .jpeg .png .gif .bmp .tif .tiff .webp`","ref":"Beacon.MediaLibrary.html#is_image?/1"},{"type":"function","title":"Beacon.MediaLibrary.list_assets/2","doc":"Returns the list of all uploaded assets of `site`.","ref":"Beacon.MediaLibrary.html#list_assets/2"},{"type":"function","title":"Options - Beacon.MediaLibrary.list_assets/2","doc":"* `:per_page` - limit how many records are returned, or pass `:infinity` to return all records. Defaults to 20.\n  * `:page` - returns records from a specific page. Defaults to 1.\n  * `:query` - search assets by file name. Defaults to `nil`, doesn't filter query.\n  * `:preloads` - a list of preloads to load. Defaults to `[:thumbnail]`.\n  * `:sort` - column in which the result will be ordered by. Defaults to `:file_name`.\n    Allowed values: `:file_name`, `:media_type`, `:inserted_at`, `:updated_at`.","ref":"Beacon.MediaLibrary.html#list_assets/2-options"},{"type":"function","title":"Beacon.MediaLibrary.read_binary/1","doc":"Returns the contents of an uploaded file as a binary.","ref":"Beacon.MediaLibrary.html#read_binary/1"},{"type":"function","title":"Beacon.MediaLibrary.read_binary!/1","doc":"Same as `read_binary/1` but raises if the file is not found.","ref":"Beacon.MediaLibrary.html#read_binary!/1"},{"type":"function","title":"Beacon.MediaLibrary.save_asset/1","doc":"This functions runs only the local persistence step of `upload/1`.","ref":"Beacon.MediaLibrary.html#save_asset/1"},{"type":"function","title":"Beacon.MediaLibrary.save_asset!/1","doc":"Same as `save_asset/1` but raises an error if unsuccessful.","ref":"Beacon.MediaLibrary.html#save_asset!/1"},{"type":"function","title":"Beacon.MediaLibrary.search/2","doc":"Search assets by file name.","ref":"Beacon.MediaLibrary.html#search/2"},{"type":"function","title":"Beacon.MediaLibrary.send_to_cdns/1","doc":"This functions runs only the external upload step of `upload/1`.","ref":"Beacon.MediaLibrary.html#send_to_cdns/1"},{"type":"function","title":"Beacon.MediaLibrary.soft_delete/1","doc":"Soft deletes a asset.","ref":"Beacon.MediaLibrary.html#soft_delete/1"},{"type":"function","title":"Example - Beacon.MediaLibrary.soft_delete/1","doc":"iex> soft_delete(asset)\n    {:ok, %Asset{}}","ref":"Beacon.MediaLibrary.html#soft_delete/1-example"},{"type":"function","title":"Beacon.MediaLibrary.srcset_for_image/2","doc":"For a given asset and list of acceptable usage tags, returns a [srcset](https://developer.mozilla.org/en-US/docs/Web/API/HTMLImageElement/srcset) for use in templates.","ref":"Beacon.MediaLibrary.html#srcset_for_image/2"},{"type":"function","title":"Beacon.MediaLibrary.upload/1","doc":"Uploads a given `UploadMetadata` and runs the `:upload_asset` lifecycle.\n\nRuns multiple steps:\n\n  * Upload to external service (see: `Beacon.MediaLibrary.Provider`)\n  * Persist the metadata to the local database\n  * Run the `:upload_asset` lifecycle (see: `t:Beacon.Config.lifecycle_stage/0`)","ref":"Beacon.MediaLibrary.html#upload/1"},{"type":"function","title":"Beacon.MediaLibrary.url_for/1","doc":"Returns the URL of the first registered provider for a given `asset`\n\nNote that if multiple URLs exist due to various providers, only the first will be returned.","ref":"Beacon.MediaLibrary.html#url_for/1"},{"type":"function","title":"Beacon.MediaLibrary.url_for/2","doc":"Returns the URL for the given `asset` registered on provider under `provider_key`.","ref":"Beacon.MediaLibrary.html#url_for/2"},{"type":"function","title":"Example - Beacon.MediaLibrary.url_for/2","doc":"Given a site config with a registered provider:\n\n```elixir\n[\n  site: :my_site,\n  assets: [\n    {\"image/*\", [providers: [MyApp.Provider.CDN], validations: []]}\n  ],\n  ...\n]\n```\n\nAnd that provider registers itself under the `\"cnd\"` key:\n\n```elixir\ndefmodule MyApp.Provider.CDN do\n  def provider_key, do: \"cdn\"\nend\n```\n\nThen you can fetch the URL from that provider with:\n\n    iex> url_for(asset, \"cdn\")\n    \"https://assets.mysite.com/logo-361ae598-4150-4ce5-814d-363c2afd1993.webp\"","ref":"Beacon.MediaLibrary.html#url_for/2-example"},{"type":"function","title":"Beacon.MediaLibrary.url_for_asset/2","doc":"Returns the URL of a media `file_name` previously uploaded to the Media Library.\n\nThis function is a convenience for `get_asset_by/2` and `url_for/1`,\nsee their docs for more info.","ref":"Beacon.MediaLibrary.html#url_for_asset/2"},{"type":"function","title":"Example - Beacon.MediaLibrary.url_for_asset/2","doc":"iex> url_for_asset(:my_site, \"logo.webp\")\n    \"https://mysite.com/__beacon_media__/logo.webp\"","ref":"Beacon.MediaLibrary.html#url_for_asset/2-example"},{"type":"function","title":"Beacon.MediaLibrary.urls_for/1","doc":"Returns a list of all URLs to the given Asset.\n\nThe number of URLs depends on how many providers are configured for the media type.","ref":"Beacon.MediaLibrary.html#urls_for/1"},{"type":"module","title":"Beacon.MediaLibrary.Asset","doc":"Assets are the images, videos, and any other media type uploaded and served by the Media Library.\n\n> #### Do not create or edit assets manually {: .warning}\n>\n> Use the public functions in `Beacon.MediaLibrary` instead.\n> The functions in that module guarantee that all dependencies\n> are created correctly and all processes are updated.\n> Manipulating data manually will most likely result\n> in inconsistent behavior and crashes.","ref":"Beacon.MediaLibrary.Asset.html"},{"type":"type","title":"Beacon.MediaLibrary.Asset.t/0","doc":"","ref":"Beacon.MediaLibrary.Asset.html#t:t/0"},{"type":"module","title":"Beacon.MediaLibrary.Provider","doc":"Connects to an external service to store and serve assets in the Media Library.\n\nBy default, Beacon comes with 2 providers:\n\n  * `Beacon.MediaLibrary.Provider.Repo` - Store assets in the database.\n  * `Beacon.MediaLibrary.Provider.S3` - Store assets in an S3 bucket.\n\n`Repo` is enabled by default for all media types since a database connection is already\navailable, and caching is used to reduce overload on the database. But you can switch to `S3`\nusing the `Beacon.MediaLibrary.Provider.S3` provider or create your own provider.\n\nSee `Beacon.Config` and each provider module doc for more info.","ref":"Beacon.MediaLibrary.Provider.html"},{"type":"module","title":"Beacon.MediaLibrary.Provider.Repo","doc":"Store assets in the database.\n\nFiles are stored as binaries (BLOB)","ref":"Beacon.MediaLibrary.Provider.Repo.html"},{"type":"module","title":"Beacon.MediaLibrary.Provider.S3","doc":"Store assets in S3 using the `ex_aws` library.\n\nAll files are stored in the same bucket under the same level,\nand the bucket name must be present in the `:ex_aws` config:\n\n    config :ex_aws, s3: [bucket: \"my_bucket\"]\n\nAnd then in your site config, register this provider for each mime type you want to use it:\n\n    assets: [\n      {\"image/*\", [providers: [Beacon.MediaLibrary.Provider.S3]]}\n    ]","ref":"Beacon.MediaLibrary.Provider.S3.html"},{"type":"module","title":"Beacon.MediaLibrary.UploadMetadata","doc":"Metadata passed to page rendering lifecycle.","ref":"Beacon.MediaLibrary.UploadMetadata.html"},{"type":"type","title":"Beacon.MediaLibrary.UploadMetadata.t/0","doc":"","ref":"Beacon.MediaLibrary.UploadMetadata.html#t:t/0"},{"type":"behaviour","title":"Beacon.RuntimeCSS","doc":"Compiles the CSS for a site using the compiler defined in `t:Beacon.Config.css_compiler/0`\n\nBeacon supports Tailwind by default implemented by `Beacon.RuntimeCSS.TailwindCompiler`,\nyou can use that module as template to implement any ther CSS engine as needed.","ref":"Beacon.RuntimeCSS.html"},{"type":"callback","title":"Beacon.RuntimeCSS.compile/1","doc":"Executes the compilation to generate the CSS for the site using the provided `:css_compiler` in `Beacon.Config`.","ref":"Beacon.RuntimeCSS.html#c:compile/1"},{"type":"callback","title":"Beacon.RuntimeCSS.config/1","doc":"Returns the CSS compiler config.\n\nFor Tailwind that would be the content of the tailwind config file,\nor return an empty string `\"\"` if the provided engine doesn't have a config file.","ref":"Beacon.RuntimeCSS.html#c:config/1"},{"type":"function","title":"Beacon.RuntimeCSS.css_config_url/1","doc":"Returns the URL to fetch the CSS config used to generate the site stylesheet.","ref":"Beacon.RuntimeCSS.html#css_config_url/1"},{"type":"module","title":"Beacon.RuntimeCSS.TailwindCompiler","doc":"Tailwind compiler for runtime CSS, used on all sites.\n\nThe default configuration is fetched from `Path.join(Application.app_dir(:beacon, \"priv\"), \"tailwind.config.js\")`,\nyou can see the actual file at https://github.com/BeaconCMS/beacon/blob/main/priv/tailwind.config.js\n\nThat's the file used by default if no value is provided in the site configuration `t:Beacon.Config.tailwind_config/0`\n\nYou can use any of the [available options from Tailwind CSS](https://tailwindcss.com/docs/configuration) but\nyou must bundle the file if using Plugins or requiring any external module, see the [Tailwind Setup guide](https://hexdocs.pm/beacon/tailwind-setup.html)\nfor more info and examples.","ref":"Beacon.RuntimeCSS.TailwindCompiler.html"},{"type":"module","title":"Beacon.Web.BeaconAssigns","doc":"Read-only container of Beacon assigns related to the current page.\n\nAvailable as `@beacon`\n\nThis data can be accessed in page templates, event handlers, and other parts of your site\nto read information about the current page as the current site, path and query params,\npath and title.\n\nMore fields may be added in the future.","ref":"Beacon.Web.BeaconAssigns.html"},{"type":"module","title":"Examples - Beacon.Web.BeaconAssigns","doc":"In a template:\n\n     <%= @beacon.page.title %> \n\nIn a event handler:\n\n    pages = Beacon.Content.list_published_pages(@beacon.site)","ref":"Beacon.Web.BeaconAssigns.html#module-examples"},{"type":"type","title":"Beacon.Web.BeaconAssigns.t/0","doc":"","ref":"Beacon.Web.BeaconAssigns.html#t:t/0"},{"type":"module","title":"Beacon.Web.ErrorHTML","doc":"Render `Beacon.Content.ErrorPage`.","ref":"Beacon.Web.ErrorHTML.html"},{"type":"module","title":"Beacon.Web.Layouts","doc":"Core layouts.\n\nThese functions are mostly used internally by Beacon but you can override the\nroot layout in `beacon_site` so you should use the functions in this\nmodule to properly build your custom root layout to avoid breaking\nBeacon functionality.\n\nSee https://github.com/BeaconCMS/beacon/blob/main/lib/beacon/web/components/layouts/runtime.html.heex for reference.","ref":"Beacon.Web.Layouts.html"},{"type":"function","title":"Beacon.Web.Layouts.app/1","doc":"","ref":"Beacon.Web.Layouts.html#app/1"},{"type":"function","title":"Beacon.Web.Layouts.asset_path/2","doc":"Returns the path to the route that serves CSS and JS assets.","ref":"Beacon.Web.Layouts.html#asset_path/2"},{"type":"function","title":"Beacon.Web.Layouts.dynamic/1","doc":"","ref":"Beacon.Web.Layouts.html#dynamic/1"},{"type":"function","title":"Beacon.Web.Layouts.live_socket_path/1","doc":"Returns the path to the live socket defined in the site configuration.","ref":"Beacon.Web.Layouts.html#live_socket_path/1"},{"type":"function","title":"Beacon.Web.Layouts.render_meta_tags/1","doc":"Renders all ` ` tags defined in the current page.","ref":"Beacon.Web.Layouts.html#render_meta_tags/1"},{"type":"function","title":"Beacon.Web.Layouts.render_page_title/1","doc":"Returns the resolved page title.\n\nPage titles may use snippets to render dynamic content.\nThis function will resolve such snippets.","ref":"Beacon.Web.Layouts.html#render_page_title/1"},{"type":"function","title":"Beacon.Web.Layouts.render_resource_links/1","doc":"Renders all resource ` ` tags defined in the current layout.","ref":"Beacon.Web.Layouts.html#render_resource_links/1"},{"type":"function","title":"Beacon.Web.Layouts.render_schema/1","doc":"Renders the Schema.org data defined in the current page.","ref":"Beacon.Web.Layouts.html#render_schema/1"},{"type":"function","title":"Beacon.Web.Layouts.runtime/1","doc":"","ref":"Beacon.Web.Layouts.html#runtime/1"},{"type":"module","title":"Beacon.Config","doc":"Configuration for sites.\n\nEach site is started with this configuration and its values are stored in a registry that can be fetched at runtime\nor updated with `update_value/3`.\n\nSee `new/1` for available options and examples.","ref":"Beacon.Config.html"},{"type":"type","title":"Beacon.Config.allowed_media_accept_types/0","doc":"Register specific media types allowed for upload. Catchalls are not allowed.","ref":"Beacon.Config.html#t:allowed_media_accept_types/0"},{"type":"function","title":"Beacon.Config.config_for_media_type/2","doc":"From a `Beacon.Config`, fetch the asset config for a given media type, raising when unsuccessful.","ref":"Beacon.Config.html#config_for_media_type/2"},{"type":"function","title":"Example - Beacon.Config.config_for_media_type/2","doc":"iex> beacon_config = Beacon.Config.fetch!(:some_site)\n    iex> jpeg_config = config_for_media_type(beacon_config, \"image/jpeg\")","ref":"Beacon.Config.html#config_for_media_type/2-example"},{"type":"type","title":"Beacon.Config.css_compiler/0","doc":"A module that implements `Beacon.RuntimeCSS`.","ref":"Beacon.Config.html#t:css_compiler/0"},{"type":"type","title":"Beacon.Config.default_meta_tags/0","doc":"Default meta tags added to new pages.","ref":"Beacon.Config.html#t:default_meta_tags/0"},{"type":"type","title":"Beacon.Config.endpoint/0","doc":"Host application Endpoint module.","ref":"Beacon.Config.html#t:endpoint/0"},{"type":"type","title":"Beacon.Config.extra_asset_field/0","doc":"Add extra fields to pages.","ref":"Beacon.Config.html#t:extra_asset_field/0"},{"type":"type","title":"Beacon.Config.extra_asset_fields/0","doc":"Add extra fields to pages.","ref":"Beacon.Config.html#t:extra_asset_fields/0"},{"type":"type","title":"Beacon.Config.extra_page_fields/0","doc":"Add extra fields to pages.","ref":"Beacon.Config.html#t:extra_page_fields/0"},{"type":"function","title":"Beacon.Config.fetch!/1","doc":"Returns the `Beacon.Config` for `site`.","ref":"Beacon.Config.html#fetch!/1"},{"type":"function","title":"Beacon.Config.get_media_type_config/2","doc":"Searches a config option for the given media type.\n\nFor config options based on media type, such as `:assets` and `:extra_asset_fields`,\nthis function will check for the presence of `media_type`, returning the config for\nthat specific type, or `nil` if the type is not present.","ref":"Beacon.Config.html#get_media_type_config/2"},{"type":"function","title":"Examples - Beacon.Config.get_media_type_config/2","doc":"iex> beacon_config = Beacon.Config.fetch!(:some_site)\n    iex> jpeg_config = config_for_media_type(beacon_config.assets, \"image/jpeg\")\n\n    iex> beacon_config = Beacon.Config.fetch!(:some_site)\n    iex> webp_config = config_for_media_type(beacon_config.extra_asset_fields, \"image/webp\")\n\n    iex> beacon_config = Beacon.Config.fetch!(:some_site)\n    iex> nil = config_for_media_type(beacon_config.assets, \"invalid/foo\")","ref":"Beacon.Config.html#get_media_type_config/2-examples"},{"type":"type","title":"Beacon.Config.lifecycle/0","doc":"Attach steps into Beacon's internal life-cycle stages to inject custom functionality.","ref":"Beacon.Config.html#t:lifecycle/0"},{"type":"type","title":"Beacon.Config.lifecycle_stage/0","doc":"Life-cycle stages.","ref":"Beacon.Config.html#t:lifecycle_stage/0"},{"type":"type","title":"Beacon.Config.live_socket_path/0","doc":"Path of a LiveView socket where Beacon should connect to.","ref":"Beacon.Config.html#t:live_socket_path/0"},{"type":"type","title":"Beacon.Config.media_type_config/0","doc":"Individual media type configs","ref":"Beacon.Config.html#t:media_type_config/0"},{"type":"type","title":"Beacon.Config.media_type_configs/0","doc":"Register providers and validations for media types. Catchalls are allowed.","ref":"Beacon.Config.html#t:media_type_configs/0"},{"type":"type","title":"Beacon.Config.mode/0","doc":"Defines the mode which the site will operate.\n\nDefault is `:live` which will load resources during boot, broadcast events on content change,\nand execute operations asyncly. That's the normal mode for production.\n\nThe `:testing` mode is suited for testing environments,\nyou should always use it when running tests that involve Beacon resources.\n\nAnd the `:manual` mode is similar to `:testing` but it won't boot load any resource,\nit's useful to seed data.\n\nYou can always change to `:live` mode at runtime by calling `Beacon.boot/1`.","ref":"Beacon.Config.html#t:mode/0"},{"type":"function","title":"Beacon.Config.new/1","doc":"Build a new `%Beacon.Config{}` instance to hold the entire configuration for each site.","ref":"Beacon.Config.html#new/1"},{"type":"function","title":"Options - Beacon.Config.new/1","doc":"* `:site` - `t:Beacon.Types.Site.t/0` (required)\n\n  * `:endpoint` - `t:endpoint/0` (required)\n\n  * `:router` - `t:router/0` (required)\n\n  * `:repo` - `t:repo/0` (required)\n\n  * `:mode` - `t:mode/0` (optional). Defaults to `:live`.\n\n  * `css_compiler` - `t:css_compiler/0` (optional). Defaults to `Beacon.RuntimeCSS.TailwindCompiler`.\n\n  * `:tailwind_config` - `t:tailwind_config/0` (optional). Defaults to `Path.join(Application.app_dir(:beacon, \"priv\"), \"tailwind.config.bundle.js\")`.\n\n  * `:tailwind_css` - `t:tailwind_css/0` (optional). Defaults to `Path.join(Application.app_dir(:beacon, \"priv\"), \"tailwind.css\")`.\n\n  * `:live_socket_path` - `t:live_socket_path/0` (optional). Defaults to `\"/live\"`.\n\n  * `:safe_code_check` - `t:safe_code_check/0` (optional). Defaults to `false`.\n\n  * `:template_formats` - `t:template_formats/0` (optional).\n\n      Defaults to:\n\n            [\n              {:heex, \"HEEx (HTML)\"},\n              {:markdown, \"Markdown (GitHub Flavored version)\"}\n            ]\n\n  Note that the default config is merged with your config.\n\n  * `lifecycle` - `t:lifecycle/0` (optional).\n\n  Note that the default config is merged with your config.\n\n  * `:extra_page_fields` - `t:extra_page_fields/0` (optional). Defaults to `[]`.\n\n  * `:extra_asset_fields` - `t:extra_asset_fields/0` (optional). Defaults to `[]`.\n\n  * `:default_meta_tags` - `t:default_meta_tags/0` (optional). Defaults to `%{}`.\n\n  * `:page_warming` - `t:page_warming/0` (optional). Defaults to `{:shortest_paths, 10}`.","ref":"Beacon.Config.html#new/1-options"},{"type":"function","title":"Example - Beacon.Config.new/1","doc":"iex> Beacon.Config.new(\n      site: :my_site,\n      endpoint: MyAppWeb.Endpoint,\n      router: MyAppWeb.Router,\n      repo: MyApp.Repo,\n      tailwind_config: Path.join(Application.app_dir(:my_app, \"priv\"), \"tailwind.config.js\"),\n      tailwind_css: Path.join([Application.app_dir(:my_app, \"assets\"), \"css\", \"app.css\"]),\n      template_formats: [\n        {:custom_format, \"My Custom Format\"}\n      ],\n      lifecycle: [\n        load_template: [\n          {:custom_format,\n           [\n             validate: fn template, _metadata -> MyEngine.validate(template) end\n           ]}\n        ],\n        render_template: [\n          {:custom_format,\n           [\n             assigns: fn %Phoenix.LiveView.Rendered{static: template} , %{assigns: assigns} -> MyEngine.parse_to_html(template, assigns) end\n           ]}\n        ],\n        after_publish_page: [\n          notify_admin: fn page -> {:cont, MyApp.Admin.send_email(page)} end\n        ]\n      ],\n      page_warming: {:specify_paths, [\"/\", \"/home\", \"/blog\"]}\n    )\n    %Beacon.Config{\n      site: :my_site,\n      endpoint: MyAppWeb.Endpoint,\n      router: MyAppWeb.Router,\n      repo: MyApp.Repo,\n      mode: :live,\n      css_compiler: Beacon.RuntimeCSS.TailwindCompiler,\n      tailwind_config: \"/my_app/priv/tailwind.config.js\",\n      tailwind_css: \"/my_app/assets/css/app.css\",\n      live_socket_path: \"/live\",\n      safe_code_check: false,\n      template_formats: [\n        heex: \"HEEx (HTML)\",\n        markdown: \"Markdown (GitHub Flavored version)\",\n        custom_format: \"My Custom Format\"\n      ],\n      media_types: [\"image/jpeg\", \"image/gif\", \"image/png\", \"image/webp\"],\n      assets:[\n        {\"image/*\", [providers: [Beacon.MediaLibrary.Provider.Repo], validations: [&SomeModule.some_function/2]]},\n      ],\n      lifecycle: [\n        load_template: [\n          heex: [],\n          markdown: [\n            convert_to_html: &Beacon.Template.Markdown.convert_to_html/2,\n          ],\n          custom_format: [\n            validate: #Function<41.3316493/2 in :erl_eval.expr/6>\n          ]\n        ],\n        render_template: [\n          heex: [],\n          markdown: [],\n          custom_format: [\n            assigns: #Function<41.3316493/2 in :erl_eval.expr/6>\n          ]\n        ],\n        after_create_page: [],\n        after_update_page: [],\n        after_publish_page: [\n          notify_admin: #Function<42.3316493/1 in :erl_eval.expr/6>\n        ],\n        after_unpublish_page: [],\n        upload_asset: [],\n      ],\n      extra_page_fields: [],\n      extra_asset_fields: [],\n      default_meta_tags: [],\n      page_warming: {:specify_paths, [\"/\", \"/home\", \"/blog\"]}\n    }","ref":"Beacon.Config.html#new/1-example"},{"type":"type","title":"Beacon.Config.option/0","doc":"","ref":"Beacon.Config.html#t:option/0"},{"type":"type","title":"Beacon.Config.page_warming/0","doc":"The strategy for pre-loading page modules at boot time.","ref":"Beacon.Config.html#t:page_warming/0"},{"type":"type","title":"Beacon.Config.repo/0","doc":"Host application Repo module.","ref":"Beacon.Config.html#t:repo/0"},{"type":"type","title":"Beacon.Config.router/0","doc":"Host application Router module.","ref":"Beacon.Config.html#t:router/0"},{"type":"type","title":"Beacon.Config.safe_code_check/0","doc":"Check safety of Elixir code using https://github.com/TheFirstAvenger/safe_code","ref":"Beacon.Config.html#t:safe_code_check/0"},{"type":"type","title":"Beacon.Config.t/0","doc":"","ref":"Beacon.Config.html#t:t/0"},{"type":"type","title":"Beacon.Config.tailwind_config/0","doc":"Path to a custom Tailwind config.","ref":"Beacon.Config.html#t:tailwind_config/0"},{"type":"type","title":"Example - Beacon.Config.tailwind_config/0","doc":"# use the config file `priv/tailwind.config.js` from your app named `my_app`\n    Path.join(Application.app_dir(:my_app, \"priv\"), \"tailwind.config.js\")\n\nSee `Beacon.RuntimeCSS.TailwindCompiler` for more info.","ref":"Beacon.Config.html#t:tailwind_config/0-example"},{"type":"type","title":"Beacon.Config.tailwind_css/0","doc":"Path to a custom Tailwind CSS\n\nNote that Tailwind base, components, and utilities must be imported in this file.","ref":"Beacon.Config.html#t:tailwind_css/0"},{"type":"type","title":"Example - Beacon.Config.tailwind_css/0","doc":"# use the file `assets/css/app.css` from your app named `my_app`\n    Path.join([Application.app_dir(:my_app, \"assets\"), \"css\", \"app.css\"])\n\nSee `Beacon.RuntimeCSS.TailwindCompiler` for more info.","ref":"Beacon.Config.html#t:tailwind_css/0-example"},{"type":"type","title":"Beacon.Config.template_formats/0","doc":"Register formats to handle templates, eg: `[{:heex, \"HEEx (HTML)\"}]`.\n\nBeacon provides two formats built-in, HEEx and Markdown, but you can register your own\nas long as you also implement the life-cycle stages `:load_template` and `:render_template`.\n\nThe description is used on user interfaces as Beacon Admin.","ref":"Beacon.Config.html#t:template_formats/0"},{"type":"function","title":"Beacon.Config.update_value/3","doc":"Updates `key` with `value` for the `site` configuration, at runtime.","ref":"Beacon.Config.html#update_value/3"},{"type":"behaviour","title":"Beacon.Content.PageField","doc":"Custom page fields for pages.\n\nEach `Beacon.Content.Page` have a default set of fields that\nfits most cases for building pages for your sites,\nbut in some cases you need custom data to either help manage\nthose page in Beacon Admin or to display such data.\n\nFor example, you can add a field to store each blog post illustration,\nor a field to include tags, and so on.\n\nEach page field will be:\n\n  * stored in the `page.extra` map field\n  * displayed in Beacon LiveAdmin\n  * validated when the pages is saved or published","ref":"Beacon.Content.PageField.html"},{"type":"behaviour","title":"Example - Beacon.Content.PageField","doc":"defmodule MyApp.TagsField do\n      use Phoenix.Component\n      import Beacon.Web.CoreComponents\n      import Ecto.Changeset\n\n      @behaviour Beacon.Content.PageField\n\n      @impl true\n      def name, do: :tags\n\n      @impl true\n      def type, do: :string\n\n      @impl true\n      def render(assigns) do\n        ~H\"\"\"\n        <.input type=\"text\" label=\"Tags\" field={@field} />\n        \"\"\"\n      end\n\n      @impl true\n      def changeset(data, attrs, _metadata) do\n        data\n        |> cast(attrs, [:tags])\n        |> validate_required([:tags])\n      end\n    end","ref":"Beacon.Content.PageField.html#module-example"},{"type":"callback","title":"Beacon.Content.PageField.changeset/3","doc":"Changeset used to validate and save data.","ref":"Beacon.Content.PageField.html#c:changeset/3"},{"type":"callback","title":"Beacon.Content.PageField.default/0","doc":"Default value for field. Defaults to `nil`.","ref":"Beacon.Content.PageField.html#c:default/0"},{"type":"callback","title":"Beacon.Content.PageField.name/0","doc":"Field identifier. Must be unique per site.","ref":"Beacon.Content.PageField.html#c:name/0"},{"type":"callback","title":"Beacon.Content.PageField.render/1","doc":"Template to render the field on LiveAdmin.","ref":"Beacon.Content.PageField.html#c:render/1"},{"type":"function","title":"Beacon.Content.PageField.render_field/3","doc":"","ref":"Beacon.Content.PageField.html#render_field/3"},{"type":"callback","title":"Beacon.Content.PageField.type/0","doc":"Field type. Can be any value supported by Ecto Schema.","ref":"Beacon.Content.PageField.html#c:type/0"},{"type":"behaviour","title":"Beacon.Lifecycle","doc":"Beacon is open for extensibility by allowing users to inject custom steps into its internal lifecycle.\n\nYou can add or overwrite those steps in `t:Beacon.Config.lifecycle/0`.\n\nEach one of these functions will be called in specific places inside Beacon's lifecycle,\nexecuting the steps defined in the site config.\n\nSee each function doc for more info and also `Beacon.Config`.","ref":"Beacon.Lifecycle.html"},{"type":"function","title":"Beacon.Lifecycle.execute/5","doc":"Initializes a `Beacon.Lifecycle` with given `lifecycle_name` and `resource`, runs all steps\nfor each lifecycle stage, and returns the Lifecycle with result stored in its `:output` key.\n\nExpects a `provider` which implements callbacks for validation before and after execution,\nas well as the logic for parsing contextual metadata.","ref":"Beacon.Lifecycle.html#execute/5"},{"type":"function","title":"Options - Beacon.Lifecycle.execute/5","doc":"* `:sub_key` - The name of a specific lifecycle stage to execute.  If provided, all other stages\n    will be skipped.  Defaults to `nil`, which runs all stages.\n  * `context` - Metadata to insert into the Lifecycle for use during execution.  Defaults to `nil`.","ref":"Beacon.Lifecycle.html#execute/5-options"},{"type":"function","title":"Beacon.Lifecycle.execute_steps/1","doc":"Runs all steps in a given Lifecycle.","ref":"Beacon.Lifecycle.html#execute_steps/1"},{"type":"callback","title":"Beacon.Lifecycle.put_metadata/3","doc":"Logic to put metadata from a given context into a `Beacon.Lifecycle`.  The `Beacon.Config` will\nalso be available here if needed.","ref":"Beacon.Lifecycle.html#c:put_metadata/3"},{"type":"function","title":"Beacon.Lifecycle.put_metadata/4","doc":"This function delegates to the `put_metadata/3` callback implemented by the `provider` module.\n\nIf the callback is not implemented, this will be a no-op.","ref":"Beacon.Lifecycle.html#put_metadata/4"},{"type":"function","title":"Beacon.Lifecycle.put_steps/3","doc":"Fetches the steps for a given Lifecycle from the provided `Beacon.Config` and puts those\nsteps into the Lifecycle.\n\nA `sub_key` may be provided to only consider the steps for a single lifecycle stage","ref":"Beacon.Lifecycle.html#put_steps/3"},{"type":"type","title":"Beacon.Lifecycle.t/0","doc":"","ref":"Beacon.Lifecycle.html#t:t/0"},{"type":"callback","title":"Beacon.Lifecycle.validate_input!/3","doc":"Logic to validate that a given `Beacon.Lifecycle` was initialized properly, according to the given\n`Beacon.Config`.  A sub-key may be provided if only one lifecycle stage is to be validated, otherwise\n`nil` is expected for the third argument.","ref":"Beacon.Lifecycle.html#c:validate_input!/3"},{"type":"function","title":"Beacon.Lifecycle.validate_input!/4","doc":"This function delegates to the `validate_input!/3` callback implemented by the `provider` module.\n\nIf the callback is not implemented, this will be a no-op.","ref":"Beacon.Lifecycle.html#validate_input!/4"},{"type":"callback","title":"Beacon.Lifecycle.validate_output!/3","doc":"After executing all the steps of a `Beacon.Lifecycle`, it is passed here to validate that the output\nis as-expected.  The `Beacon.Config` will also be available here if needed.","ref":"Beacon.Lifecycle.html#c:validate_output!/3"},{"type":"function","title":"Beacon.Lifecycle.validate_output!/4","doc":"This function delegates to the `validate_output!/3` callback implemented by the `provider` module.\n\nIf the callback is not implemented, this will be a no-op.","ref":"Beacon.Lifecycle.html#validate_output!/4"},{"type":"behaviour","title":"Beacon.MediaLibrary.AssetField","doc":"Custom asset fields for assets.\n\nEach `Beacon.MediaLibrary.Asset` has a default set of fields that\nfits most cases for assets for your sites,\nbut in some cases you need custom data to either help manage\nthose assets in Beacon Admin or to display such data.\n\nEach asset field will be:\n\n  * stored in the `asset.extra` map field\n  * displayed in Beacon Admin\n  * validated when the assets are saved or published","ref":"Beacon.MediaLibrary.AssetField.html"},{"type":"behaviour","title":"Example - Beacon.MediaLibrary.AssetField","doc":"defmodule MyApp.AltTextField do\n      use Phoenix.Component\n      import Beacon.Web.CoreComponents\n      import Ecto.Changeset\n\n      @behaviour Beacon.MediaLibrary.AssetField\n\n      @impl true\n      def name, do: :alt\n\n      @impl true\n      def type, do: :string\n\n      @impl true\n      def render_input(assigns) do\n        ~H\"\"\"\n        <.input type=\"text\" label=\"Alt Text\" field={@field} />\n        \"\"\"\n      end\n\n      @impl true\n      def render_show(assigns) do\n        ~H\"\"\"\n        <.input type=\"text\" label=\"Alt Text\" value={@value} />\n        \"\"\"\n      end\n\n      @impl true\n      def changeset(data, attrs) do\n        data\n        |> cast(attrs, [:alt])\n        |> validate_required([:alt])\n      end\n    end","ref":"Beacon.MediaLibrary.AssetField.html#module-example"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.changeset/3","doc":"Changeset used to validate and save data.","ref":"Beacon.MediaLibrary.AssetField.html#c:changeset/3"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.default/0","doc":"Default value for field. Defaults to `nil`.","ref":"Beacon.MediaLibrary.AssetField.html#c:default/0"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.name/0","doc":"Field identifier. Must be unique per site.","ref":"Beacon.MediaLibrary.AssetField.html#c:name/0"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.render_input/1","doc":"Template to render the form field on Admin.","ref":"Beacon.MediaLibrary.AssetField.html#c:render_input/1"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.render_show/1","doc":"Template to render the show field on Admin.","ref":"Beacon.MediaLibrary.AssetField.html#c:render_show/1"},{"type":"callback","title":"Beacon.MediaLibrary.AssetField.type/0","doc":"Field type. Can be any value supported by Ecto Schema.","ref":"Beacon.MediaLibrary.AssetField.html#c:type/0"},{"type":"module","title":"Beacon.Template.LoadMetadata","doc":"Metadata passed to page loading lifecycle.","ref":"Beacon.Template.LoadMetadata.html"},{"type":"type","title":"Beacon.Template.LoadMetadata.t/0","doc":"","ref":"Beacon.Template.LoadMetadata.html#t:t/0"},{"type":"module","title":"Beacon.Template.RenderMetadata","doc":"Metadata passed to page rendering lifecycle.","ref":"Beacon.Template.RenderMetadata.html"},{"type":"type","title":"Beacon.Template.RenderMetadata.t/0","doc":"","ref":"Beacon.Template.RenderMetadata.html#t:t/0"},{"type":"module","title":"Beacon.Types.JsonArrayMap","doc":"Convert between json and map enforcing the data shape as array of objects/maps.","ref":"Beacon.Types.JsonArrayMap.html"},{"type":"function","title":"Beacon.Types.JsonArrayMap.embed_as/1","doc":"","ref":"Beacon.Types.JsonArrayMap.html#embed_as/1"},{"type":"function","title":"Beacon.Types.JsonArrayMap.equal?/2","doc":"","ref":"Beacon.Types.JsonArrayMap.html#equal?/2"},{"type":"type","title":"Beacon.Types.JsonArrayMap.t/0","doc":"List of maps","ref":"Beacon.Types.JsonArrayMap.html#t:t/0"},{"type":"module","title":"Beacon.Types.Site","doc":"Sites are identified as atoms and stored as string in the database.","ref":"Beacon.Types.Site.html"},{"type":"function","title":"Beacon.Types.Site.embed_as/1","doc":"","ref":"Beacon.Types.Site.html#embed_as/1"},{"type":"function","title":"Beacon.Types.Site.equal?/2","doc":"","ref":"Beacon.Types.Site.html#equal?/2"},{"type":"type","title":"Beacon.Types.Site.t/0","doc":"Atom to identify a site, eg: `:my_site`","ref":"Beacon.Types.Site.html#t:t/0"},{"type":"function","title":"Beacon.Types.Site.valid?/1","doc":"Returns `true` if `site` is valid, otherwise returns `false`.\n\nA site is valid if it's a atom with no special characters, except `_`.","ref":"Beacon.Types.Site.html#valid?/1"},{"type":"function","title":"Examples - Beacon.Types.Site.valid?/1","doc":"iex> valid?(:my_site)\n    true\n\n    iex> valid?(:\"my-site!\")\n    false","ref":"Beacon.Types.Site.html#valid?/1-examples"},{"type":"function","title":"Beacon.Types.Site.valid_name?/1","doc":"","ref":"Beacon.Types.Site.html#valid_name?/1"},{"type":"function","title":"Beacon.Types.Site.valid_path?/1","doc":"","ref":"Beacon.Types.Site.html#valid_path?/1"},{"type":"exception","title":"Beacon.ConfigError","doc":"Raised when some option in `Beacon.Config` is invalid.\n\nIf you are seeing this error, check `application.ex` where your site's config is defined.\nDescription and examples of each allowed configuration option can be found in `Beacon.Config.new/1`","ref":"Beacon.ConfigError.html"},{"type":"exception","title":"Beacon.InvokeError","doc":"Raised when Beacon attempts to `apply` a module/function/arguments unsuccessfully.\n\nIf you see this error consistently, make sure the resource being called is created and published,\notherwise that might be a bug in Beacon.","ref":"Beacon.InvokeError.html"},{"type":"exception","title":"Beacon.LoaderError","doc":"Raised when Beacon attempts to load content into memory unsuccessfully.\n\nThere are several causes that can lead to this error, so if you're seeing it, be sure to read the\nfull error message for any additional information or steps to take.  If a LoaderError is crashing\nyour app on startup, it could indicate a problem in your `Beacon.Config`","ref":"Beacon.LoaderError.html"},{"type":"exception","title":"Beacon.ParserError","doc":"Raised when Beacon's Markdown engine attempts to convert Markdown to HTML unsuccessfully.\n\nThis error can be triggered by layouts, pages, and any other resource that display HTML.\nIf you're seeing it, ensure that your template formats are correct. The full error message\nwill have details on where to look.","ref":"Beacon.ParserError.html"},{"type":"exception","title":"Beacon.RuntimeError","doc":"Raised when Beacon attempts to read content from memory unsuccessfully.\n\nIf you see this error consistently, check the message to see if `handle_event` is mentioned.\nIf so, this can indicate that pages on your site are sending events (e.g. button clicks,\nform submissions) to an event handler which hasn't been implemented, or that the event name\ndoes not match your handler.","ref":"Beacon.RuntimeError.html"},{"type":"exception","title":"Beacon.SnippetError","doc":"Raised when Beacon attempts to render a `Beacon.Content.Snippets.Helper` unsuccessfully.\n\nIf you're seeing this error, check for typos in your helpers' `:body`.","ref":"Beacon.SnippetError.html"},{"type":"type","title":"Beacon.SnippetError.t/0","doc":"","ref":"Beacon.SnippetError.html#t:t/0"},{"type":"exception","title":"Beacon.Web.NotFoundError","doc":"Raised when Beacon attempts to serve a page or asset on an invalid path.\n\nIf you're seeing this error, it means some of your users are using the wrong URL.\nTo some extent, this is unavoidable, but consistently seeing this error with the same path\ncan indicate that somewhere in your page content, you are creating invalid links.","ref":"Beacon.Web.NotFoundError.html"},{"type":"exception","title":"Beacon.Web.ServerError","doc":"Raised when a `Beacon.Content.EventHandler` or a `Beacon.Content.InfoHandler` returns an invalid response.\n\nIf you're seeing this error, check the code in your site's event handlers or info handlers, and\nensure that each one returns `{:noreply, socket}`.","ref":"Beacon.Web.ServerError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.5.0 (2025-03-18) - Changelog","doc":"","ref":"changelog.html#0-5-0-2025-03-18"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Router] Add option `:session` in `beacon_site`\n  - [Media Library] Add customizable asset delete callbacks - @ddink\n  - [Visual Editor] Include assigns defined by `on_mount` in the page context\n  - [Tasks] Move `beacon_site` to the end of the router (mount it after other routes)\n  - [Tasks] Rename default Phoenix route when adding a site at root \"/\" to avoid conflicts","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- [Migrations] Hardcode `inserted_at` and `updated_at` - @sltong\n  - [Media Library] Don't crash on invalid search query\n  - [Proxy Endpoint] Check when host/port doesn't match","ref":"changelog.html#fixes"},{"type":"extras","title":"Chores - Changelog","doc":"- [CI] Organize workflows and run Igniter tasks separately","ref":"changelog.html#chores"},{"type":"extras","title":"Docs - Changelog","doc":"- Clean up and fix typos","ref":"changelog.html#docs"},{"type":"extras","title":"0.4.3 (2025-03-04) - Changelog","doc":"","ref":"changelog.html#0-4-3-2025-03-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add argument `--host-dev` in `gen.site` task to define the host for dev (local) environments.","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.4.2 (2025-02-26) - Changelog","doc":"","ref":"changelog.html#0-4-2-2025-02-26"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix the select router prompt on `gen.site` when multiple routers are found","ref":"changelog.html#fixes"},{"type":"extras","title":"Docs - Changelog","doc":"- Add `@doc type` for `:event_handlers`\n  - Add `Beacon.Web.ErrorHTML` and `Beacon.Web.Layouts` to web docs\n  - Hide `Beacon.Web.RobotsTxt` and `Beacon.Web.SitemapXML` from docs (private modules)","ref":"changelog.html#docs"},{"type":"extras","title":"0.4.1 (2025-02-20) - Changelog","doc":"","ref":"changelog.html#0-4-1-2025-02-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Added new lifecycle stage `:after_unpublish_page` to execute hooks after a page is unpublished\n  - Added `MediaLibrary.url_for_asset/2` to return URL of an asset given the site and the file name\n  - `gen.site` added option `--endpoint` to overwrite the generated site endpoint module name","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- `gen.site` doesn't fail on pre-existing endpoint module\n  - `gen.site` doesn't remove existing beacon runtime config\n  - `gen.proxy_endpoint` add missing `:render_errors` config for ProxyEndpoint\n  - Do not call compile-time endpoint functions during boot\n  - Pin esbuild version up to 0.8.x on OTP v24 and below (esbuild 0.0 requires OTP v25)\n  - Do not crash `url_for/1`, `url_for/2`, or `urls_for/1` when provider is invalid\n  - Fix rendering mermaid blocks on docs","ref":"changelog.html#fixes"},{"type":"extras","title":"Chores - Changelog","doc":"- Updated `ex_doc` to generate better docs","ref":"changelog.html#chores"},{"type":"extras","title":"0.4.0 (2025-02-12) - Changelog","doc":"","ref":"changelog.html#0-4-0-2025-02-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Added support for user-defined JS Hooks\n  - Added `Beacon.ProxyEndpoint` for serving multiple domains within a single app\n  - `mix beacon.gen.site` now creates a new Endpoint per site\n  - Added `--host` option for `mix beacon.gen.site` to serve your site at an alternative domain\n  - Beacon will now automatically generate a `robots.txt` for each domain served by your app\n  - Beacon will now automatically generate a `sitemap_index.txt` for each domain, and `sitemap.txt` for each site","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fixed a bug where MediaLibrary could check for file contents on the wrong node in multi-node deployments\n  - Fixed a missing function error when using Beacon with LiveView 1.0.0\n  - Visual Editor no longer crashes when attempting to edit a new page before saving\n  - Asset requests now properly follow `accept-encoding` headers\n  - Prevent app Endpoints from accepting requests before Beacon is ready to serve them\n  - `Beacon.Config` now falls back to the default value when a `nil` option is provided\n  - Live Data `query_params` now defauls to `%{}`","ref":"changelog.html#fixes"},{"type":"extras","title":"Docs - Changelog","doc":"- Setup Tailwind v3\n  - Updated Deploy to Fly.io guides to properly install tailwind and esbuild\n  - Updated Install guides to use `igniter.install` instead of manually adding deps\n  - Recipe on how to use external JS libraries in JS hooks","ref":"changelog.html#docs"},{"type":"extras","title":"Chores - Changelog","doc":"- Relax `:oembed` to `~> 0.4`\n  - Make `:igniter` optional","ref":"changelog.html#chores"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Require `esbuild` binary in releases\n  - Require minimum `:igniter` v0.5\n  - `beacon.install` - removed command aliases `-s` and `-p`\n  - `beacon.gen.site` - removed command aliases `-s` and `-p`\n  - `beacon.gen.tailwind_config` - removed command alias `-s`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.3.3 (2024-12-13) - Changelog","doc":"","ref":"changelog.html#0-3-3-2024-12-13"},{"type":"extras","title":"Fixes - Changelog","doc":"- Support LiveView v1.0.1\n  - Fix variant roll changing when fetching assets","ref":"changelog.html#fixes"},{"type":"extras","title":"0.3.2 (2024-12-11) - Changelog","doc":"","ref":"changelog.html#0-3-2-2024-12-11"},{"type":"extras","title":"Fixes - Changelog","doc":"- Make the logic to find reachable sites less strict\n  - Prevent components module reloading in manual mode","ref":"changelog.html#fixes"},{"type":"extras","title":"Doc - Changelog","doc":"- Add missing instructions on Upgrade Guide v0.3.0 where to place the Beacon tuple","ref":"changelog.html#doc"},{"type":"extras","title":"Chore - Changelog","doc":"- Exclude Dialyzer files from package","ref":"changelog.html#chore"},{"type":"extras","title":"0.3.1 (2024-12-10) - Changelog","doc":"","ref":"changelog.html#0-3-1-2024-12-10"},{"type":"extras","title":"Fixes - Changelog","doc":"- Avoid unloading imported dynamic Components modules without a replacement","ref":"changelog.html#fixes"},{"type":"extras","title":"0.3.0 (2024-12-05) - Changelog","doc":"","ref":"changelog.html#0-3-0-2024-12-05"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support Phoenix LiveView 1.0\n  - Add `Beacon.Plug` in the `:beacon` pipeline\n  - Generate sites in the main host app alias to allow mixing routes\n  - Introduce global process lock for Loader Workers, preventing multiple workers from\n    attempting to compile the same module simultaneously\n  - Page Warming - Beacon will now eagerly load a small number of pages (default 10) at boot time for SEO\n  - `Beacon.Config` option `:page_warming` can change the number of pages to warm, specify which pages, or disable warming per site\n  - Only start reachable sites on boot to save resources\n  - Warn when a site defined in the router is not reachable\n  - Add `Beacon.Plug` for consistent rendering when using Page Variants\n  - `mix beacon.install` now adds `Beacon.Plug` to host app Router","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Rename arg from `name` to `tag_name` in dynamic_tag component\n  - Remove self dependency on ErrorPage module\n  - Allow `:admin` source for BeaconAssigns in unpublished Pages","ref":"changelog.html#fixes"},{"type":"extras","title":"Chores - Changelog","doc":"- Isolate dynamic function calls","ref":"changelog.html#chores"},{"type":"extras","title":"Documentation - Changelog","doc":"- Add Deployment Topologies guide","ref":"changelog.html#documentation"},{"type":"extras","title":"0.2.2 (2024-11-17) - Changelog","doc":"","ref":"changelog.html#0-2-2-2024-11-17"},{"type":"extras","title":"Fixes - Changelog","doc":"- Do not duplicate default beacon.webp media asset\n  - Load resources on dev.exs script","ref":"changelog.html#fixes"},{"type":"extras","title":"Chores - Changelog","doc":"- Improve `Beacon.apply_mfa/4` error output","ref":"changelog.html#chores"},{"type":"extras","title":"0.2.1 (2024-11-14) - Changelog","doc":"","ref":"changelog.html#0-2-1-2024-11-14"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix site scoping for media asset path/url\n  - Serve media library assets inside aliased scopes\n  - Only reset cache and route table for published pages","ref":"changelog.html#fixes"},{"type":"extras","title":"Documentation - Changelog","doc":"- Fix indentation on install guide\n  - Add upgrade guide from pre-rc to v0.1","ref":"changelog.html#documentation"},{"type":"extras","title":"Chores - Changelog","doc":"- Rename asset URL from `__beacon_assets__` to `__beacon_media__` to avoid conflicts\n  - Expose option `:root_layout` in `beacon_site`","ref":"changelog.html#chores"},{"type":"extras","title":"0.2.0 (2024-11-08) - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-11-08"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Introduce `Beacon.ErrorHandler` to load resources and dependencies\n  - Add `beacon.gen.tailwind_config` task to generate a custom Tailwind config\n  - Add `beacon.gen.site` task to generate new sites\n  - Rework `beacon.install` with Igniter to be composable\n  - Introduce config `:tailwind_css`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Documentation - Changelog","doc":"- Create recipe Protect Pages with Basic Auth\n  - Update docs to use the new tasks created with Igniter\n  - Update Deploy to Fly.io guide to use a release step to copy files into the release\n  - Create recipe Reuse app.css","ref":"changelog.html#documentation"},{"type":"extras","title":"Chore - Changelog","doc":"- Only subscribe to page changes on `:live` sites","ref":"changelog.html#chore"},{"type":"extras","title":"0.1.4 (2024-10-31) - Changelog","doc":"","ref":"changelog.html#0-1-4-2024-10-31"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix Page and Layout publish on cluster environments\n  - Skip dependency `:vix` v0.31.0 due to a bug to open files\n  - Fix page title not updating on page patch","ref":"changelog.html#fixes"},{"type":"extras","title":"Chores - Changelog","doc":"- TailwindCompiler - increase timeout to 4 minutes when waiting to generate template files","ref":"changelog.html#chores"},{"type":"extras","title":"0.1.3 (2024-10-29) - Changelog","doc":"","ref":"changelog.html#0-1-3-2024-10-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Auto populate Media beacon.webp to be used on components","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Exclude the node modules from Tailwind content #622 by @anu788\n - Allow to patch (navigate patching the content) to another site","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.2 (2024-10-23) - Changelog","doc":"","ref":"changelog.html#0-1-2-2024-10-23"},{"type":"extras","title":"Fixes - Changelog","doc":"- [Content/Component] - Validate attr opts and slot opts to avoid invalid state and compilation errors","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.1 (2024-10-22) - Changelog","doc":"","ref":"changelog.html#0-1-1-2024-10-22"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support Phoenix LiveView v1.0.0-rc.7","ref":"changelog.html#enhancements"},{"type":"extras","title":"Documentation - Changelog","doc":"- Link to latest version\n  - Guide for `on_mount` and `handle_info` - #599 by @djcarpe","ref":"changelog.html#documentation"},{"type":"extras","title":"0.1.0 (2024-10-09) - Changelog","doc":"","ref":"changelog.html#0-1-0-2024-10-09"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Require minimum Elixir v1.14.0\n  - Require minimum `:mdex` v0.2.0\n  - Removed config `:skip_boot?` in favor of `:mode` which can be `:live`, `:testing`, and `:manual` (defaults to `:live`) - the major difference between then is that live loads all modules and broadcasts all messages, testing only does that when it makes sense for tests (for example it does reload modules on fixtures), and manual does pretty much nothing, it's useful to seed data or to test specific scenarios where you need total control over Beacon.Loader","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Beacon.Test` that provides testing utilities to use on host apps\n  - Add `Beacon.Test.Fixtures` to expose fixtures to seed test data, the same used by Beacon itself\n  - Reload modules synchronously on `testing` mode\n  - Leverage `:manual` mode during boot to avoid unnecessary calls to Tailwind compiler, speeding up the whole process to start sites\n  - Enable Markdown options: `:footnotes`, `:multiline_block_quotes`, `:shortcodes` (emojis), `:underline`, `:relaxed_tasklist_matching`, and `:relaxed_autolinks`.\n    See https://docs.rs/comrak/latest/comrak/struct.ExtensionOptions.html and https://docs.rs/comrak/latest/comrak/struct.ParseOptions.html for more info.\n  - Added Shared Info Handlers (`info_handle` callbacks) - 578 by @ddink","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Remove unnecessary `:plug_cowboy` dependency\n  - [Heroicons] Namespace the vendorized module as `Beacon.Heroicons` to avoid conflicts","ref":"changelog.html#fixes"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added \"Testing\" recipe to demonstrate usage of `Beacon.Test`\n  - Added guide to customize the Markdown options\n  - Added guide on how to embed tweets using the Twitter JS api","ref":"changelog.html#documentation"},{"type":"extras","title":"0.1.0-rc.2 (2024-09-20) - Changelog","doc":"","ref":"changelog.html#0-1-0-rc-2-2024-09-20"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Renamed component `.icon` to `.heroicon` to make it explicit that it's rendering Heroicons and also to avoid future conflicts\n  - Require minimum Gettext v0.26 to use the new backend module\n  - Default `:sort` option value in `Content.list_pages/2` changed from `:title` to `:path`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixes - Changelog","doc":"- [Components] Bring back the `.icon` (heroicon) component using https://hex.pm/packages/heroicons instead of a Tailwind plugin that depends on the unavailable `fs` API\n  - [Media Library] Guard against invalid values for `:sort` option in `MediaLibrary.list_assets/2`\n  - [Content] Guard against invalid values for `:sort` option in `Content.list_layouts/2`\n  - [Content] Guard against invalid values for `:sort` option in `Content.list_pages/2`\n  - [HEEx Decoder] Handle attr values with `nil` values, for example the `defer` in script tags","ref":"changelog.html#fixes"},{"type":"extras","title":"Documentation - Changelog","doc":"- Updated Heroicons recipes to reflect changes on configuration and naming","ref":"changelog.html#documentation"},{"type":"extras","title":"0.1.0-rc.1 (2024-08-27) - Changelog","doc":"","ref":"changelog.html#0-1-0-rc-1-2024-08-27"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Added Shared Event Handlers which are global event handlers shared among all pages.\n    That's a simple model to work with where a layout, component, or multiple pages may share the same event handler,\n    for example a newsletter subscription form in a component called in a layout doesn't need to duplicate the same\n    event handler in all pages.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Removed Page Event Handlers in favor of Shared Event Handlers.\n    With Shared Event Handlers, it doesn't make sense to have page event handlers unless overriding becomes a necessity.\n    The data is automatically migrated in a best-effort way, duplicated event handler names (from multiple pages) are\n    consolidated into a single shared event handler. See the migration `V002` for more info.\n  - Removed \"page event handlers\" in `Content` API in favor of \"event handlers\" (removed the prefix `page`),\n    for example: `update_event_handler_for_page -> create_event_handler` and `change_page_event_handler -> change_event_handler`.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixes - Changelog","doc":"- Display parsed page title on live renders","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.0-rc.0 (2024-08-02) - Changelog","doc":"","ref":"changelog.html#0-1-0-rc-0-2024-08-02"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Loader to fetch resources from DB and compile modules\n  - Media Library to upload and serve images and other media\n    - Built-in Repo (DB) and S3 storage\n    - Post-process images to optimized .webp format\n  - Error Page to handle failures and display custom pages\n    - Pre-defined 404 and 500 pages\n  - Components\n    - Pre-defined set of default components\n    - Support attrs and slots\n    - Support for Elixir and HEEx parts\n  - Layouts\n    - Pre-defined default layout\n    - Meta tags\n    - Resource links\n    - Revisions\n  - Pages\n    - Pre-defined default home page\n    - Meta tags\n    - Schema.org support\n    - Events (handle_event)\n    - Revisions\n  - Snippets (liquid template)\n    - Page title\n    - Meta tags\n  - Stylesheets\n  - Live Data to define and manage assigns at runtime\n    - Support Elixir and text content\n  - Custom Page fields to extend the Page schema\n  - Router helper `~p` to generate paths with site prefixes\n  - Content management through the `Beacon.Content` API\n  - A/B Variants\n  - TailwindCSS compiler\n  - `@beacon` read-only assign\n  - mix task `beacon.install` to bootstrap a new Beacon site\n  - Lifecycle hooks to inject custom logic into multiple layers of the process","ref":"changelog.html#enhancements"},{"type":"extras","title":"Installation","doc":"# Installation","ref":"installation.html"},{"type":"extras","title":"Objective - Installation","doc":"Install Beacon and Beacon LiveAdmin in a new Phoenix LiveView application to enable running and managing sites.","ref":"installation.html#objective"},{"type":"extras","title":"Notes - Installation","doc":"- If you already have a Phoenix LiveView application that meets the minimum requirements for Beacon and Beacon LiveAdmin, you can skip to \"Install in an existing application\".\n- Beacon LiveAdmin can be installed in a separate application in a cluster environment but such advanced setup is not covered in this guide.","ref":"installation.html#notes"},{"type":"extras","title":"Steps - Installation","doc":"1. Setup a PostgreSQL database server\n\nMake sure it's [installed and running](https://wiki.postgresql.org/wiki/Detailed_installation_guides).\n\nCurrently, only PostgreSQL is supported.\n\n2. Install Elixir v1.14 or later\n\nCheck out the official [Elixir install guide](https://elixir-lang.org/install.html) for more info.\n\n3. Either install as a new application or add into an existing application\n\nFirstly update Hex:\n\n```sh\nmix local.hex\n```\n\nThen install or update Phoenix and Igniter Installers\n\n```sh\nmix archive.install hex phx_new\nmix archive.install hex igniter_new\n```\n\nNow choose one of the methods below:","ref":"installation.html#steps"},{"type":"extras","title":"Install as a new application - Installation","doc":"If you're starting a new project, you can follow these steps to generate a new Phoenix application with Beacon and Beacon LiveAdmin installed.\n\nRun the following to generate a new app:\n\n```sh\nmix igniter.new my_app --install beacon,beacon_live_admin --with phx.new\n```\n\nReplace `my_app` with the name of your application and follow the prompts.","ref":"installation.html#install-as-a-new-application"},{"type":"extras","title":"Install in an existing application - Installation","doc":"Run the following to install Beacon and Beacon LiveAdmin in an existing Phoenix application.\n\n```sh\nmix igniter.install beacon,beacon_live_admin\n```","ref":"installation.html#install-in-an-existing-application"},{"type":"extras","title":"Next Steps - Installation","doc":"Beacon is installed but you have no sites yet. Follow the [Your First Site](your-first-site.md) guide to set up one and get familiar with Beacon.","ref":"installation.html#next-steps"},{"type":"extras","title":"Your First Site","doc":"# Your First Site","ref":"your-first-site.html"},{"type":"extras","title":"Objective - Your First Site","doc":"Create your first site to get familiar with Beacon resources and the most common operations to manage your site.","ref":"your-first-site.html#objective"},{"type":"extras","title":"Prerequisites - Your First Site","doc":"In order to create and serve sites, you'll need a Phoenix LiveView application with both Beacon and Beacon LiveAdmin installed.\n\nFollow the [Beacon installation guide](installation.md) to set up the environment before proceeding with this guide.","ref":"your-first-site.html#prerequisites"},{"type":"extras","title":"Setup - Your First Site","doc":"Beacon provides a command to get your site up and running quickly. Open a terminal and run the following command:\n\n```sh\nmix beacon.gen.site --site my_site --path /\n```\n\nRouting is an important aspect of the site configuration, especially the order the routes are defined.\nInspect the generated changes in the `router.ex` file and see [Beacon.Router](https://hexdocs.pm/beacon/Beacon.Router.html)  for more info.","ref":"your-first-site.html#setup"},{"type":"extras","title":"Starting the application - Your First Site","doc":"Firstly execute the following to install all dependencies:\n\n```sh\nmix setup\n```\n\nAnd now start your Phoenix app:\n\n```sh\nmix phx.server\n```\n\nVisit http://localhost:4000 to see the default home page created by Beacon.","ref":"your-first-site.html#starting-the-application"},{"type":"extras","title":"Accessing LiveAdmin to manage your site - Your First Site","doc":"Let's customize the home page. Visit http://localhost:4000/admin and you should see the `my_site` that you just created listed on the admin interface.\n\nNow let's create the resources for our first site, and we'll starting by creating the resources used by the home page before we customize its template.","ref":"your-first-site.html#accessing-liveadmin-to-manage-your-site"},{"type":"extras","title":"Live Data - Your First Site","doc":"We want to display the current year at the footer of the home page as the assign `<%= @current_year %>`. In Beacon we use Live Data to create and modify assigns at runtime, let's do it.\n\nGo to http://localhost:4000/admin/my_site/live_data to create a new path `/` and then create a new assign named `current_year` using the `elixir` format with the following value:\n\n```elixir\nDate.utc_today().year\n```\n\nSave the changes.","ref":"your-first-site.html#live-data"},{"type":"extras","title":"Handle the form submission event - Your First Site","doc":"Besides displaying some data, we will have a form to subscribe people to our newsletter, so we need to react to that event. Let's create an event handler for that.\n\nGo to http://localhost:4000/admin/my_site/events and create a new event handler named `join` with the following content:\n\n```elixir\n%{\"waitlist\" => %{\"email\" => email}} = event_params\nIO.puts(\"#{email} joined the waitlist\")\n{:noreply, assign(socket, :joined, true)}\n```","ref":"your-first-site.html#handle-the-form-submission-event"},{"type":"extras","title":"Upload an image - Your First Site","doc":"The last resource we want in our home is an image. Beacon provides a Media Library to upload and serve images, so first find an image and rename it to `image.jpg` (or the extension you prefer). Or use one of the images in https://github.com/BeaconCMS/beacon/tree/main/assets/images\n\nGo to http://localhost:4000/admin/my_site/media_library and upload that image.","ref":"your-first-site.html#upload-an-image"},{"type":"extras","title":"Update the home page template - Your First Site","doc":"Finally, let's update the home page template to make use of the live data, the event handler, and the image we just uploaded.\n\nGo to http://localhost:4000/admin/my_site/pages and you should see a page already created for the `/` path. We're going to change it.\n\nEdit the template to replace with this content:\n\n```heex\n \n    \n   \n     \n      <.link patch=\"/\" class=\"flex items-center gap-2\">\n         \n            \n            \n         \n         CMS Platform \n      </.link>\n     \n     \n      <.link patch=\"/blog\" class=\"hidden md:inline-flex text-sm font-medium hover:underline\">\n        Blog\n      </.link>\n       \n        Sign In\n       \n     \n   \n   \n     \n       \n         \n           \n             \n              Unlock the power\n             \n            of your content with our CMS platform\n           \n           \n            Streamline your content management with our intuitive, high-performance CMS built on Phoenix LiveView.\n           \n           \n             \n               \n               \n                Join Waitlist\n               \n             \n           \n           \n            Congrats! You joined the watchlist.\n           \n         \n         \n          <.image\n            site={@beacon.site}\n            name=\"image.webp\"\n            alt=\"My Page Image\"\n            width=\"600\"\n            height=\"600\"\n            style=\"aspect-ratio: 600 / 600; object-fit: cover;\"\n          />\n         \n       \n     \n   \n   \n     \n       © <%= @current_year %> CMS Platform. All rights reserved. \n       \n         \n           \n              \n           \n           Facebook \n         \n         \n           \n             \n             \n           \n           Twitter \n         \n         \n           \n              \n              \n              \n           \n           Instagram \n         \n         \n           \n             \n             \n              \n              \n           \n           LinkedIn \n         \n         \n          Privacy\n         \n         \n          Terms\n         \n         \n          Contact\n         \n       \n     \n   \n \n```\n\nSave the changes and Publish the page. Go to http://localhost:4000 to check the final result!\n\nYou should see the image you uploaded, the current year on the footer, and a message after joining the waitlist.\n\n-----------------------\n\nCongratulations! You have a site up and running.\n\nCheck out all available recipes to learn how to deploy your site, customize it even further, and more.","ref":"your-first-site.html#update-the-home-page-template"},{"type":"extras","title":"Access Page Info","doc":"# Access Page Info\n\nIn order to build pages you might need to access the current site or page information, as query params, path, and so on.\n\nBeacon provides a read-only assign `@beacon` that is available on all templates and also on Elixir code blocks as event handlers.\n\nFor example, to access the current page title in a template:\n\n```heex\n <%= @beacon.page.title %> \n```\n\nSee the module [Beacon.Web.BeaconAssigns](https://hexdocs.pm/beacon/Beacon.Web.BeaconAssigns.html) for more info.","ref":"access-page-info.html"},{"type":"extras","title":"Add Custom Page Fields","doc":"# Add Custom Page Fields\n\nEvery Beacon page contains a set of pre-defined fields which behave the same way across all pages. For example, the Title and Description fields are used to fill meta tags for SEO purposes;  that's the same behavior regardless of whether the page is your homepage, a blog post, or something else.\n\nHowever, you often need to store custom data, perform some logic on that data, or display extra information on the page. Some examples include tags for blog posts, authors, links, and so on.\n\nIn this recipe we'll add a custom page field `Type` to allow users identify the type of the page on the admin interface and use that data to list recent blog posts.","ref":"add-custom-page-fields.html"},{"type":"extras","title":"Add a module that implements the `Beacon.Content.PageField` behavior - Add Custom Page Fields","doc":"In this module we'll define how the data is stored and validated, and how the field is displayed on the admin interface. Create a file with this content:\n\n```elixir\ndefmodule MyApp.Beacon.PageFields.Type do\n  @moduledoc false\n  @behaviour Beacon.Content.PageField\n\n  use Phoenix.Component\n  import Beacon.Web.CoreComponents\n  import Ecto.Changeset\n\n  @impl true\n  def name, do: :type\n\n  @impl true\n  def type, do: :string\n\n  @impl true\n  def default, do: \"page\"\n\n  @impl true\n  def changeset(data, attrs, %{page_changeset: _page_changeset}) do\n    cast(data, attrs, [:type])\n  end\n\n  @impl true\n  def render(assigns) do\n    assigns = Map.put(assigns, :opts, [{\"Page\", \"page\"}, {\"Blog Post\", \"blog_post\"}])\n\n    ~H\"\"\"\n    <.input type=\"select\" label=\"Type\" prompt=\"Choose type\" options={@opts} field={@field} />\n    \"\"\"\n  end\nend\n```\n\nLet's break down each part of the module:\n\n* `name/0` - can be any atom that represents the field name, for example `:tags` for a lists of tags or `:author_id` to store a reference to the page author.\n* `type/0` - any valid [Ecto Schema type](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)\n* `default/0` - the value with which to pre-populate the field\n* `changeset/3` - this is where you can add your own validation logic to that field (note: `page_changeset` is the changeset for the `%Beacon.Content.Page{}` itself)\n* `render/1` - the template to display the field on the page editor in Beacon LiveAdmin","ref":"add-custom-page-fields.html#add-a-module-that-implements-the-beacon-content-pagefield-behavior"},{"type":"extras","title":"Add the field to Beacon.Config - Add Custom Page Fields","doc":"Next we'll configure Beacon to include this type for your pages.  In `application.ex` add to your\nexisting Beacon config:\n\n```elixir\nsites: [\n  [\n    name: :my_site,\n    ...\n    extra_page_fields: [\n      ...\n      MyApp.Beacon.PageFields.Type\n    ]\n  ]\n]\n```","ref":"add-custom-page-fields.html#add-the-field-to-beacon-config"},{"type":"extras","title":"Access the field content - Add Custom Page Fields","doc":"Now, when a page is created or updated with your custom field, the content will be stored in the `:extra` field of the `%Beacon.Content.Page{}` record, under the name of the custom field.\n\nWith the example code above, a page with type `blog_post` will look like this:\n\n```elixir\n%Beacon.Content.Page{\n  # ...\n  extra: %{\"type\" => \"blog_post\"},\n}\n```\n\nYou can make use of this field to filter pages on the backend or display that extra information in the page template!","ref":"add-custom-page-fields.html#access-the-field-content"},{"type":"extras","title":"Create a Blog","doc":"# Create a Blog","ref":"create-a-blog.html"},{"type":"extras","title":"Objective - Create a Blog","doc":"Create blog posts with custom fields (type and tags), render each one using the tailwind typography plugin, and list the most recent posts on an index page.","ref":"create-a-blog.html#objective"},{"type":"extras","title":"Prerequisites - Create a Blog","doc":"This guide will skip initial setup steps that are already covered in the [your first site guide](../introduction/your-first-site.md),\nso if that's the first site in your application, or you're not familiar with Beacon, please follow that guide first to make sure\nboth Beacon and BeaconLiveAdmin are correctly installed and configured.","ref":"create-a-blog.html#prerequisites"},{"type":"extras","title":"Create the blog site - Create a Blog","doc":"Now that we have fulfilled the requirements, let's create the blog site mounted at `/blog`. Go ahead and execute:\n\n```sh\nmix beacon.gen.site --site blog --path /blog\n```\n\nReview the changes applied to your project `router.ex` file to make sure the order of routes is correct,\nfor example, if you already have a site mounted at the root `/` it should be placed after the `/blog` route.\n\nSee [Beacon.Router](https://hexdocs.pm/beacon/Beacon.Router.html) for more info.\n\nA basic site was just created but we have some work to do before executing the Phoenix server.","ref":"create-a-blog.html#create-the-blog-site"},{"type":"extras","title":"Custom Fields - Create a Blog","doc":"Every Beacon page has some essential fields like title, path, description, and others. But since Beacon can be used to build any kind of site, from small to big,\nwe can't include all possible fields to accommodate all kinds of pages that might exist. The solution to this problem is allowing users to create their own custom fields,\nwhich are displayed in the Beacon LiveAdmin editor, stored in the page record in the database, and can be used in queries and templates.\n\nFor our blog we'll create 2 custom fields: type and tags. Type to allow us to distinguish between a regular page and a blog post, and tags to allow us to categorize our posts.\n\nThose custom fields are modules that implement the `Beacon.Content.PageField` behaviour. Let's create them.","ref":"create-a-blog.html#custom-fields"},{"type":"extras","title":"Custom field `type` - Create a Blog","doc":"Create a new file `lib/my_app/beacon/page_fields/type.ex` with the following content:\n\n```elixir\ndefmodule MyApp.Beacon.PageFields.Type do\n  @moduledoc \"\"\"\n  Custom beacon page field to distinguish between a regular page and a blog post.\n  \"\"\"\n\n  use Phoenix.Component\n  import Beacon.Web.CoreComponents\n  import Ecto.Changeset\n\n  @behaviour Beacon.Content.PageField\n\n  @impl true\n  def name, do: :type\n\n  @impl true\n  def type, do: :string\n\n  @impl true\n  def default, do: \"page\"\n\n  @impl true\n  def render(assigns) do\n    assigns = Map.put(assigns, :opts, [{\"Page\", \"page\"}, {\"Blog Post\", \"blog_post\"}])\n\n    ~H\"\"\"\n    <.input type=\"select\" label=\"Type\" prompt=\"Choose type\" options={@opts} field={@field} />\n    \"\"\"\n  end\n\n  @impl true\n  def changeset(data, attrs, %{page_changeset: _page_changeset}) do\n    cast(data, attrs, [:type])\n  end\nend\n```\n\nNote that the file location and the module name don't need to follow any special convention, it's up to you to organize your code as you see fit.","ref":"create-a-blog.html#custom-field-type"},{"type":"extras","title":"Custom field `tags` - Create a Blog","doc":"A blog post without tags is like a cake without icing, so let's create a custom field to store a list of tags for each post.\n\nCreate a new file `lib/my_app/beacon/page_fields/tags.ex` with the following content:\n\n```elixir\ndefmodule MyApp.Beacon.PageFields.Tags do\n  @moduledoc \"\"\"\n  Custom beacon page field to store tags for a blog post.\n\n  Tags are separated by commas.\n  \"\"\"\n\n  use Phoenix.Component\n  import Beacon.Web.CoreComponents\n  import Ecto.Changeset\n\n  @behaviour Beacon.Content.PageField\n\n  @impl true\n  def name, do: :tags\n\n  @impl true\n  def type, do: :string\n\n  @impl true\n  def default, do: \"2024\"\n\n  @impl true\n  def render(assigns) do\n    ~H\"\"\"\n    <.input type=\"text\" label=\"Tags\" field={@field} />\n    \"\"\"\n  end\n\n  @impl true\n  def changeset(data, attrs, _metadata) do\n    cast(data, attrs, [:tags])\n  end\nend\n```\n\nGreat, now we need to tell Beacon to use those custom page fields. Open the file `lib/my_app/application.ex` and add the following to your site configuration:\n\n```elixir\nextra_page_fields: [\n  MyApp.Beacon.PageFields.Type,\n  MyApp.Beacon.PageFields.Tags\n]\n```\n\nIt should look like this:\n\n```elixir\n{Beacon,\n sites: [\n   [\n     site: :blog,\n     repo: MyApp.Repo,\n     endpoint: MyAppWeb.Endpoint,\n     router: MyAppWeb.Router,\n     extra_page_fields: [\n       MyApp.Beacon.PageFields.Type,\n       MyApp.Beacon.PageFields.Tags\n     ]\n   ]\n ]}\n```","ref":"create-a-blog.html#custom-field-tags"},{"type":"extras","title":"Create the layout - Create a Blog","doc":"It's time to spin up the server, access the admin interface, and create the resources of our site. Execute:\n\n```sh\nmix phx.server\n```\n\nVisit http://localhost:4000/admin/blog/layouts, edit the Default layout, and change the template to:\n\n```heex\n \n   \n     \n      <.page_link path={~p\"/\"} class=\"text-2xl font-bold\">My Blog</.page_link>\n       \n        <.page_link path={~p\"/\"} class=\"text-muted-foreground hover:text-foreground transition-colors\">Blog</.page_link>\n        <.page_link path={~p\"/about\"} class=\"text-muted-foreground hover:text-foreground transition-colors\">About</.page_link>\n        <.page_link path={~p\"/contact\"} class=\"text-muted-foreground hover:text-foreground transition-colors\">Contact</.page_link>\n       \n     \n   \n   \n    <%= @inner_content %>\n   \n\n   \n    <%= @inner_content %>\n   \n \n```\n\nSave and publish the changes.\n\nSome notes about this layout:\n- The `prose` classes are defined by the Tailwind typography plugin and are responsible for improving the look & feel of the pages.\n- Conditionally apply the `prose` classes only to the blog posts, not to the home page.","ref":"create-a-blog.html#create-the-layout"},{"type":"extras","title":"Create the first blog post - Create a Blog","doc":"Visit http://localhost:4000/admin/blog/pages and create a new page with the following data:\n\n- Path: /the-elixir-language\n- Title: The Elixir language\n- Description: What's Elixir, and how and where it can be used.\n- Format: Markdown\n- Type: Blog Post\n- Tags: 2024,eng,elixir\n\nAnd the following template:\n\n````markdown\n# The Elixir language\n\nIn the ever-evolving world of web development, Elixir has emerged as a powerful and innovative language that is transforming the way we build web applications. This blog post explores the unique features and benefits of Elixir, and how it can revolutionize your web development workflow.","ref":"create-a-blog.html#create-the-first-blog-post"},{"type":"extras","title":"The Rise of Elixir - Create a Blog","doc":"Elixir is a dynamic, functional programming language that runs on the Erlang Virtual Machine (BEAM). Inspired by Erlang, Elixir inherits its robust concurrency model, fault tolerance, and scalability, making it an excellent choice for building highly available and scalable web applications.\n\nOne of the key features of Elixir is its focus on functional programming principles. By embracing immutable data structures and concise syntax, Elixir enables developers to write more expressive and maintainable code. This, combined with Elixir's powerful metaprogramming capabilities, allows for the creation of domain-specific languages (DSLs) that can greatly enhance developer productivity.","ref":"create-a-blog.html#the-rise-of-elixir"},{"type":"extras","title":"Elixir and Web Development - Create a Blog","doc":"Elixir's strengths make it an excellent choice for web development. The Phoenix framework, which is often compared to Ruby on Rails, provides a robust and scalable foundation for building web applications. With its focus on real-time communication and fault-tolerance, Elixir and Phoenix are well-suited for building applications that require high availability and low latency, such as chat applications, real-time dashboards, and IoT platforms.\n\nHere's an example of a simple Elixir web application using the Phoenix framework:\n\n```elixir\n# lib/my_app_web/controllers/page_controller.ex\ndefmodule MyAppWeb.PageController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    render(conn, \"index.html\", message: \"Welcome to Elixir!\")\n  end\n```","ref":"create-a-blog.html#elixir-and-web-development"},{"type":"extras","title":"The Future of Elixir in Web Development - Create a Blog","doc":"As Elixir and the Phoenix framework continue to evolve, we can expect to see even more exciting developments in the world of web development. Advancements in areas like real-time data processing, distributed systems, and seamless integration with other technologies will further enhance the capabilities of Elixir-based web applications.\n\nBy embracing Elixir, web developers can unlock new levels of scalability, fault-tolerance, and developer productivity, paving the way for a future where the focus is on building innovative and user-centric web experiences, rather than managing complex infrastructure.\n````\n\nClick on Create Draft Page and Publish. Now visit http://localhost:4000/blog/the-elixir-language to see the result. There is your first blog post!","ref":"create-a-blog.html#the-future-of-elixir-in-web-development"},{"type":"extras","title":"Create the blog index - Create a Blog","doc":"Now that we have a blog post, let's create an index page to list published posts.\n\nFirst, we need to fetch published posts, so let's create a live data assign that exposes data to pages. Visit http://localhost:4000/admin/blog/live_data,\ncreate a new live data for the path `/`, and then create a new assign `most_recent_posts` with the following code:\n\n```elixir\nimport Ecto.Query\n\nBeacon.Content.list_published_pages(\n  :blog,\n  search: fn -> dynamic([q], fragment(\"extra->>'type' = 'blog_post'\"))\nend)\n```\n\nVisit http://localhost:4000/admin/blog/pages and edit the \"My Home Page\" with the following data:\n\n- Title: My Blog\n- Type: Page\n- Tags: leave empty (no tags)\n\nAnd the following template:\n\n```heex\n \n   \n    Welcome to My Blog\n   \n   \n    Discover the latest insights and trends in web development, design, and technology.\n   \n \n\n Latest Blog Posts \n\n \n   \n     \n      <.page_link path={~p\"/#{post}\"} class=\"text-xl font-bold mb-2 block text-primary\">\n        <%= post.title %>\n      </.page_link>\n       \n         \n          <%= tag %>\n         \n       \n       \n        <%= post.description %>\n       \n     \n   \n \n```\n\nSave the changes and publish the page. Visit http://localhost:4000/blog to see the result!","ref":"create-a-blog.html#create-the-blog-index"},{"type":"extras","title":"Custom Repo","doc":"# Custom Repo\n\nRepo is the layer of communication with databases and Beacon requires one to store and fetch data.\n\nBy default, the Beacon installer will infer the repo name and add it to your site configuration. But you may use a custom repo that connects\nto a different database and/or is fine-tuned for your sites.\n\nOpen the file `application.ex`, find the `:repo` option in the site configuration and replace the default repo with your custom repo:\n\n```elixir\nchildren = [\n  ...\n  {Beacon: sites: [[... repo: YourCustomRepo, ...]]}\n]\n```","ref":"custom-repo.html"},{"type":"extras","title":"Customize Markdown Rendering","doc":"# Customize Markdown Rendering\n\nMarkdown pages are rendered using the [Beacon.Template.Markdown](https://hexdocs.pm/beacon/Beacon.Template.Markdown.html) module with a set of default options that works for most cases,\nbut you might want to change how some features work or enable more features. For example, let's suppose you want to generate an ID for every header of the page with the suffix `\"topic-\"`.\n\nYou can do that by changing the `:load_template` lifecycle of `:markdown` in your [site configuration](https://hexdocs.pm/beacon/Beacon.html#start_link/1) as the example below:\n\n```elixir\n[\n  site: :my_site,\n  lifecycle: [\n    load_template: [\n      {:markdown,\n        [\n          markdown_with_header_ids: &markdown_with_header_ids/2\n        ]\n      }\n    ]\n  ]\n  # rest omitted for brevity...\n]\n\ndef markdown_with_header_ids(template, _metadata) do\n  template = MDEx.to_html!(markdown, extension: [header_ids: \"topic-\"])\n  {:cont, template}\nend\n```\n\nBut keep in mind this operation will replace the default markdown rendering with the one you provided,\nso the only features enabled will be the one you set and the others will take the default value.\n\nFor example with this configuration the `:header_ids` extension is enabled but others like tables and autolinks would be turned off.\n\nYou can inspect the actual configuration in the [Beacon.Template.Markdown](https://hexdocs.pm/beacon/Beacon.Template.Markdown.html) module and also check the [MDEx docs](https://hexdocs.pm/mdex/MDEx.html#to_html/2) for more info.","ref":"customize-markdown-rendering.html"},{"type":"extras","title":"Embed tweets","doc":"# Embed tweets\n\nYou'll notice that embedded tweets generated by https://publish.twitter.com usually won't render correctly on your pages,\nand that's due to the fact that the client JavaScript used to load such tweets is executed asyncly along with the LiveView rendering process,\ncausing it to be executed too early. To fix this, we need to load the tweets _after_ LiveView has finished rendering the page.","ref":"embed-tweets.html"},{"type":"extras","title":"Steps - Embed tweets","doc":"","ref":"embed-tweets.html#steps"},{"type":"extras","title":"1. Load the Twitter JS script - Embed tweets","doc":"Find the Layout you're using on the pages where you're embedding tweets, then edit its template to include the following script:\n\n```heex\n \nwindow.twttr = (function(d, s, id) {\n  var js, fjs = d.getElementsByTagName(s)[0],\n    t = window.twttr || {};\n  if (d.getElementById(id)) return t;\n  js = d.createElement(s);\n  js.id = id;\n  js.src = \"https://platform.twitter.com/widgets.js\";\n  fjs.parentNode.insertBefore(js, fjs);\n\n  t._e = [];\n  t.ready = function(f) {\n    t._e.push(f);\n  };\n\n  return t;\n}(document, \"script\", \"twitter-wjs\"));\n\nwindow.addEventListener(\"phx:page-loading-stop\", function(info){\n  twttr.widgets.load()\n});\n \n```\n\nThis will first include the global `twttr` object and then load the tweets as soon as LiveView has finished rendering the page.\n\nVisit https://developer.x.com/en/docs/x-for-websites/javascript-api/guides/set-up-twitter-for-websites for more info.","ref":"embed-tweets.html#1-load-the-twitter-js-script"},{"type":"extras","title":"2. Embed tweets - Embed tweets","doc":"Now you can embed tweets on your pages _but_ remember to remove the ` ` that's included at the end of the embedded code,\nas we're already loading the Twitter JS script in the Layout template.\n\nUsually it looks like this:\n\n```html\n ><!-- content omitted -->    \n```\n\nRemove the ` ` tag at the end of the embedded tweet to look like this:\n\n```html\n ><!-- content omitted --> \n```","ref":"embed-tweets.html#2-embed-tweets"},{"type":"extras","title":"Heroicons","doc":"# Heroicons\n\n[Heroicons](https://heroicons.com/) is a popular icon library added by default in new Phoenix applications and also available in Beacon as a component.\n\nThe component `heroicon` is added by default in new Beacon sites without extra configuration to let you render icons in your pages.\n\nFor example, to render a `light-bulb` icon in the solid style:\n\n```heex\n<.heroicon name=\"light-bulb\" solid />\n```","ref":"heroicons.html"},{"type":"extras","title":"JavaScript interoperability","doc":"# JavaScript interoperability\n\nBeacon provides support for custom client-side JavaScript via LiveView's [JS Hooks](https://hexdocs.pm/phoenix_live_view/js-interop.html#client-hooks-via-phx-hook).\n\nHooks are blocks of custom JS code which run when an element in a Beacon template reaches a specific\nstage of its lifecycle.  There are six possible stages which you can use:\n\n  * `mounted` - the element has been added to the DOM and its server\n    LiveView has finished mounting\n  * `beforeUpdate` - the element is about to be updated in the DOM.\n    *Note*: any call here must be synchronous as the operation cannot\n    be deferred or cancelled.\n  * `updated` - the element has been updated in the DOM by the server\n  * `destroyed` - the element has been removed from the page, either\n    by a parent update, or by the parent being removed entirely\n  * `disconnected` - the element's parent LiveView has disconnected from the server\n  * `reconnected` - the element's parent LiveView has reconnected to the server\n\nTo add code to one or more of these, first go into your BeaconLiveAdmin dashboard and navigate to the \n\"JS Hooks\" section.  Create a new hook with the name of your choice, and you will see a code editor\nwith each of the above callbacks.  For an example, let's add a log message to output to the console when\nour page's logo is mounted:\n\n```javascript\nmounted() {\n  console.log(\"logo mounted!\");\n}\n```\n\nThen in the page's template, add a `phx-hook` attr to the desired element:\n```html\n \n   \n \n```\n\n`MyHookName` should be replaced with whatever name you chose for your hook.  Also note the element\nmust have an `id` attr defined.\n\nWith these changes saved and the page published, you can open that page in your browser and should see\n`logo mounted!` in the javascript console.","ref":"javascript-interoperability.html"},{"type":"extras","title":"Libraries in JS Hooks","doc":"# Libraries in JS Hooks\n\nIn this guide we'll load a JS library from CDN to use in JS Hooks.\n\nBeacon supports user-defined JS Hooks created and maintained in the Beacon LiveAdmin interface,\nand usually we need to load and call external JavaScript libraries in those hooks.\n\nCurrently it's not possible to directly `import` libraries defined in your application's `package.json` file,\nbut an alternative is to load libraries from a CDN as https://unpkg.com\n\nLet's start by editing a layout or a page where you want to load a library.\nFor this guide we'll use [Tippy.js](https://atomiks.github.io/tippyjs) as example,\nso include these lines in the template:\n\n```heex\n  \n  \n```\n\nThose scripts will make `window.tippy` available in your page,\nwhich you can then use in your JS Hook. Let's create it now.\n\nGo to the JS Hooks page in Admin, create a new hooke named `tippy`\nand input the following code:\n\n```js\nexport const tippy = {\n  mounted() {\n    window.tippy(this.el, {content: 'My message'})\n  }\n}\n```\n\nNow whenever you want to use that hook, just include the `phx-hook=\"tippy\"` property in an element,\neither in the page you loaded the scripts or any page that use the layout that contains those scripts.\nFor example in a button to show a tooltip in a button:\n\n```heex\n Sign Up \n```","ref":"libraries-in-js-hooks.html"},{"type":"extras","title":"Multiple Domains/Tenants","doc":"# Multiple Domains/Tenants\n\nYou can host multiple domains or subdomains with Phoenix by using the `:host` option of the scope function in your router:\n\n`:host` - a string or list of strings containing the host scope, or prefix host scope (e.g. `\"foo.bar.com\"`, `\"foo.\"`)\n\n```elixir\n# match `admin.` subdomain\nscope \"/\", MyAppWeb, host: \"admin.\" do\n  live \"/\", AdminLive, :new\nend\n\n# match `example.com` and `example2.com`\nscope \"/\", MyAppWeb, host: [\"example.com\", \"example2.com\"] do\n  live \"/\", LandingLive, :new\nend\n\n# match only `my-example.org`\nscope \"/\", MyAppWeb, host: \"my-example.org\" do\n  live \"/\", HomeLive, :new\nend\n```","ref":"multiple-domains-hosting.html"},{"type":"extras","title":"Multiple Domains/Tenants hosting with BeaconCMS - Multiple Domains/Tenants","doc":"If you need to host multiple domains or subdomains with Beacon, you can still use the same `:host` option described above!\n\nHere's another example, but now using Beacon:\n\n```elixir\n# serve the `:demo` site at demo.org/demo\nscope \"/\", host: \"demo.org\" do\n  pipe_through :browser\n\n  beacon_site \"/demo\", site: :demo\nend\n\n# serve the `:blog` site at blog.com\nscope \"/\", host: \"blog.com\" do\n  pipe_through :browser\n\n  beacon_site \"/\", site: :blog\nend\n\n# serve the admin interface at the prefix /admin on the root domain\nscope \"/admin\" do\n  pipe_through :browser\n  beacon_live_admin \"/\"\nend\n```\n\nYou also need to pass the `:check_origin` option when configuring your endpoint, in order to explicitly outline which origins are allowed.\n\nIn `config/runtime.exs` edit the following config to include your domains:\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  # ...\n  check_origin: [\n    \"https://beacon-demo.com/\",\n    \"https://demo.org\",\n    \"https://blog.com\"\n  ]\n```\n\nCheck out the [Deployment Topologies](https://hexdocs.pm/beacon/deployment-topologies.html) guide for more information.","ref":"multiple-domains-hosting.html#multiple-domains-tenants-hosting-with-beaconcms"},{"type":"extras","title":"Notify Page Published","doc":"# Notify Page Published\n\nLet's suppose you want to notify a group of people via email when a page is published and your Phoenix application is already integrated with a mailer.\n\nBeacon provides some lifecycle hooks that you can use to inject custom logic when something happens, in this case, when a page is published the hook `:after_publish_page` is triggered.\n\nYou can check out the full documentation and the list of available in the `Beacon.Config` module. This is the spec for the `:after_publish_page` hook:\n\n```elixir\n{:after_publish_page, [{identifier :: atom(), fun :: (Content.Page.t() -> {:cont, Content.Page.t()} | {:halt, Exception.t()})}]}\n```\n\nIt may look a bit complex, but it's quite simple. You need to define a function that receives a `Content.Page.t()` struct and returns either\n`{:cont, Content.Page.t()}` to continue the process or `{:halt, Exception.t()}` to stop it. And the identifier is just a unique name for your custom logic.\n\nFor our example, the site config would look like:\n\n```elixir\nlifecycle: [\n  after_publish_page: [\n    notify_page_published: &MyApp.CMS.notify_page_published/1\n  ]\n]\n```\n\nAnd the corresponding function in `MyApp.CMS` module:\n\n```elixir\ndefmodule MyApp.CMS do\n  def notify_page_published(%Beacon.Content.Page{path: path} = page) do\n    email =  MyApp.CMS.notify_email(%{path: path})\n\n    case MyApp.Mailer.deliver(email) do\n      {:ok, _} ->\n        {:cont, page}\n\n      {:error, reason} ->\n        message = \"\"\"\n        failed to notify that page #{path} was published\n\n        Got:\n\n          #{inspect(reason)}\n        \"\"\"\n\n        # or use a custom exception\n        {:halt, %RuntimeError{message: message}}\n    end\n  end\nend\n```","ref":"notify-page-published.html"},{"type":"extras","title":"Handle async and continuous updates with handle_info and on_mount","doc":"# Handle async and continuous updates with handle_info and on_mount\n\nUsing [Live Data](https://hexdocs.pm/beacon/Beacon.Content.html#create_live_data/1) assigns, [on_mount](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#on_mount/1) callbacks, and corresponding [handle_info](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#c:handle_info/2) handlers is a pattern that you can leverage to achieve optimistic UI for your pages.\n\nSuppose you're displaying a Weather widget on your page but fetching the data takes a second, delaying the whole page, even though that widget is not critical to your page and could display dummy data (or no data at all) while the rest of the page is loaded. In this scenario, a Live Data assign would provide the initial dummy data, the on_mount hook request the data to be loaded asyncly, and finally the handle_info loads the data and update that assign.\n\nFor this example, we'll build a similar scenario but also add a continuous monitor to update the assign frequently.\n\nIn this example we're going to update a `DateTime` every `1_000` milliseconds from the server, starting that loop `on_mount`, then continuously looping over a `handle_info`.","ref":"on_mount_handle_info_loop.html"},{"type":"extras","title":"Creating the On Mount Handler Module - Handle async and continuous updates with handle_info and on_mount","doc":"After completing [your first site](../introduction/your-first-site.md) guide, create a module to handle\nyour `on_mount` callbacks. This is an example, so find a place that works best\nfor your project structure. Here, we're creating a `my_site_on_mount.ex` file\nin our web path.\n\n```bash\n├── lib\n│   ├── my_site\n│   ├── my_site_web\n│   │   ├── components\n│   │   │   ├── core_components.ex\n│   │   │   ├── layouts\n│   │   │   │   ├── app.html.heex\n│   │   │   │   └── root.html.heex\n│   │   │   ├── callbacks\n│   │   │   │   └── my_site_on_mount.ex\n│   │   │   └── layouts.ex\n│   │   ├── controllers\n│   │   │   ├── error_html.ex\n│   │   │   ├── error_json.ex\n│   │   │   ├── page_controller.ex\n│   │   │   ├── page_html\n│   │   │   │   └── home.html.heex\n│   │   │   └── page_html.ex\n│   │   ├── endpoint.ex\n│   │   ├── gettext.ex\n│   │   ├── router.ex\n│   │   └── telemetry.ex\n│   └── my_site_web.ex\n```\n\nWhen the socket is connected, we will send a message to `self()` in 1_000 milliseconds\nto `:update_current_time`.\n\n```elixir\ndefmodule MySiteWeb.OnMount do\n  use MySiteWeb, :live_view\n\n  def on_mount(_path, _params, _session, socket) do\n    if connected?(socket) do\n      Process.send_after(self(), :update_current_time, 1_000)\n    end\n\n    {:cont, socket}\n  end\nend\n```","ref":"on_mount_handle_info_loop.html#creating-the-on-mount-handler-module"},{"type":"extras","title":"Updating the Live Session - Handle async and continuous updates with handle_info and on_mount","doc":"In your `router.ex` file, add the `on_mount` option to your `my_site` route so it is handled in the live session.\n\n```elixir\nscope \"/\" do\n    pipe_through :browser\n    beacon_site \"/\",\n      site: :my_site,\n      on_mount: {MySiteWeb.OnMount, []}\n  end\n```","ref":"on_mount_handle_info_loop.html#updating-the-live-session"},{"type":"extras","title":"Accessing LiveAdmin to manage your site - Handle async and continuous updates with handle_info and on_mount","doc":"Let's customize the home page. Visit http://localhost:4000/admin and you should see the `my_site` that you just created listed on the admin interface.\n\nNow let's create the resources for our first site, and we'll starting by creating the resources used by the home page before we customize its template.","ref":"on_mount_handle_info_loop.html#accessing-liveadmin-to-manage-your-site"},{"type":"extras","title":"Live Data - Handle async and continuous updates with handle_info and on_mount","doc":"We want to display the current time on the home page as the assign `<%= @current_time %>`. In Beacon we use Live Data to create and modify assigns at runtime, let's do it.\n\nGo to http://localhost:4000/admin/my_site/live_data and navigate to the \"/\" path and create a new assign named `current_time` using the `elixir` format with the following value:\n\n```elixir\nDateTime.now!(\"Etc/UTC\")\n```\n\nSave the changes.","ref":"on_mount_handle_info_loop.html#live-data"},{"type":"extras","title":"Handle the :update_current_time info callback - Handle async and continuous updates with handle_info and on_mount","doc":"Besides displaying some data, we will have a form to subscribe people to our newsletter, so we need to react to that event. Let's create an event handler for that.\n\nGo to http://localhost:4000/admin/my_site/info_handlers and create a new event handler named `update_current_time` with the following content:\n\n```elixir\nProcess.send_after(self(), :update_current_time, 1000)\n\n{:noreply, assign(socket, current_time: DateTime.now!(\"Etc/UTC\"))}\n```","ref":"on_mount_handle_info_loop.html#handle-the-update_current_time-info-callback"},{"type":"extras","title":"Update the home page template - Handle async and continuous updates with handle_info and on_mount","doc":"Finally, let's update the home page template to make use of the info handler we created.\n\nGo to http://localhost:4000/admin/my_site/pages and you should see a page already created for the `/` path. We're going to change it.\n\nEdit the template to replace with this content:\n\n```heex\n \n    \n   \n     \n      <.link patch=\"/\" class=\"flex items-center gap-2\">\n         \n            \n            \n         \n         CMS Platform \n      </.link>\n     \n     \n      <.link patch=\"/blog\" class=\"hidden md:inline-flex text-sm font-medium hover:underline\">\n        Blog\n      </.link>\n       \n        Sign In\n       \n     \n   \n   \n     \n       \n         \n           \n             \n              Unlock the power\n             \n            of your content with our CMS platform\n           \n           \n            Streamline your content management with our intuitive, high-performance CMS built on Phoenix LiveView.\n           \n           \n             \n               \n               \n                Join Waitlist\n               \n             \n           \n           \n            Congrats! You joined the watchlist.\n           \n         \n         \n          <.image\n            site={@beacon.site}\n            name=\"image.webp\"\n            alt=\"My Page Image\"\n            width=\"600\"\n            height=\"600\"\n            style=\"aspect-ratio: 600 / 600; object-fit: cover;\"\n          />\n         \n       \n     \n   \n   \n     \n       ©<%= @current_time %> <%= @current_year %> CMS Platform. All rights reserved. \n       \n         \n           \n              \n           \n           Facebook \n         \n         \n           \n             \n             \n           \n           Twitter \n         \n         \n           \n              \n              \n              \n           \n           Instagram \n         \n         \n           \n             \n             \n              \n              \n           \n           LinkedIn \n         \n         \n          Privacy\n         \n         \n          Terms\n         \n         \n          Contact\n         \n       \n     \n   \n \n```\n\nSave the changes and Publish the page. Go to http://localhost:4000 to check the final result!\n\nYou should see `DateTime` updating every `1_000` milliseconds from the server.","ref":"on_mount_handle_info_loop.html#update-the-home-page-template"},{"type":"extras","title":"Protect Pages with Basic Auth","doc":"# Protect Pages with Basic Auth\n\nIf you want to protect your app with a basic username and password protection, you can easily do so by using the [`Plug.BasicAuth`](https://hexdocs.pm/plug/Plug.BasicAuth.html) plug.\n\nPlugs in Phoenix are basically functions that get a `Plug.Conn` instance (which is basically _the whole universe_ of your app's request) on any web request and returns a slightly modified instance.\nBecause every request goes through these sequence of plugs, it's a great place to set up authentication, and possibly avoid any requests to pass through if not authenticated.","ref":"protect-pages-with-basic-auth.html"},{"type":"extras","title":"Create an authentication plug - Protect Pages with Basic Auth","doc":"You can build your own module plug that gets the _conn_, verifies for authentication with `Plug.BasicAuth`, and then returns the transformed _conn_.\n\n```elixir\ndefmodule MyWebApp.Plugs.SiteBasicAuth do\n  @moduledoc false\n  @behaviour Plug\n\n  def init(opts), do: opts\n\n  def call(conn, _opts) do\n    Plug.BasicAuth.basic_auth(conn, username: \"admin\", password: \"protected123\")\n  end\nend\n```\n\nThis is as easy as it could be. Let's break it down:\n- We've created a module plug\n- A plug always needs a `init` and a `call` definition\n- We don't want to do anything on initialization, so we just make `init` a passthrough function\n- On `call`, we let the `Plug.BasicAuth.basic_auth` do it's thing, which means it will check for existing authentication and otherwise popup a prompt to authenticate. It will return the (transformed) `conn`, so nothing we need to do there ourselves","ref":"protect-pages-with-basic-auth.html#create-an-authentication-plug"},{"type":"extras","title":"Use the plug in your router - Protect Pages with Basic Auth","doc":"To use the plug we just created, we have to implement it in our router file, so it will be invoked on each request.\nIf you want to add this authentication to any part of your app, add it to the pipeline of the scope that covers your Beacon app:\n\n```elixir\nscope \"/\" do\n  pipe_through [:browser, MyWebApp.Plugs.SiteBasicAuth]\n  beacon_site \"/\", site: :my_site\nend\n```\n\nAnd that's it! Run your app and go to any page. It should popup a username and password prompt in order to see your app.\n\n# Specific pages authentication\n\nThe previous example protects your whole app. In some cases you might only want to protect a few specific pages of your app.\nAs Beacon is handling all routing of a scope, we cannot just add a router's scope to cover this, because it will always go through the default scope where you define the `beacon_site` as a \"catch-all\".\nYou can change your existing plug (or create a separate one) to do a check on what page is being requested (we're using `conn.request_path`).\nFor hardcoded paths, we can use a guard to only match when it's one of your protected pages, and keep a fallback / default `call` definition for all the other pages that shouldn't do anything.\n\n\n```elixir\ndefmodule MyWebApp.Plugs.ProtectedPages do\n  @moduledoc false\n  @behaviour Plug\n\n  @protected_pages [\n    \"/your/protected/page\",\n    \"/another/secret\"\n  ]\n\n  def init(opts), do: opts\n\n  def call(conn, _opts) when conn.request_path in @protected_pages do\n    Plug.BasicAuth.basic_auth(conn, username: \"admin\", password: \"protected123\")\n  end\n\n  def call(conn, _opts), do: conn\nend\n```\n\nNote: this example is just for easy hardcoded authentication. For more advanced authentication you could extend this approach in many ways, for example by setting an environment variable for the username and password, or even setting up a database with users.\nSame for the protected pages which could be stored somewhere else, or might be more dynamic or having a more complicated check on what the path looks like.","ref":"protect-pages-with-basic-auth.html#use-the-plug-in-your-router"},{"type":"extras","title":"Reuse app.css","doc":"# Reuse app.css\n\nYour Phoenix application has an `app.css` that you might want to use to generate styles for your site,\nand in this guide, we'll see how to configure your site to reuse the same styles for your application and sites.\n\nIn your site config, usually located at `runtime.exs`, you should add a new config `:tailwind_css`\nto let Beacon know where to find the `app.css` file:\n\n```elixir\ntailwind_css =\n  if config_env() == :prod do\n    Path.expand(\"../../app.css\", __DIR__)\n  else\n    Path.expand(\"../assets/css/app.css\", __DIR__)\n  end\n\nconfig :beacon,\n  my_site: [\n    site: :my_site,\n    repo: MyApp.Repo,\n    endpoint: MyAppWeb.Endpoint,\n    router: MyAppWeb.Router,\n    tailwind_css: tailwind_css\n  ]\n```\n\nNote the path is different for development and production environments because deployment usually are packed as releases.","ref":"reuse-app-css.html"},{"type":"extras","title":"Tailwind Setup","doc":"# Tailwind Setup\n\nBeacon has built-in TailwindCSS support, any page can use its classes out of the box and a stylesheet will be automatically generated and served.\n\nThat default configuration is already bundled in the Beacon package, so you can skip this guide if that suits your needs.\n\nOtherwise, keep reading to learn how to set up a custom configuration with more advanced features such as custom plugins, themes, and more.\n\n**Note** that the Tailwind configuration must respect some constraints to work properly with Beacon,\nso if you want to reuse an existing configuration make sure to follow the steps below and make the necessary adjustments.\nIt might be a good idea to keep separate configs, one for your application and another one for Beacon sites, and reuse\nparts that are common between them.","ref":"tailwind-setup.html"},{"type":"extras","title":"Objective - Tailwind Setup","doc":"Make sure the proper Tailwind version is installed, create a valid Tailwind config in the ESM format, and bundle everything together into a single package.","ref":"tailwind-setup.html#objective"},{"type":"extras","title":"Prerequisites - Tailwind Setup","doc":"A site must be already configured. Otherwise, follow the [Your First Site](https://hexdocs.pm/beacon/your-first-site.html) guide first.","ref":"tailwind-setup.html#prerequisites"},{"type":"extras","title":"Constraints - Tailwind Setup","doc":"Since Beacon uses the same configuration to generate stylesheets for your sites and also to preview pages on the Visual Editor in the browser,\nthat configuration must respect some constraints to work properly in both environments:\n\n  - [Use the ESM format](https://tailwindcss.com/blog/tailwindcss-v3-3#esm-and-type-script-support)\n  - Can't call node APIs such as `require(\"fs\")` and `require(\"path\")`","ref":"tailwind-setup.html#constraints"},{"type":"extras","title":"Steps - Tailwind Setup","doc":"","ref":"tailwind-setup.html#steps"},{"type":"extras","title":"Tailwind v3.3.0 or higher - Tailwind Setup","doc":"Any recent Phoenix application should have the [tailwind](https://hex.pm/packages/tailwind) library already installed and updated but let's double check, execute:\n\n```sh\nmix run -e \"IO.inspect Tailwind.bin_version()\"\n```\n\nIf it fails or the version is lower than 3.3.0 then follow the [tailwind install guide](https://github.com/phoenixframework/tailwind?tab=readme-ov-file#installation)\nto get it installed or updated. It's required to install a recent Tailwind version higher than 3.3.0","ref":"tailwind-setup.html#tailwind-v3-3-0-or-higher"},{"type":"extras","title":"Esbuild - Tailwind Setup","doc":"Similar to Tailwind, any recent Phoenix application should have it installed already but let's check by executing:\n\n```sh\nmix run -e \"IO.inspect Esbuild.bin_version()\"\n```\n\nIf it fails then follow the [esbuild install guide](https://github.com/phoenixframework/esbuild?tab=readme-ov-file#installation) to get it installed.\nAny recent version that is installed should work just fine.","ref":"tailwind-setup.html#esbuild"},{"type":"extras","title":"Install plugins - Tailwind Setup","doc":"The config we'll generate imports the plugins `@tailwindcss/forms` and `@tailwindcss/typography` so they must be installed before we can actually generate the bundle file.\nExecute at the root of your project:\n\n```sh\nnpm install --prefix assets --save tailwindcss@3 @tailwindcss/forms @tailwindcss/typography\n```","ref":"tailwind-setup.html#install-plugins"},{"type":"extras","title":"Create a new valid config file and update site config - Tailwind Setup","doc":"Beacon provides a generator that will create and change the files needed to set up a custom Tailwind configuration.\n\nExecute at the root of your project and follow the prompts:\n\n```sh\nmix beacon.gen.tailwind_config --site my_site\n```\n\n_Replace `my_site` with the name of the site you want to install the custom Tailwind configuration._\n\nThe generated config file is in the ESM format, ie: it exports the config as `default export` instead of `module.exports`,\nand it doesn't use node APIs like `fs` or `path`, so it can't bundle heroicons as the default Phoenix config does,\nbut Beacon provides a component to render such icons, see the [Heroicons guide](../recipes/heroicons.md) for more information.","ref":"tailwind-setup.html#create-a-new-valid-config-file-and-update-site-config"},{"type":"extras","title":"Testing","doc":"# Testing\n\nIntegrating your app with Beacon might require testing some parts of that integration,\nso in this recipe let's suppose you have added a hook in the publish page lifecycle as described in the [Notify Page Published](notify-page-published.md) recipe,\nand now you want to make sure the email is being sent when a page is published, as it was defined in the hook.","ref":"testing.html"},{"type":"extras","title":"Config - Testing","doc":"First you should enable the `:testing` mode for each site you want to test:\n\nAssuming your site configuration looks like:\n\n```elixir\n@impl Application\ndef start(_type, _args) do\n  children = [\n    # ... omitted for brevity\n    {Beacon, sites: [Application.fetch_env!(:my_app, :my_site)]}\n  ]\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nThen open the file `config/test.exs` and add the following config to change the Beacon mode:\n\n```elixir\n# config/test.exs\nconfig :my_app, :my_site, mode: :testing\n```","ref":"testing.html#config"},{"type":"extras","title":"Test - Testing","doc":"And the actual test would look like the example below.\n\nFirst we create and publish a page by calling the [fixture](https://hexdocs.pm/beacon/Beacon.Test.Fixtures.html) `beacon_published_page_fixture/0`,\nthen we build and assert the correct email was delivered, using [assert_delivered_email/1](https://hexdocs.pm/bamboo/Bamboo.Test.html#assert_delivered_email/2)\nfrom the Bamboo library.\n\n```elixir\ndefmodule MyApp.CMS.NotificationEmailTest do\n  use ExUnit.Case\n  use Bamboo.Test\n  use Beacon.Test, site: :my_site # <- use this module to enable testing utilities\n\n  test \"notify via email when a page is published\" do\n    page = beacon_published_page_fixture()\n    expected_email = MyApp.CMS.notify_email(page)\n    assert_delivered_email expected_email\n  end\nend\n```","ref":"testing.html#test"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting\n\nSolutions to common problems.","ref":"troubleshooting.html"},{"type":"extras","title":"Server crashes due to missing tailwind or tailwind fails to generate stylesheets - Troubleshooting","doc":"**Possible causes:**\n- Tailwind library not installed\n- Outdated tailwind version\n- Missing tailwind binary\n- Invalid tailwind configuration\n\nAny recent Phoenix application should come with the [tailwind](https://hex.pm/packages/tailwind) library already installed and updated\nso let's check if everything is in place. Execute:\n\n```sh\nmix run -e \"IO.inspect Tailwind.bin_version()\"\n```\n\nIt should display `{:ok, \"3.4.3\"}` (or any other version).\n\nIf it fails or the version is lower than **3.3.0** then follow the [tailwind install guide](https://github.com/phoenixframework/tailwind?tab=readme-ov-file#installation)\nto get it installed or updated. It's important to install a recent Tailwind version higher than 3.3.0","ref":"troubleshooting.html#server-crashes-due-to-missing-tailwind-or-tailwind-fails-to-generate-stylesheets"},{"type":"extras","title":"Site not booting because it's not reachable - Troubleshooting","doc":"Depending on the [deployment topology](https://hexdocs.pm/beacon/deployment-topologies.html) and your router configuration,\na site prefix can never match and it will never receive requests.\n\nThat's is not necessarily an error if you have multiple sites in the same project and is using the router scope `:host` option\nto selectively start sites based on the current host.\n\nThe most common causes for this error are: \n\n1. Mismatch between the Endpoint host/ip config and the route scope `:host` \n\nSuppose your Endpoint configuration is:\n\n```elixir\nconfig :my_app,\n       MyAppWeb.BlogEndpoint,\n       http: [ip: {127, 0, 0, 1}, port: 4586],\n       ...\n```\n\nAnd in your Router:\n\n```elixir\nscope \"/blog\", alias: MyAppWeb, host: [\"blog.mysite.com\"] do\n  pipe_through [:browser, :beacon]\n  beacon_site \"/\", site: :blog\nend\n```\n\nNote that `beacon_site` can only match requests coming from `\"blog.mysite.com\"` but\nyour site endpoint is binding to localhost, so it will never match. To fix it just\nadd `\"localhost\"` in the `:host` scope list:\n\n```elixir\nscope \"/blog\", alias: MyAppWeb, host: [\"blog.mysite.com\", \"localhost\"] do\n```\n\n2. Incorrect routes order\n\nThe macro `beacon_site` is a catch-all route defined as `/*` so you have to place\nthe most specific routes first, eg:\n\n```elixir\nscope \"/\", alias: MyAppWeb, host: [\"blog.mysite.com\"] do\n  pipe_through [:browser, :beacon]\n  beacon_site \"/blog\", site: :blog\n  beacon_site \"/\", site: :my_site\nend\n```\n\nThe opposite would not work, the `:blog` would never match if placed after the `:my_site` site.\n\n3. Missing `use Beacon.Router` and/or missing `beacon_site` in your app's router file.\n\nMisconfiguration may also cause that error.\n\nAlso check the [Beacon.Router](https://hexdocs.pm/beacon/Beacon.Router.html) for more information.","ref":"troubleshooting.html#site-not-booting-because-it-s-not-reachable"},{"type":"extras","title":"Could not resolve \"tailwindcss/plugin\" - Troubleshooting","doc":"Usually caused by having installed Tailwind v4 instead of Tailwind v3\n\nTailwind v4 doesn't have the `plugin` module and is not supported yet,\nso make sure your `package.json` file specify `tailwindcss` 3.x","ref":"troubleshooting.html#could-not-resolve-tailwindcss-plugin"},{"type":"extras","title":"Deploy to Fly.io","doc":"# Deploy to Fly.io\n\nOnce you have a [Beacon site up and running](../introduction/your-first-site.md) locally, you can deploy it on [Fly.io](https://fly.io) by following this guide.","ref":"deploy-to-flyio.html"},{"type":"extras","title":"Fly.io CLI - Deploy to Fly.io","doc":"First, install the fly cli tool, as described at [Install flyctl](https://fly.io/docs/hands-on/install-flyctl). You'll need it to deploy your site.","ref":"deploy-to-flyio.html#fly-io-cli"},{"type":"extras","title":"Sign in or sign up - Deploy to Fly.io","doc":"Don't have an account yet? Sign up by running:\n\n```sh\nfly auth signup\n```\n\nOr sign in to your existing account:\n\n```sh\nfly auth login\n```","ref":"deploy-to-flyio.html#sign-in-or-sign-up"},{"type":"extras","title":"Dockerfile - Deploy to Fly.io","doc":"Fly applications run in containers. Let's generate a Dockerfile and make a couple of changes:\n\nRun:\n\n```sh\nmix phx.gen.release --docker\n```\n\nEdit the generated `Dockerfile` and make some changes:\n\n1. Install `npm` by adding it to the `apt-get install` list\n\nIt should look like this:\n\n```dockerfile\nRUN apt-get update -y && apt-get install -y build-essential git npm \\\n```\n\n2. Add the following code before `RUN mix assets.deploy`:\n\n```dockerfile\nRUN npm install --prefix assets\n```\n\n3. Add the following code before `RUN mix release`:\n\n```dockerfile\nRUN mix tailwind.install --no-assets --if-missing\nRUN mix esbuild.install --if-missing\n```","ref":"deploy-to-flyio.html#dockerfile"},{"type":"extras","title":"Copy files into the release - Deploy to Fly.io","doc":"Open the file `mix.exs` and locate the `project/0` function. Add a new `:releases` config that contains a custom step to copy Beacon files:\n\n```elixir\nreleases: [\n  my_app: [\n    steps: [:assemble, &copy_beacon_files/1]\n  ]\n]\n```\n\nReplace `my_app` with your actual app name. The whole function will look similar to this:\n\n```elixir\ndef project do\n  [\n    app: :my_app,\n    version: \"0.1.0\",\n    elixir: \"~> 1.14\",\n    elixirc_paths: elixirc_paths(Mix.env()),\n    start_permanent: Mix.env() == :prod,\n    consolidate_protocols: Mix.env() != :dev,\n    aliases: aliases(),\n    deps: deps(),\n    releases: [\n      my_app: [\n        steps: [:assemble, &copy_beacon_files/1]\n      ]\n    ]\n  ]\nend\n```\n\nNow create a new function `copy_beacon_files/1` at any place in the `mix.exs` file:\n\n```elixir\ndefp copy_beacon_files(%{path: path} = release) do\n  build_path = Path.join([path, \"bin\", \"_build\"])\n  File.mkdir_p!(build_path)\n\n  copy_bin! = fn name, pattern, cmd ->\n    case Path.wildcard(pattern) do\n      [] ->\n        raise \"\"\"\n        #{name} binary not found in the release package\n\n        You should execute the following command to download it:\n\n            #{cmd}\n\n        Note the binary must be present in the environment where the\n        release is generated, either locally or in the Dockerfile for example.\n\n        \"\"\"\n\n      bin_path ->\n        for file <- bin_path do\n          File.cp!(file, Path.join(build_path, Path.basename(file)))\n        end\n    end\n  end\n\n  copy_bin!.(\"tailwind\", \"_build/tailwind-*\", \"mix tailwind.install --no-assets\")\n  copy_bin!.(\"esbuild\", \"_build/esbuild-*\", \"mix esbuild.install\")\n\n  File.cp!(Path.join([\"assets\", \"css\", \"app.css\"]), Path.join(path, \"app.css\"))\n\n  release\nend\n```\n\nEssentially this function will copy the `tailwind-cli` and `esbuild` binaries, and the `app.css` files into the release.\n\nNote that both `tailwind-cli` and `esbuild` are required but the `app.css` file is actually optional,\nit's only used if you [reuse it on your sites](reuse-app-css.md), so you can adjust `copy_beacon_files/1` for your own project needs.\n\nSee  https://hexdocs.pm/mix/Mix.Tasks.Release.html for more info about releases configuration.","ref":"deploy-to-flyio.html#copy-files-into-the-release"},{"type":"extras","title":"Launch - Deploy to Fly.io","doc":"With your account in place and all files updated, it's time to launch your application. Run:\n\n```sh\nfly launch\n```\n\nWhen asked if you would like to set up a PostgreSQL database, answer YES and choose the most appropriate configuration for your site.\n\nWhen asked if you would like to deploy, answer YES or run `fly deploy` afterward when you're ready to deploy.","ref":"deploy-to-flyio.html#launch"},{"type":"extras","title":"Deploy - Deploy to Fly.io","doc":"Beacon is designed to minimize deployments as much as possible, but eventually, you can trigger new deployments by running:\n\n```sh\nfly deploy\n```","ref":"deploy-to-flyio.html#deploy"},{"type":"extras","title":"Open - Deploy to Fly.io","doc":"Finally, if you followed the guides to set up your site, run the following command to see it live:\n\n```sh\nfly open /\n```\n\nIf you have created a custom page, simply replace `/` in the above command to match its prefix.","ref":"deploy-to-flyio.html#open"},{"type":"extras","title":"Certificates - Deploy to Fly.io","doc":"If the site was [generated with a custom host](https://hexdocs.pm/beacon/Mix.Tasks.Beacon.Gen.Site.html),\nyou'll need a certificate to access that domain:\n\n```sh\nfly certs add your-domain.com\n```\n\nSee the [fly certs](https://fly.io/docs/flyctl/certs/) doc for more info.","ref":"deploy-to-flyio.html#certificates"},{"type":"extras","title":"More commands - Deploy to Fly.io","doc":"You can find all available commands in the [Fly.io docs](https://fly.io/docs/flyctl) and also find more tips on the official [Phoenix Deploying on Fly.io guide](https://fly.io/docs/elixir/getting-started/).","ref":"deploy-to-flyio.html#more-commands"},{"type":"extras","title":"Troubleshooting - Deploy to Fly.io","doc":"The default config file `fly.toml` created by `fly launch` defines `min_machines_running = 0` so Fly will auto-stop machines\nthat receive no traffic for a period of time. You might want to change this value to `1` otherwise it will look like your app\nis not working, when in fact it's just Fly proxy doing its job.","ref":"deploy-to-flyio.html#troubleshooting"},{"type":"extras","title":"Deployment Topologies","doc":"# Deployment Topologies\n\nThe routing system in Phoenix combined with the OTP distribution model opens many possibilities for deployment strategies and in this guide we'll cover the most common ones.\n\nCode examples might be abbreviated and infrastructure details like load balancers are not covered in this guide to keep it short.\n\nClustering your application is also not covered in this guide but you can find the documentation on the platform's docs,\nfor example https://fly.io/docs/elixir/the-basics/clustering and https://www.gigalixir.com/docs/cluster.","ref":"deployment-topologies.html"},{"type":"extras","title":"Core Concepts - Deployment Topologies","doc":"Firstly, we need to understand a few core concepts:","ref":"deployment-topologies.html#core-concepts"},{"type":"extras","title":"URL Generation - Deployment Topologies","doc":"The URL of a Beacon Page inside a Phoenix Router [scope](https://hexdocs.pm/phoenix/Phoenix.Router.html#scope/2)\nis generated by the combination of the Endpoint and the Router configuration as `endpoint host` + `scope path` + `site prefix` + `page path`, for example:\n\n```elixir\n# endpoint\nconfig :my_app, MyAppWeb.Endpoint, url: [host: \"mysite.com\"]\n\n# router scope\nscope \"/2024\" do\n  beacon_site \"/campaigns\"\nend\n\n# beacon page\n%Beacon.Content.Page{path: \"christmas\"}\n```\n\nResults in:\n\n```\nhttp://mysite.com/2024/campaigns/christmas\n^                 ^    ^         ^\n|                 |    |         |\nendpoint          |    |         page path\n                  |    site prefix\n                  scope path\n```\n\nSee [Beacon.Router](https://hexdocs.pm/beacon/Beacon.Router.html) and [Phoenix.Router](https://hexdocs.pm/phoenix/Phoenix.Router.html) for added context.","ref":"deployment-topologies.html#url-generation"},{"type":"extras","title":"URL Matching - Deployment Topologies","doc":"A common technique to [host multiple sites](https://hexdocs.pm/beacon/multiple-domains-hosting.html) (hosts) in the same application is matching the `:host` in the scope:\n\n```elixir\n# only matches requests from sitea.com\nscope host: \"sitea.com\" do\n  # ...\nend\n\n# only matches requests from siteb.com\nscope host: \"siteb.com\" do\n  # ...\nend\n```\n\nThe downside of having such flexibility is that it's also easy to create a configuration that is invalid or not optimized. Take for instance this configuration:\n\n```elixir\nscope host: \"sitea.com\" do\n  beacon_site \"/\", site: :site_a\nend\n\nscope host: \"siteb.com\" do\n  beacon_site \"/\", site: :site_b\nend\n```\n\nNote that `:site_b` will never match when deployed to `sitea.com` and `:site_a` will never match on `siteb.com`.\nYou can already tell that starting `:site_b` in the node that is hosting `sitea.com` is a complete waste of resources since it will never match any request,\nwhich is not a big problem when you have a couple of small sites, but that becomes a problem as your environment grows.\n\nTo avoid this problem, Beacon will selectively boot only the sites that are reachable in the current host, so in the example above,\nonly `:site_a` will boot in the node hosting `sitea.com` and only `:site_b` in the node hosting `siteb.com`.\n\nOr this other example:\n\n```elixir\nscope do\n  beacon_site \"/\", site: :main_site\n  beacon_site \"/campaigns\", site: :campaigns_site\nend\n```\n\nThe macro `beacon_site` creates a catch-all route `/*` so the second site will never be reached since `/campaigns`\nis a valid route for the first site.\n\nThose might look obvious but that's a common source of confusion, especially in long and more complex router files.\n\nFor these cases, whenever possible, Beacon will emit warnings during the boot process.","ref":"deployment-topologies.html#url-matching"},{"type":"extras","title":"Admin Sites Discovery - Deployment Topologies","doc":"[BeaconLiveAdmin](https://hex.pm/packages/beacon_live_admin) is designed to scan the all apps connected in the same cluster to find running sites\nand make them available in the admin interface as displayed below:\n\n```mermaid\ngraph TD\n    subgraph Cluster\n        subgraph ProjectA[Project A]\n            site_a[site_a]\n            project_a_site_b[site_b]\n        end\n        subgraph ProjectB[Project B]\n            site_b[site_b]\n            site_c[site_c]\n        end\n        subgraph Admin[Admin]\n            admin_site_a[site_a]\n            admin_site_b[site_b]\n            admin_siet_c[site_c]\n        end\n\n        site_a --> admin_site_a\n        site_b --> admin_site_b\n        project_a_site_b --> admin_site_b\n        site_c --> admin_siet_c\n    end\n```\n\nThe most important word here is \"running\" sites. Suppose `:site_a` in the node Project A is not running, either due to a route conflict, invalid configuration,\nor any other reason, then Admin won't find it and it won't be available in the admin interface:\n\n```mermaid\ngraph TD\n    subgraph Cluster\n        subgraph ProjectA[Project A]\n            project_a_site_b[site_b]\n        end\n        subgraph ProjectB[Project B]\n            site_b[site_b]\n            site_c[site_c]\n        end\n        subgraph Admin[Admin]\n            admin_site_b[site_b]\n            admin_siet_c[site_c]\n        end\n\n        site_b --> admin_site_b\n        project_a_site_b --> admin_site_b\n        site_c --> admin_siet_c\n    end\n```\n\nWith these constraints in mind, let's check some deployment strategies.","ref":"deployment-topologies.html#admin-sites-discovery"},{"type":"extras","title":"Strategies - Deployment Topologies","doc":"Below we'll describe some common deployment strategies but Beacon is not limited to the strategies below,\nyou can adapt to your needs.","ref":"deployment-topologies.html#strategies"},{"type":"extras","title":"1. Single application on the same host - Deployment Topologies","doc":"The most simple strategy is a single project with one or more sites and the admin interface in the same host.\n\n```elixir\n# endpoint\nconfig :my_app, MyAppWeb.Endpoint, url: [host: \"mysite.com\"]\n\n# router\nscope \"/\" do\n  beacon_live_admin \"/admin\"\n  beacon_site \"/campaigns\", site: :campaigns\n  beacon_site \"/\", site: :root\nend\n```\n\n```mermaid\nflowchart TD\n    subgraph Node1[\"Node1\"]\n        site_a[\"/campaigns, site: :campaigns\"]\n        site_b[\"/, site: :root\"]\n        subgraph Admin[\"/admin\"]\n          admin_site_a[\":campaigns\"]\n          admin_site_b[\":root\"]\n        end\n    end\n    subgraph Cluster[\"Cluster\"]\n        Node1\n        Admin\n    end\n\n    site_a --> admin_site_a\n    site_b --> admin_site_b\n    r1[\"mysite.com/campaigns/christmas\"] --> site_a\n    r2[\"mysite.com/contact\"] --> site_b\n    r3[\"mysite.com/admin\"] --> Admin\n```","ref":"deployment-topologies.html#1-single-application-on-the-same-host"},{"type":"extras","title":"2. Clustered single applications - Deployment Topologies","doc":"Same project as the previous strategy but with multiple nodes deployed in the same cluster.\n\nGives more capacity to serve more requests but still sharing the same Endpoint and same host.\n\n```mermaid\nflowchart TD\n    subgraph Node1[\"Node1\"]\n        n1_site_a[\"/campaigns, site: :campaigns\"]\n        n1_site_b[\"/, site: :root\"]\n        subgraph Node1Admin[\"/admin\"]\n          n1_admin_site_a[\":campaigns\"]\n          n1_admin_site_b[\":root\"]\n        end\n    end\n    subgraph Node2[\"Node2\"]\n        n2_site_a[\"/campaigns, site: :campaigns\"]\n        n2_site_b[\"/, site: :root\"]\n        subgraph Node2Admin[\"/admin\"]\n          n2_admin_site_a[\":campaigns\"]\n          n2_admin_site_b[\":root\"]\n        end\n    end\n    subgraph Cluster[\"Cluster\"]\n        Node1\n        Node2\n    end\n\n    n1_site_a --> n1_admin_site_a\n    n1_site_b --> n1_admin_site_b\n    n2_site_a --> n2_admin_site_a\n    n2_site_b --> n2_admin_site_b\n    r1[\"mysite.com/campaigns/christmas\"] --> n1_site_a\n    r1[\"mysite.com/campaigns/christmas\"] --> n2_site_a\n    r2[\"mysite.com/contact\"] --> n1_site_b\n    r2[\"mysite.com/contact\"] --> n2_site_b\n    r3[\"mysite.com/admin\"] --> Node1Admin\n    r3[\"mysite.com/admin\"] --> Node2Admin\n```","ref":"deployment-topologies.html#2-clustered-single-applications"},{"type":"extras","title":"3. Clustered applications with separated admin - Deployment Topologies","doc":"You can observe the previous strategy duplicates the admin interface in each node, which works fairly well when you have\nno more than a couple of sites, but that setup tends to become harder to manage and also become a waste of resources\nif you start booting more site and more nodes.\n\nSo an optimization is to move the Admin interface into its own project and node (a new Phoenix project),\nand keep the sites in their own projects.\n\nThat scenario also opens the possibility to deploy the admin interface behind a VPN to increase security.\n\nNote that in order for BeaconLiveAdmin to find all running sites, all the apps must be connected in the same cluster.\n\n```elixir\n# endpoint\nconfig :my_app, MyAppWeb.Endpoint, url: [host: \"admin.mysite.com\"]\n\n# router\nscope \"/\" do\n  beacon_live_admin \"/\"\nend\n```\n\n```mermaid\nflowchart TD\n    subgraph Node1[\"NodeApp1\"]\n        n1_site_a[\"/campaigns, site: :campaigns\"]\n        n1_site_b[\"/, site: :root\"]\n    end\n    subgraph Node2[\"NodeApp2\"]\n        n2_site_a[\"/campaigns, site: :campaigns\"]\n        n2_site_b[\"/, site: :root\"]\n    end\n    subgraph NodeAdmin[\"NodeAdmin\"]\n        admin_site_a[\":campaigns\"]\n        admin_site_b[\":root\"]\n    end\n    subgraph Cluster[\"Cluster\"]\n        Node1\n        Node2\n        NodeAdmin\n    end\n\n    n1_site_a --> admin_site_a\n    n1_site_b --> admin_site_b\n    n2_site_a --> admin_site_a\n    n2_site_b --> admin_site_b\n    r1[\"mysite.com/campaigns/christmas\"] --> n1_site_a\n    r1[\"mysite.com/campaigns/christmas\"] --> n2_site_a\n    r2[\"mysite.com/contact\"] --> n1_site_b\n    r2[\"mysite.com/contact\"] --> n2_site_b\n    r3[\"admin.mysite.com\"] --> NodeAdmin\n```\n\nA huge benefit of this topology is the flexibility to protect the Admin interface behind a VPN or scale it independently from the main applications.","ref":"deployment-topologies.html#3-clustered-applications-with-separated-admin"},{"type":"extras","title":"4. Multiple hosts in single project - Deployment Topologies","doc":"Still a single project but now it will serve each site on its own host (domain).\n\nThis scenario introduces a [Proxy Endpoint](https://hexdocs.pm/beacon/Beacon.ProxyEndpoint.html) to route requests to the appropriate Endpoint serving the site,\nthis configuration can be generated with `mix beacon.gen.site --site my_site --host mysite.com` - see [docs](https://hexdocs.pm/beacon/Mix.Tasks.Beacon.Gen.Site.html) for more info.\n\nIn this case we're still deploying just one application but serving multiple domains for each site:\n\n```mermaid\nflowchart TD\n    subgraph Node1[\"Node1\"]\n        proxy[\"ProxyEndpoint\"]\n        site_a[\"site: :my_site, endpoint: MySiteEndpoint\"]\n        site_b[\"site: :campaigns, endpoint: CampaignsEndpoint\"]\n        subgraph Admin[\"/admin\"]\n          admin_site_a[\":my_site\"]\n          admin_site_b[\":campaigns\"]\n        end\n    end\n    subgraph Cluster[\"Cluster\"]\n        Node1\n        Admin\n    end\n\n    site_a --> admin_site_a\n    site_b --> admin_site_b\n    r1[\"mysite.com\"] --> proxy\n    r2[\"campaigns.mysite.com\"] --> proxy\n    proxy --> site_a\n    proxy --> site_b\n    r3[\"mysite.com/admin\"] --> Admin\n```","ref":"deployment-topologies.html#4-multiple-hosts-in-single-project"},{"type":"extras","title":"Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"# Upgrading from pre v0.1.0 to v0.1.0-rc.0\n\nThis guide applies to instances of Beacon that started before the v0.1.0-rc.0 release,\nand need to migrate to latest version by migrating the database schema and data, and making changes to the codebase.\n\nFor this guide, we'll assume your Beacon dependency is set to [1d8d2fd636a40c107d47d0a87869930fdd0f82d0](https://github.com/BeaconCMS/beacon/tree/1d8d2fd636a40c107d47d0a87869930fdd0f82d0)\nfrom Oct 11, 2023, but it should apply to any revision.","ref":"v0-1-0-rc-0.html"},{"type":"extras","title":"Backup - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Before proceeding make sure you have a backup of your database and the current codebase.","ref":"v0-1-0-rc-0.html#backup"},{"type":"extras","title":"Migrate the database schema and data - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Before v0.1.0-rc.0 we used to keep the list of migrations in the `priv/repo/migrations` directory,\nbut eventually we merged all those files into a single migration for the v0.1.0-rc.0 release.\nSo before we can bump Beacon to the latest version, we need to execute the remaining migrations up to v0.1.0-rc.0\n\nYou can see that list of migrations before the merge in\nhttps://github.com/BeaconCMS/beacon/tree/c87777d8559378502188a19a696ac465e1618424/priv/repo/migrations\n\nSo let's update the Beacon dependency to that specific revision to execute all pending migrations.\n\nOpen the file `mix.exs` and update the Beacon dependency along with the other required dependencies:\n\n```elixir\n[\n  {:beacon, github: \"BeaconCMS/beacon\", ref: \"c87777d8559378502188a19a696ac465e1618424\", override: true},\n  {:beacon_live_admin, github: \"BeaconCMS/beacon_live_admin\", ref: \"9e31e0e307cf5ad44be50c689a472905c976bff3\"},\n  {:phoenix_html, \"~> 4.0\"},\n  {:phoenix_live_view, \"~> 0.20\"},\n]\n```\n\nNow execute `mix deps.update --all` to resolve all conflicts.\n\nBefore executing the migrations, we need to fix the breaking changes.","ref":"v0-1-0-rc-0.html#migrate-the-database-schema-and-data"},{"type":"extras","title":"Phoenix HTML - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Phoenix HTML 4.0 has introduced breaking changes, so make sure to follow [these instructions ](https://github.com/phoenixframework/phoenix_html/blob/main/CHANGELOG.md#v400-2023-12-19)\nto fix your application.","ref":"v0-1-0-rc-0.html#phoenix-html"},{"type":"extras","title":"beacon_api - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"The macro `beacon_api` has been removed. Open the file `router.ex` and remove any call to that macro.","ref":"v0-1-0-rc-0.html#beacon_api"},{"type":"extras","title":"First site config change - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Some changes are required in your site configuration for this intermediate Beacon revision.\n\n1. Add a `:router` key pointing to your app router module.\n2. Remove the `:data_source` key.\n\nEdit your site(s) configuration, usually in the file `application.ex` to make the changes. It will look like this:\n\n```elixir\n{Beacon,\n sites: [\n   [\n     site: :my_site,\n     endpoint: BeaconDemoWeb.Endpoint,\n     router: BeaconDemoWeb.Router\n     # ...\n   ]\n ]},\n```","ref":"v0-1-0-rc-0.html#first-site-config-change"},{"type":"extras","title":"Check for errors - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Execute `mix compile --all-warnings` and check for errors and warnings, other than the missing `Beacon.DataSource.Behaviour` module (we will cover this later).\n\nIt should compile without errors and with no extra warnings, otherwise, fix them before proceeding.","ref":"v0-1-0-rc-0.html#check-for-errors"},{"type":"extras","title":"Execute the migrations - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Execute `mix ecto.migrate` to run the remaining migrations.\n\nIf you have seeds files or some sort of automation, it will require changes to work with the updated schema.","ref":"v0-1-0-rc-0.html#execute-the-migrations"},{"type":"extras","title":"Bump Beacon to latest version - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Replace both `:beacon` and `:beacon_live_admin` to use the latest version, which at this moment is in the `0.1.x` series,\nand add `:igniter` which is now used to execute generators for Beacon.\n\n```elixir\n[\n  {:beacon, \"~> 0.1.0\"},\n  {:beacon_live_admin, \"~> 0.1.0\"},\n  {:igniter, \"~> 0.4\"},\n]\n```\n\nNow execute `mix deps.update --all` to resolve all conflicts.","ref":"v0-1-0-rc-0.html#bump-beacon-to-latest-version"},{"type":"extras","title":"Site config for v0.1 - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"To make it easier to manage sites configuration, we're now defining them in the `config/runtime.exs` file.\n\nOpen the file `application.ex`, usually where sites are defined, and replace with:\n\n```elixir\n{Beacon, [sites: [\n  Application.fetch_env!(:beacon, :my_site)\n]]}\n```\n\nReplace `:my_site` with your site name or add more sites if needed.\n\nNow open the file `config/runtime.exs` and copy the configuration you had in `application.ex` to this file.\n\nNote that we have added the new required key `:repo` in the site configuration, so make sure you have one as well.\n\nIt should look like this:\n\n```elixir\nconfig :beacon, :my_site,\n  site: :my_site,\n  repo: MyApp.Repo,\n  endpoint: MyAppWeb.Endpoint,\n  router: MyAppWeb.Router\n  # ... omitted other keys\n```\n\nThose are the minimum required keys.\n\nYou can add other sites as `config :beacon, :other_site, ...` if needed and reference them in `application.ex`.","ref":"v0-1-0-rc-0.html#site-config-for-v0-1"},{"type":"extras","title":"Remove Beacon.Repo - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"The module `Beacon.Repo` has been removed, so search for that module in your project and remove any mention of it,\nwhich usually is in the config files.","ref":"v0-1-0-rc-0.html#remove-beacon-repo"},{"type":"extras","title":"Update Esbuild and Tailwind - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Open the file `config/config.exs`, find the configs `:esbuild` and `:tailwind`, and change the `:version` to the latest release.\n\n```elixir\nconfig :esbuild,\n  version: \"0.24.0\",\n  # ...\n\nconfig :tailwind,\n  version: \"3.4.13\",\n  # ...\n```\n\nExecute the following to update the binaries:\n\n```shell\nmix esbuild.install\nmix tailwind.install --no-assets\n```","ref":"v0-1-0-rc-0.html#update-esbuild-and-tailwind"},{"type":"extras","title":"Beacon.Web.ErrorHTML - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"Error pages require a change in the endpoint configuration. Open the file `config/config.exs` and look for the `:render_errors` key in the endpoint configuration,\nand replace the `:html` format with `Beacon.Web.ErrorHTML`.\n\nIt should look like\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  render_errors: [\n    formats: [html: Beacon.Web.ErrorHTML, json: MyAppWeb.ErrorJSON],\n    layout: false\n  ],\n  # ...\n```","ref":"v0-1-0-rc-0.html#beacon-web-errorhtml"},{"type":"extras","title":"Migrate to latest schema - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"The schema should be up to date now, but to make sure let's run the latest migration.\n\nExecute `mix ecto.gen.migration create_beacon_tables` and edit the generated file to the following:\n\n```elixir\nuse Ecto.Migration\ndef up, do: Beacon.Migration.up()\ndef down, do: Beacon.Migration.down()\n```","ref":"v0-1-0-rc-0.html#migrate-to-latest-schema"},{"type":"extras","title":"Test the app - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"You should be able to execute the server now.\n\nMake sure everything is up to date:\n\n```shell\nmix setup\n```\n\nRun the server:\n\n```shell\nmix phx.server\n```","ref":"v0-1-0-rc-0.html#test-the-app"},{"type":"extras","title":"Beacon.DataSource.Behaviour - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"That module has been removed if favor of LiveData, which is also a way to define data for pages but at runtime\ninstead of compile time. There's no automatic migration for this data, so you'll need to access the Live Data page\nin the admin interface and recreate the data there _or_ automate it yourself by calling the [Content API functions](https://hexdocs.pm/beacon/Beacon.Content.html#functions-live-data).\n\nAccess the Live Data page in the admin interface, and first create the path you want to apply the assign and then create the assign itself.\n\nFor example if you used to have this live data in your data source module:\n\n```elixir\ndef live_data(:my_site, [\"blog\", \"categories\", category], params) do\n  %{blog_posts:  MyApp.Blog.list_blog_posts(category, params)}\nend\n```\n\nYou should create a new Live Data in admin for the path `/blog/categories/:category` and then create an Assign `blog_posts` with format `elixir` with the content:\n\n```elixir\nMyApp.Blog.list_blog_posts(category, params)\n```","ref":"v0-1-0-rc-0.html#beacon-datasource-behaviour"},{"type":"extras","title":"Publish pages - Upgrading from pre v0.1.0 to v0.1.0-rc.0","doc":"It may be necessary to republish the pages due to changes in the schema, so if a page is not loading execute the following in the console:\n\n```elixir\nsite = :my_site # replace with your site name\npages = Beacon.Content.list_published_pages(site, per_page: :infinity)\nBeacon.Content.publish_pages(pages)\n```","ref":"v0-1-0-rc-0.html#publish-pages"},{"type":"extras","title":"Upgrading to v0.1.0-rc.1","doc":"# Upgrading to v0.1.0-rc.1","ref":"v0-1-0-rc-1.html"},{"type":"extras","title":"Bump your deps - Upgrading to v0.1.0-rc.1","doc":"Update Beacon to the specific release candidate version:\n\n```elixir\n[\n  {:beacon, \"~> 0.1.0-rc.1\"}\n]\n```\n\nRemember to keep the option `override: true` if running BeaconLiveAdmin in the same project.","ref":"v0-1-0-rc-1.html#bump-your-deps"},{"type":"extras","title":"Update Beacon tables - Upgrading to v0.1.0-rc.1","doc":"```sh\nmix ecto.gen.migration update_beacon_v002\n```\n\nWithin the migration module:\n\n```elixir\nuse Ecto.Migration\ndef up, do: Beacon.Migration.up()\ndef down, do: Beacon.Migration.down()\n```","ref":"v0-1-0-rc-1.html#update-beacon-tables"},{"type":"extras","title":"Upgrading to v0.1.0-rc.2","doc":"# Upgrading to v0.1.0-rc.2","ref":"v0-1-0-rc-2.html"},{"type":"extras","title":"Bump your deps - Upgrading to v0.1.0-rc.2","doc":"Update Beacon to the specific release candidate version:\n\n```elixir\n[\n  {:beacon, \"~> 0.1.0-rc.2\"}\n]\n```\n\nRemember to keep the option `override: true` if running BeaconLiveAdmin in the same project.","ref":"v0-1-0-rc-2.html#bump-your-deps"},{"type":"extras","title":"Update Gettext - Upgrading to v0.1.0-rc.2","doc":"[Gettext](https://hex.pm/packages/gettext) has been updated to `~> 0.26` which requires a change,\nbut fortunately it's a small and beneficial one.\n\nIn your `mix.exs` file, update the `:gettext` dependency to at least `~> 0.26`:\n\n```elixir\n[\n  {:gettext, \"~> 0.26\"}\n]\n```\n\nThen change your application's `Gettext` module to use the new `Gettext.Backend` module:\n\n```elixir\ndefmodule MyAppWeb.Gettext do\n  use Gettext.Backend, otp_app: :my_app_web\nend\n```\n\n_Change the names accordingly, the only change needed here is adding the `.Backend` suffix._\n\nAnd now in every place you used to `import MyAppWeb.Gettext`, just replace with `use Gettext, backend: MyAppWeb.Gettext`\n\nFor more info see https://github.com/elixir-gettext/gettext/blob/main/CHANGELOG.md#v0260","ref":"v0-1-0-rc-2.html#update-gettext"},{"type":"extras","title":"Upgrading to v0.1.0","doc":"# Upgrading to v0.1.0","ref":"v0-1-0.html"},{"type":"extras","title":"Bump your deps - Upgrading to v0.1.0","doc":"Update Beacon to v0.1.x\n\n```elixir\n[\n  {:beacon, \"~> 0.1.0\"}\n]\n```\n\nNote that a more relaxed version like `~> 0.1`  or `>= 0.0.0` would install the version `1.1.0` which is not the Beacon CSM project,\nie: you have to install a version that is less than `1.0.0`.\n\nRemember to keep the option `override: true` if running BeaconLiveAdmin in the same project.","ref":"v0-1-0.html#bump-your-deps"},{"type":"extras","title":"Update sites config - Upgrading to v0.1.0","doc":"Replace the option `:skip_boot?` with `:mode` if you were using it (most sites don't need to set this option).","ref":"v0-1-0.html#update-sites-config"},{"type":"extras","title":"Update Elixir version - Upgrading to v0.1.0","doc":"Beacon v0.1.0 requires Elixir 1.14.0 or later, please update your tools if necessary.","ref":"v0-1-0.html#update-elixir-version"},{"type":"extras","title":"Update mdex version (only if using `:mdex` directly) - Upgrading to v0.1.0","doc":"Update `:mdex` to `~> 0.2` in your `mix.exs` file, and replace any calls to `MDEx.to_html` with `MDEx.to_html/1`.","ref":"v0-1-0.html#update-mdex-version-only-if-using-mdex-directly"},{"type":"extras","title":"Upgrading to v0.3.0","doc":"# Upgrading to v0.3.0","ref":"v0-3-0.html"},{"type":"extras","title":"Move `Beacon` after the endpoint in application.ex - Upgrading to v0.3.0","doc":"Beacon needs to access each site's endpoint info in order to boot the sites,\nso those endpoints must be started before the `Beacon` supervisor.\n\nOpen the file `application.ex`, find the Beacon tuple in the list of `children`,\nand move it to after the endpoint(s) declarations, usually at the end of that list:\n\n```elixir\n@impl true\ndef start(_type, _args) do\n  children = [\n    # ...\n    MyAppWeb.Endpoint,\n    {Beacon, [sites: [Application.fetch_env!(:beacon, :my_site)]]} # <- moved to after `MyAppWeb.Endpoint`\n  ]\n```","ref":"v0-3-0.html#move-beacon-after-the-endpoint-in-application-ex"},{"type":"extras","title":"Add `Beacon.Plug` to your Router - Upgrading to v0.3.0","doc":"In your application's `router.ex` file, first find where your sites are defined:\n\n```elixir\nscope \"/\", MyAppWeb do\n  pipe_through :browser\n  beacon_site \"/\", site: :my_site\nend\n```\n\nIn the above case we can see the site `:my_site` is inside a scope piped through `:browser`,\nso let's add a new `:beacon` pipeline with the `Beacon.Plug`:\n\n```elixir\npipeline :beacon do\n  plug Beacon.Plug\nend\n```\n\nAnd add that pipeline into the existing scope:\n\n```elixir\nscope \"/\", MyAppWeb do\n  pipe_through [:browser, :beacon] # <- add the pipeline here\n  beacon_site \"/\", site: :my_site\nend\n```\n\nNow the `Beacon.Plug` will ensure consistent rendering, particularly important when Page Variants are used.","ref":"v0-3-0.html#add-beacon-plug-to-your-router"},{"type":"extras","title":"Upgrading to v0.4.0","doc":"# Upgrading to v0.4.0","ref":"v0-4-0.html"},{"type":"extras","title":"Update Beacon tables - Upgrading to v0.4.0","doc":"Beacon v0.4 requires some new database tables for the latest features.  To create these tables,\nsimply run the generator for a new Ecto Migration module:\n\n```sh\nmix ecto.gen.migration update_beacon_v003\n```\n\nThen, within the generated module, call Beacon's migration helpers:\n\n```elixir\nuse Ecto.Migration\ndef up, do: Beacon.Migration.up()\ndef down, do: Beacon.Migration.down()\n```","ref":"v0-4-0.html#update-beacon-tables"},{"type":"extras","title":"Move `Beacon` before the endpoint in application.ex - Upgrading to v0.4.0","doc":"Open the file `application.ex`, find the Beacon tuple in the list of `children`,\nand move it to before the endpoint(s) declarations:\n\n```elixir\n@impl true\ndef start(_type, _args) do\n  children = [\n    # ...\n    {Beacon, [sites: [Application.fetch_env!(:beacon, :my_site)]]}, # <- moved to before `MyAppWeb.Endpoint`\n    MyAppWeb.Endpoint\n  ]\n```\n\nIn v0.3.0 the order was inverted which caused a regression bug that didn't allow application to recover from crashes.","ref":"v0-4-0.html#move-beacon-before-the-endpoint-in-application-ex"},{"type":"extras","title":"Generate your site Endpoint and the global Proxy Endpoint - Upgrading to v0.4.0","doc":"Beacon now requires an Endpoint per site and a Proxy endpoint to properly route requests to sites,\neven if your application has only one site and host.\n\nRun the [beacon.gen.site](https://hexdocs.pm/beacon/Mix.Tasks.Beacon.Gen.Site.html) task to create those files\nand update your project configuration:\n\n```sh\nmix beacon.gen.site --site my_site\n```\n\nReplace `my_site` with the actual name of your site, and execute that command for each site you have configured.\n\nNote that if you're using a custom host (domain) then execute this command instead:\n\n```sh\nmix beacon.gen.site --site my_site --host mysite.com\n```\n\nReplacing the values with the ones you actually use.\n\nIt's important to commit any changes _before_ executing that command and reviewing the proposed changes,\nespecially your site config in `runtime.exs` which may not match if you have a custom config.","ref":"v0-4-0.html#generate-your-site-endpoint-and-the-global-proxy-endpoint"},{"type":"extras","title":"Copy the `esbuild` binary in the release - Upgrading to v0.4.0","doc":"The binary `esbuild` is now required at runtime to bundle JS hooks,\nso it should be available in the release package.\n\nThis config only applies if you have deployed Beacon in a release,\notherwise you can skip this topic.\n\nFollow these instructions to update your project:\n\n1. Update `copy_beacon_files/1` function in `mix.exs`\n\nThat function is responsible for checking and copying the binary files \ninto the right place. See the [Deploy to Fly.io](https://hexdocs.pm/beacon/deploy-to-flyio.html)\nguides to copy the actual code and paste into your project `mix.exs` file.\n\n2. Update `Dockerfile`\n\nIn order to make sure both `tailwind` and `esbuild` are available you should include\nthese lines before `RUN mix release`:\n\n```dockerfile\nRUN mix tailwind.install --no-assets --if-missing\nRUN mix esbuild.install --if-missing\n```\n\nMore info in the [Deploy to Fly.io](https://hexdocs.pm/beacon/deploy-to-flyio.html) guide.","ref":"v0-4-0.html#copy-the-esbuild-binary-in-the-release"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}